/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 01/04/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.elastic.controller;

import eu.europa.empl.skpt.dto.ErrorDTO;
import eu.europa.empl.skpt.elastic.service.ElasticIndexService;
import eu.europa.empl.skpt.elastic.service.ElasticQueryService;
import eu.europa.empl.skpt.languages.LanguageCodeFormatService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Set;

/**
 * Elastic Controller
 *
 * @author ARHS Spikeseed
 */
@CrossOrigin
@Controller
public class ElasticController {

    private static final Logger LOGGER = LoggerFactory.getLogger(ElasticController.class);

    private static final String API_PREFIX = "moocs/recommendations/courses";
    private final ElasticQueryService elasticQueryService;
    private final ElasticIndexService elasticIndexService;
    private final LanguageCodeFormatService langFormatService;

    /**
     * Constructor
     * @param elasticQueryService   elastic query service
     * @param elasticIndexService   elastic index service
     * @param langFormatService     lang format service
     */
    public ElasticController(ElasticQueryService elasticQueryService,
                             ElasticIndexService elasticIndexService,
                             LanguageCodeFormatService langFormatService) {
        this.elasticQueryService = elasticQueryService;
        this.elasticIndexService = elasticIndexService;
        this.langFormatService = langFormatService;
    }

    /**
     * Returns the courses corresponding to "Improve language skills" in the Expectations section
     *
     * @param language        language(s)
     * @param motherTongue    mother tongue(s)
     * @return a ResponseEntity
     */
    @RequestMapping(
            value = API_PREFIX + "/language",
            method = RequestMethod.GET,
            produces = "application/json"
    )
    public ResponseEntity getLanguageCourses(@RequestParam(required = false, defaultValue = "") Set<String> language,
                                                @RequestParam(required = false, defaultValue = "") Set<String> motherTongue) {

        try {
            language = langFormatService.toConceptUris(language);
            motherTongue = langFormatService.toConceptUris(motherTongue);

            return new ResponseEntity(elasticQueryService.getLanguageCourses(language, motherTongue), HttpStatus.OK);
        } catch (Exception e) {
            LOGGER.error("Error when retrieving the language courses", e);
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorDTO(e.getMessage()));
        }
    }

    /**
     * Returns the courses corresponding to "Follow integration course" in the Expectations section
     *
     * @param hostCountryLanguage   hostCountryLanguage
     * @param understoodLanguages   mother tongue(s)
     * @return a ResponseEntity
     */
    @RequestMapping(
            value = API_PREFIX + "/integration",
            method = RequestMethod.GET,
            produces = "application/json"
    )
    public ResponseEntity getIntegrationCourses(@RequestParam(required = false, defaultValue = "") Set<String> hostCountryLanguage,
                                                   @RequestParam(required = false, defaultValue = "") Set<String> understoodLanguages) {

        try {
            hostCountryLanguage = langFormatService.toConceptUris(hostCountryLanguage);
            understoodLanguages = langFormatService.toConceptUris(understoodLanguages);

            return new ResponseEntity(
                    elasticQueryService.getIntegrationCourses(hostCountryLanguage, understoodLanguages),
                    HttpStatus.OK);
        } catch (Exception e) {
            LOGGER.error("Error when retrieving the language courses", e);
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorDTO(e.getMessage()));
        }
    }

    /**
     * Returns the courses corresponding to "Undertake education and / or training" in the Expectations section
     *
     * @param educationLevel        type(s) of education
     * @param studyField            field(s) of study
     * @param understoodLanguages   mother tongue(s)
     * @return a ResponseEntity
     */
    @RequestMapping(
            value = API_PREFIX + "/study",
            method = RequestMethod.GET,
            produces = "application/json"
    )
    public ResponseEntity getStudyCourses(@RequestParam(required = false, defaultValue = "") Set<String> educationLevel,
                                             @RequestParam(required = false, defaultValue = "") Set<String> studyField,
                                             @RequestParam(required = false, defaultValue = "") Set<String> understoodLanguages) {

        try {
            understoodLanguages = langFormatService.toConceptUris(understoodLanguages);

            return new ResponseEntity(
                    elasticQueryService.getStudyCourses(educationLevel, studyField, understoodLanguages),
                    HttpStatus.OK);
        } catch (Exception e) {
            LOGGER.error("Error when retrieving the language courses", e);
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorDTO(e.getMessage()));
        }
    }

    /**
     * Index all courses
     *
     * @return a ResponseEntity
     */
    @RequestMapping(
            value = "/elastic/indexAllCourses",
            method = RequestMethod.POST,
            produces = "application/json"
    )
    public ResponseEntity indexAllCourses() {
        elasticIndexService.indexAllCourses();
        return new ResponseEntity(HttpStatus.OK);
    }

}
