/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 01/04/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.elastic.repository;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Preconditions;
import io.vavr.control.Option;
import io.vavr.control.Try;
import org.elasticsearch.Version;
import org.elasticsearch.action.admin.cluster.node.info.NodesInfoResponse;
import org.elasticsearch.action.admin.indices.exists.indices.IndicesExistsRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.client.Requests;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.transport.TransportAddress;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.IndexNotFoundException;
import org.elasticsearch.transport.client.PreBuiltTransportClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Repository;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;

/**
 * Elastic Repository
 *
 * @author ARHS Spikeseed
 */
@Repository
public class ElasticRepository {

    public static final String CLIENT_WAS_NOT_SET_UP = "client was not set up";
    @Value("${elasticsearch.cluster-name}")
    private String clusterName;

    @Value("${elasticsearch.url}")
    private String url;

    @Value("${elasticsearch.javaApiPort}")
    private int port;

    private static final Logger LOGGER = LoggerFactory.getLogger(ElasticRepository.class);

    private final ObjectMapper objectMapper = new ObjectMapper();

    private TransportClient client;

    /**
     * Initialize the connection with ElasticSearch.
     */
    public void prepareClientNoAuth() {
        boolean alreadySetup = client != null;
        try {
            Settings settings = Settings.builder()
                    .put("cluster.name", clusterName).build();
            // Sonar ignores the next line because we do not want to close the connection.
            client = new PreBuiltTransportClient(settings)
                    .addTransportAddress(new TransportAddress(InetAddress.getByName(url), port)); //NOSONAR
            if (!alreadySetup) {
                LOGGER.info("Connection to Elasticsearch established.");
                compareServerClientVersions();
            }
        } catch (UnknownHostException e) {
            throw new IllegalStateException(e);
        }
    }

    /**
     * Checks if the server version matches the client version.
     */
    public void compareServerClientVersions() {
        Version server = getServerVersion();
        Version client = Version.CURRENT;

        if (server.major != client.major) {
            throw new IllegalStateException("Elastic Major versions were different, client = " + client + ", server = " + server);
        }
        if (server.minor != client.minor || server.revision != client.revision) {
            LOGGER.warn("Elastic Minor versions were different, client = {}, server = {}", client, server);
        }
    }

    /**
     * Returns the server version
     *
     * @return a Version
     */
    public Version getServerVersion() {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);
        NodesInfoResponse response = client.admin().cluster().prepareNodesInfo().all().get();

        if (response.getNodes().isEmpty()) {
            throw new IllegalStateException("Could not get nodes info from Elastic server");
        }

        return response.getNodes().get(0).getVersion();
    }

    /***
     * Check if the index already exists
     *
     * @param indexName   index name
     * @return a boolean
     */
    public boolean indexExist(String indexName) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);
        return client.admin().indices().exists(new IndicesExistsRequest(indexName)).actionGet().isExists();
    }

    /**
     * Prepare the index
     *
     * @param index       index name
     * @param shardCount  number of shard
     */
    public void prepareIndex(String index, int shardCount) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);
        try {
            client.admin().indices().prepareGetIndex().addIndices(index).get();
        } catch (IndexNotFoundException e) {
            client.admin()
                    .indices()
                    .prepareCreate(index)
                    .get();
        }
    }

    /**
     * Reset the index
     *
     * @param index       index name
     * @param shardCount  number of shard
     */
    public void resetIndex(String index, int shardCount) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);
        if (indexExist(index)) {
            client.admin().indices().delete(Requests.deleteIndexRequest(index)).actionGet();
        }
        prepareIndex(index, shardCount);
    }

    /**
     * This method indexes a JSON document into an index with the name, type and id in parameters.
     *
     * @param index   index name
     * @param type    index type
     * @param id      id of the indexe to be retrieved
     * @param json    document source to index
     * @return the response's id
     */
    public String insertJson(String index, String type, String id, String json) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);
        IndexResponse response = client.prepareIndex(index, type, id).setSource(json, XContentType.JSON).get();
        return response.getId();
    }

    /**
     * This method indexes a JSON document into an index with the name, type and id in parameters.
     *
     * @param index   index name
     * @param type    index type
     * @param id      id of the indexe to be retrieved
     * @param json    document source to index
     * @return the response's id
     */
    public String insertJson(String index, String type, String id, JsonNode json) {
        return insertJson(index, type, id, json.toString());
    }

    /**
     * This method indexes a JSON document into an index with the name, type and id in parameters if this document id not exists.
     *
     * @param index   index name
     * @param type    index type
     * @param id      id of the indexe to be retrieved
     * @param json    document source to index
     * @return the response's id
     */
    public String insertJsonIfNotExistById(String index, String type, String id, String json) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);
        GetResponse getResponse = client.prepareGet(index, type, id).get();
        if (getResponse.isExists()) {
            return getResponse.getId();
        }

        IndexResponse response = client.prepareIndex(index, type, id).setSource(json, XContentType.JSON).get();
        return response.getId();
    }

    /**
     * Insert pojo
     *
     * @param index   index name
     * @param type    index type
     * @param id      id of the indexe to be retrieved
     * @param pojo    pojo
     * @param <T>     <T>
     * @return the response's id
     */
    public <T> String insertPojo(String index, String type, String id, T pojo) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);

        byte[] bytes = Try.of(() -> objectMapper.writeValueAsBytes(pojo)).get();

        IndexResponse response = client.prepareIndex(index, type, id).setSource(bytes, XContentType.JSON).get();
        return response.getId();
    }

    /**
     * This method allows to get a document based on their index, type and id
     *
     * @param clazz   class related to the data contained in the index
     * @param index   index name
     * @param type    index type
     * @param id      id of the indexe to be retrieved
     * @param <T>     <T>
     * @return an Option<T>
     */
    public <T> Option<T> getById(Class<T> clazz, String index, String type, String id) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);

        GetResponse response = client.prepareGet(index, type, id).get();
        if (!response.isExists()) {
            return Option.none();
        }

        T result = Try.of(() -> objectMapper.readValue(response.getSourceAsBytes(), clazz)).get();

        return Option.of(result);
    }

    /**
     * This method allows to get a list of documents based on their index, type and id.
     *
     * @param clazz   class related to the data contained in the index
     * @param index   index name
     * @param type    index type
     * @param ids     ids of the indexes to be retrieved
     * @param <T>     <T>
     * @return a Stream<T>
     */
    public <T> Stream<T> getByIds(Class<T> clazz, String index, String type, Collection<String> ids) {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);

        if (ids.isEmpty()) {
            return Stream.empty();
        }

        List<byte[]> sources = new ArrayList<>();
        client.prepareMultiGet()
                .add(index, type, ids).get()
                .forEach(response -> {
                    if (response.getResponse() != null) {
                        sources.add(response.getResponse().getSourceAsBytes());
                    }
                });

        return sources.stream()
                .filter(Objects::nonNull)
                .map(bytes -> Try.of(() -> objectMapper.readValue(bytes, clazz)).get());
    }

    /**
     * Return the TransportClient which connects remotely to an Elasticsearch cluster using the transport module
     *
     * @return a TransportClient
     */
    public TransportClient getTransportClient() {
        Preconditions.checkNotNull(client, CLIENT_WAS_NOT_SET_UP);
        return client;
    }

}