package eu.europa.empl.skpt.elastic.service;

import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.function.Supplier;

import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.INTEG_LABOUR_URL;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.INTEG_NEW_CULT_ENVIRONMENT_URL;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.LANGUAGES_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.SUBTITLES_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.TOPIC_KEYWORD;

/**
 * Elastic Integration Courses Query Service
 * @author ARHS Spikeseed
 */
@Service
public class ElasticIntegrationCoursesQueryService {

    /**
     * Aggregates the results of the following methods :
     *  - queryIntegrationCoursesOfferedAndSubtitledInCountryLanguageAndInUnderstoodLanguage
     *  - queryIntegrationCoursesOfferedInCountryLanguageAndInUnderstoodLanguage
     *  - queryIntegrationCoursesSubtitledInCountryLanguageAndInUnderstoodLanguage
     *  - queryIntegrationCoursesOfferedAndSubtitledInUnderstoodLanguage
     *  - queryIntegrationCoursesOfferedInUnderstoodLanguage
     *  - queryIntegrationCoursesSubtitledInUnderstoodLanguage
     *  - queryIntegrationCoursesInAnyLanguage
     *
     * @param countryLanguage         country language
     * @param understoodLanguages     selected mother tongue(s) in the "Skill identification" section
     * @return the aggregation of the different queries
     */
    public List<Supplier<BoolQueryBuilder>> getIntegrationCoursesQuery(Set<String> countryLanguage, Set<String> understoodLanguages) {
         return Arrays.asList(
                queryIntegrationCoursesOfferedAndSubtitledInCountryLanguageAndInUnderstoodLanguage(countryLanguage, understoodLanguages),
                queryIntegrationCoursesOfferedInCountryLanguageAndInUnderstoodLanguage(countryLanguage, understoodLanguages),
                queryIntegrationCoursesSubtitledInCountryLanguageAndInUnderstoodLanguage(countryLanguage, understoodLanguages),
                queryIntegrationCoursesOfferedAndSubtitledInUnderstoodLanguage(understoodLanguages),
                queryIntegrationCoursesOfferedInUnderstoodLanguage(understoodLanguages),
                queryIntegrationCoursesSubtitledInUnderstoodLanguage(understoodLanguages),
                queryIntegrationCoursesInAnyLanguage()
        );
    }

    private Supplier<BoolQueryBuilder> queryIntegrationCoursesOfferedAndSubtitledInCountryLanguageAndInUnderstoodLanguage(
            Set<String> countryLanguage,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, countryLanguage))
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, countryLanguage))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, INTEG_LABOUR_URL,
                        INTEG_NEW_CULT_ENVIRONMENT_URL));
    }

    private Supplier<BoolQueryBuilder> queryIntegrationCoursesOfferedInCountryLanguageAndInUnderstoodLanguage(
            Set<String> countryLanguage,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, countryLanguage))
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, INTEG_LABOUR_URL,
                        INTEG_NEW_CULT_ENVIRONMENT_URL));
    }

    private Supplier<BoolQueryBuilder> queryIntegrationCoursesSubtitledInCountryLanguageAndInUnderstoodLanguage(
            Set<String> countryLanguage,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, countryLanguage))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, INTEG_LABOUR_URL,
                        INTEG_NEW_CULT_ENVIRONMENT_URL));
    }

    private Supplier<BoolQueryBuilder> queryIntegrationCoursesOfferedAndSubtitledInUnderstoodLanguage(Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, INTEG_LABOUR_URL,
                        INTEG_NEW_CULT_ENVIRONMENT_URL));
    }

    private Supplier<BoolQueryBuilder> queryIntegrationCoursesOfferedInUnderstoodLanguage(Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, INTEG_LABOUR_URL,
                        INTEG_NEW_CULT_ENVIRONMENT_URL));
    }

    private Supplier<BoolQueryBuilder> queryIntegrationCoursesSubtitledInUnderstoodLanguage(Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, INTEG_LABOUR_URL,
                        INTEG_NEW_CULT_ENVIRONMENT_URL));
    }

    private Supplier<BoolQueryBuilder> queryIntegrationCoursesInAnyLanguage() {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, INTEG_LABOUR_URL,
                        INTEG_NEW_CULT_ENVIRONMENT_URL));
    }
}
