package eu.europa.empl.skpt.elastic.service;

import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.function.Supplier;

import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.LANGUAGES_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.LANGUAGE_SKILLS_LANGUAGE_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.SUBTITLES_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.TOPIC_KEYWORD;

/**
 * Elastic Language Courses Query Service
 *
 * @author ARHS Spikeseed
 */
@Service
public class ElasticLanguageCoursesQueryService {
    private static final String LANG_TRAINING_URL = "http://data.europa.eu/esco/ConceptScheme/at-topics/lang-training";

    /**
     * Aggregates the results of the following methods :
     * - queryCoursesToImproveSelectedLanguageOfferedAndSubtitledInMotherTongue
     * - queryCoursesToImproveSelectedLanguageOfferedInMotherTongue
     * - queryCoursesToImproveSelectedLanguageSubtitledInMotherTongue
     * - queryCoursesToImproveSelectedLanguage
     *
     * @param language     selected language(s) in the "Improve language skills" section
     * @param motherTongue selected mother tongue(s) in the "Skill identification" section
     * @return the aggregation of the different queries
     */
    public List<Supplier<BoolQueryBuilder>> getLanguageCoursesQuery(Set<String> language, Set<String> motherTongue) {
        return Arrays.asList(
                queryCoursesToImproveSelectedLanguageOfferedAndSubtitledInMotherTongue(language, motherTongue),
                queryCoursesToImproveSelectedLanguageOfferedInMotherTongue(language, motherTongue),
                queryCoursesToImproveSelectedLanguageSubtitledInMotherTongue(language, motherTongue),
                queryCoursesToImproveSelectedLanguage(language)
        );
    }

    private Supplier<BoolQueryBuilder> queryCoursesToImproveSelectedLanguageOfferedAndSubtitledInMotherTongue(Set<String> language,
                                                                                                              Set<String> motherTongue) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGE_SKILLS_LANGUAGE_KEYWORD, language))
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, motherTongue))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, LANG_TRAINING_URL))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, motherTongue));
    }

    private Supplier<BoolQueryBuilder> queryCoursesToImproveSelectedLanguageOfferedInMotherTongue(Set<String> language,
                                                                                                  Set<String> motherTongue) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGE_SKILLS_LANGUAGE_KEYWORD, language))
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, motherTongue))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, LANG_TRAINING_URL));
    }

    private Supplier<BoolQueryBuilder> queryCoursesToImproveSelectedLanguageSubtitledInMotherTongue(Set<String> language,
                                                                                                    Set<String> motherTongue) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGE_SKILLS_LANGUAGE_KEYWORD, language))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, LANG_TRAINING_URL))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, motherTongue));
    }

    private Supplier<BoolQueryBuilder> queryCoursesToImproveSelectedLanguage(Set<String> language) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGE_SKILLS_LANGUAGE_KEYWORD, language))
                .must(QueryBuilders.termsQuery(TOPIC_KEYWORD, LANG_TRAINING_URL));
    }
}
