package eu.europa.empl.skpt.elastic.service;

import eu.europa.empl.skpt.elastic.repository.ElasticRepository;
import org.elasticsearch.action.search.SearchRequestBuilder;
import org.elasticsearch.client.transport.NoNodeAvailableException;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

/**
 * Elastic Query Result Service
 *
 * @author ARHS Spikeseed
 */
@Service
public class ElasticQueryResultService {
    private static final Logger LOGGER = LoggerFactory.getLogger(ElasticQueryResultService.class);

    private final ElasticRepository elasticRepository;

    @Value("${elasticsearch.courses.default-index}")
    private String index;

    /**
     * Constructor
     *
     * @param elasticRepository     ElasticRepository
     */
    public ElasticQueryResultService(ElasticRepository elasticRepository) {
        this.elasticRepository = elasticRepository;
    }

    /**
     * Initialization
     */
    @PostConstruct
    public void init() {
        try {
            elasticRepository.prepareClientNoAuth();
        } catch (NoNodeAvailableException | IllegalStateException e) {
            LOGGER.error("Could not connect to elastic", e);
            return;
        }
        elasticRepository.prepareIndex("courses", 4);
    }

    /**
     * return the results of the query
     *
     * @param queries   queries
     * @return          List<Map<String, Object>>
     */
    public List<Map<String, Object>> getResults(List<Supplier<BoolQueryBuilder>> queries) {
        return getResults(queries, queryAsSearchRequest(), QueryProcessStrategy.limit(5));
    }

    private <T> List<Map<String, Object>> getResults(List<T> queries,
                                                     Function<T, SearchRequestBuilder> asSearchRequestBuilder,
                                                     QueryProcessStrategy queryProcessStrategy) {
        List<Map<String, Object>> result = new ArrayList<>();

        for (T query : queries) {
            SearchRequestBuilder searchRequestBuilder = asSearchRequestBuilder.apply(query);
            List<Map<String, Object>> queryResult = Arrays.stream(searchRequestBuilder.execute().actionGet().getHits().getHits())
                    .map(SearchHit::getSourceAsMap)
                    .collect(Collectors.toList());

            queryProcessStrategy.addResults(result, queryResult);
            if (queryProcessStrategy.shouldStop(result)) {
                break;
            }
        }
        return result;
    }

    private <T extends QueryBuilder> Function<Supplier<T>, SearchRequestBuilder> queryAsSearchRequest() {
        return queryBuilderSupplier -> elasticRepository.getTransportClient()
                .prepareSearch(index)
                .setQuery(queryBuilderSupplier.get());
    }

    private abstract static class QueryProcessStrategy {

        public static QueryProcessStrategy limit(int limit) {
            return new LimitQuery(limit);
        }

        public abstract void addResults(List<Map<String, Object>> fullResult, List<Map<String, Object>> queryResult);

        public abstract boolean shouldStop(List<Map<String, Object>> fullResult);

    }

    private static class LimitQuery extends QueryProcessStrategy {

        private final int limit;

        public LimitQuery(int limit) {
            this.limit = limit;
        }

        @Override
        public void addResults(List<Map<String, Object>> fullResult, List<Map<String, Object>> queryResult) {
            int fetchSize = Math.min(queryResult.size(), limit - fullResult.size());
            fullResult.addAll(queryResult.subList(0, fetchSize));
        }

        @Override
        public boolean shouldStop(List<Map<String, Object>> fullResult) {
            return fullResult.size() >= 5;
        }
    }
}
