/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 01/04/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.elastic.service;

import org.elasticsearch.index.query.BoolQueryBuilder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Supplier;

/**
 * Elastic Query Service
 *
 * @author ARHS Spikeseed
 */
@Service
public class ElasticQueryService {

    private final ElasticLanguageCoursesQueryService languageCoursesQueryService;

    private final ElasticIntegrationCoursesQueryService integrationCoursesQueryService;

    private final ElasticStudyCoursesQueryService studyCoursesQueryService;

    private final ElasticQueryResultService elasticQueryResultService;

    /**
     * Constructor
     *
     * @param languageCoursesQueryService       languageCoursesQueryService
     * @param integrationCoursesQueryService    integrationCoursesQueryService
     * @param studyCoursesQueryService          studyCoursesQueryService
     * @param elasticQueryResultService         elasticQueryResultService
     */
    public ElasticQueryService(ElasticLanguageCoursesQueryService languageCoursesQueryService,
                               ElasticIntegrationCoursesQueryService integrationCoursesQueryService,
                               ElasticStudyCoursesQueryService studyCoursesQueryService,
                               ElasticQueryResultService elasticQueryResultService) {
        this.languageCoursesQueryService = languageCoursesQueryService;
        this.integrationCoursesQueryService = integrationCoursesQueryService;
        this.studyCoursesQueryService = studyCoursesQueryService;
        this.elasticQueryResultService = elasticQueryResultService;
    }

    /**
     * Return the language courses corresponding to the language and mother tongue in parameters
     *
     * @param language        selected language(s) in the "Improve language skills" section
     * @param motherTongue    selected mother tongue(s) in the "Skill identification" section
     * @return the language courses corresponding to the language and mother tongue in parameters
     */
    public List<Map<String, Object>> getLanguageCourses(Set<String> language, Set<String> motherTongue) {
        List<Supplier<BoolQueryBuilder>> queries = languageCoursesQueryService.getLanguageCoursesQuery(language, motherTongue);
        return elasticQueryResultService.getResults(queries);
    }


    /**
     * Return the integration courses corresponding to the country language and understood language in parameters
     *
     * @param countryLanguage         country language
     * @param understoodLanguages     selected mother tongue(s) in the "Skill identification" section
     * @return the integration courses corresponding to the country language and understood language in parameters
     */
    public List<Map<String, Object>> getIntegrationCourses(Set<String> countryLanguage, Set<String> understoodLanguages) {
        List<Supplier<BoolQueryBuilder>> queries = integrationCoursesQueryService
                .getIntegrationCoursesQuery(countryLanguage, understoodLanguages);
        return elasticQueryResultService.getResults(queries);
    }

    /***
     * Return the study courses corresponding to the education level, study field and understood language in parameters
     *
     * @param educationLevel            selected type(s) of education in the "Undertake education and / or training"
     * @param studyField                selected field(s) of study selected in the "Undertake education and / or training"
     * @param understoodLanguages       selected mother tongue(s) in the "Skill identification" section
     * @return the study courses corresponding to the education level, study field and understood language in parameters
     */
    public List<Map<String, Object>> getStudyCourses(Set<String> educationLevel, Set<String> studyField, Set<String> understoodLanguages) {
        List<Supplier<BoolQueryBuilder>> queries = studyCoursesQueryService
                .getStudyCoursesQuery(educationLevel, studyField, understoodLanguages);
        return elasticQueryResultService.getResults(queries);
    }
}