package eu.europa.empl.skpt.elastic.service;

import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.function.Supplier;

import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.EDUCATION_LEVEL_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.ISCEDS_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.LANGUAGES_KEYWORD;
import static eu.europa.empl.skpt.elastic.service.ElasticKeywords.SUBTITLES_KEYWORD;

/**
 * Elastic Study Courses Query Service
 *
 * @author ARHS Spikeseed
 */
@Service
public class ElasticStudyCoursesQueryService {

    /***
     * Aggregates the results of the following methods :
     *  - queryStudyCoursesForEducationLevelAndStudyFieldOfferedAndSubtitledInUnderstoodLanguage
     *  - queryStudyCoursesForEducationLevelAndStudyFieldOfferedInUnderstoodLanguage
     *  - queryStudyCoursesForEducationLevelAndStudyFieldSubtitledInUnderstoodLanguage
     *  - queryStudyCoursesForStudyFieldOfferedAndSubtitledInUnderstoodLanguage
     *  - queryStudyCoursesForStudyFieldOfferedInUnderstoodLanguage
     *  - queryStudyCoursesForStudyFieldSubtitledInUnderstoodLanguage
     *  - queryStudyCoursesForEducationLevelOfferedAndSubtitledInUnderstoodLanguage
     *  - queryStudyCoursesForEducationLevelOfferedInUnderstoodLanguage
     *  - queryStudyCoursesForEducationLevelSubtitledInUnderstoodLanguage
     *  - queryStudyCoursesForStudyFieldAndEducationLevel
     *  - queryStudyCoursesForStudyField
     *  - queryStudyCoursesForEducationLevel
     *
     * @param educationLevel            selected type(s) of education in the "Undertake education and / or training"
     * @param studyField                selected field(s) of study selected in the "Undertake education and / or training"
     * @param understoodLanguages       selected mother tongue(s) in the "Skill identification" section
     * @return the aggregation of the different queries
     */
    public List<Supplier<BoolQueryBuilder>> getStudyCoursesQuery(Set<String> educationLevel,
                                                                 Set<String> studyField,
                                                                 Set<String> understoodLanguages) {
        return Arrays.asList(
                queryStudyCoursesForEducationLevelAndStudyFieldOfferedAndSubtitledInUnderstoodLanguage(
                        educationLevel, studyField, understoodLanguages
                ),
                queryStudyCoursesForEducationLevelAndStudyFieldOfferedInUnderstoodLanguage(
                        educationLevel, studyField, understoodLanguages
                ),
                queryStudyCoursesForEducationLevelAndStudyFieldSubtitledInUnderstoodLanguage(
                        educationLevel, studyField, understoodLanguages
                ),
                queryStudyCoursesForStudyFieldOfferedAndSubtitledInUnderstoodLanguage(studyField, understoodLanguages),
                queryStudyCoursesForStudyFieldOfferedInUnderstoodLanguage(studyField, understoodLanguages),
                queryStudyCoursesForStudyFieldSubtitledInUnderstoodLanguage(studyField, understoodLanguages),
                queryStudyCoursesForEducationLevelOfferedAndSubtitledInUnderstoodLanguage(educationLevel, understoodLanguages),
                queryStudyCoursesForEducationLevelOfferedInUnderstoodLanguage(educationLevel, understoodLanguages),
                queryStudyCoursesForEducationLevelSubtitledInUnderstoodLanguage(educationLevel, understoodLanguages),
                queryStudyCoursesForStudyFieldAndEducationLevel(studyField, educationLevel),
                queryStudyCoursesForStudyField(studyField),
                queryStudyCoursesForEducationLevel(educationLevel)
        );
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForEducationLevelAndStudyFieldOfferedAndSubtitledInUnderstoodLanguage(
            Set<String> educationLevel,
            Set<String> studyField,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel))
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForEducationLevelAndStudyFieldOfferedInUnderstoodLanguage(
            Set<String> educationLevel,
            Set<String> studyField,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel))
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForEducationLevelAndStudyFieldSubtitledInUnderstoodLanguage(
            Set<String> educationLevel,
            Set<String> studyField,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel))
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForStudyFieldOfferedAndSubtitledInUnderstoodLanguage(
            Set<String> studyField,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForStudyFieldOfferedInUnderstoodLanguage(
            Set<String> studyField,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForStudyFieldSubtitledInUnderstoodLanguage(
            Set<String> studyField,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForEducationLevelOfferedAndSubtitledInUnderstoodLanguage(
            Set<String> educationLevel,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForEducationLevelOfferedInUnderstoodLanguage(
            Set<String> educationLevel,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(LANGUAGES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForEducationLevelSubtitledInUnderstoodLanguage(
            Set<String> educationLevel,
            Set<String> understoodLanguages) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(SUBTITLES_KEYWORD, understoodLanguages))
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel));
    }


    private Supplier<BoolQueryBuilder> queryStudyCoursesForStudyFieldAndEducationLevel(
            Set<String> educationLevel,
            Set<String> studyField) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel))
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForEducationLevel(Set<String> educationLevel) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(EDUCATION_LEVEL_KEYWORD, educationLevel));
    }

    private Supplier<BoolQueryBuilder> queryStudyCoursesForStudyField(Set<String> studyField) {
        return () -> QueryBuilders.boolQuery()
                .must(QueryBuilders.termsQuery(ISCEDS_KEYWORD, studyField));
    }
}
