/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 01/04/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.languages;

import com.google.common.base.Preconditions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Language Code Format Service
 *
 * @author ARHS Spikeseed
 */
@Service
public class LanguageCodeFormatService {

    private static final Logger LOGGER = LoggerFactory.getLogger(LanguageCodeFormatService.class);

    private final Iso6391CodeToConceptService iso6391CodeToUriService;

    /**
     * Constructor
     *
     * @param iso6391CodeToUriService     iso6391CodeToUriService
     */
    public LanguageCodeFormatService(Iso6391CodeToConceptService iso6391CodeToUriService) {
        this.iso6391CodeToUriService = iso6391CodeToUriService;
    }

    /**
     * This method retrieves the URI associated with a specific language code.
     *
     * @param code    code for which the URI must be retrieved
     * @return an Optional String
     */
    public Optional<String> toConceptUri(String code) {
        Preconditions.checkNotNull(code);

        if (code.isEmpty()) {
            return Optional.empty();
        }
        if (isUri(code)) {
            return Optional.of(code);
        }

        if (isIso6391Code(code)) {
            Optional<String> optCode = iso6391CodeToUriService.getConceptUri(code);
            if (!optCode.isPresent()) {
                LOGGER.warn("Could not convert language code '{}' to uri", code);
            }
            return optCode;
        }

        LOGGER.error("Could not convert language code '" + code + "' to uri");
        throw new RuntimeException("Could not convert language code '" + code + "' to uri");
    }

    /**
     * For each code in the collection, the method toConceptUri will be called.
     *
     * @param codes   codes for which the URIs must be retrieved
     * @return the URI corresponding to each code
     */
    public Set<String> toConceptUris(Collection<String> codes) {
        return codes.stream().map(this::toConceptUri).filter(Optional::isPresent).map(Optional::get).collect(Collectors.toSet());
    }

    private boolean isUri(String code) {
        return code.startsWith("http://");
    }

    private boolean isIso6391Code(String code) {
        return code.length() == 2;
    }


}
