/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 01/04/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.mail;

import eu.europa.empl.skpt.Course;
import eu.europa.empl.skpt.dto.ErrorDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.mail.MailException;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import eu.europa.empl.skpt.elastic.repository.ElasticRepository;

import java.util.List;
import java.util.stream.Collectors;

/**
 * Mail Sender Controller
 *
 * @author ARHS Spikeseed
 */
@CrossOrigin
@Controller
public class MailSenderController {

    private final MailSenderService mailSenderService;

    private final ElasticRepository elasticRepository;

    private static final Logger LOGGER = LoggerFactory.getLogger(MailSenderController.class);

    /**
     * Constructor
     *
     * @param mailSenderService   mailSenderService
     * @param elasticRepository   elasticRepository
     */
    public MailSenderController(MailSenderService mailSenderService, ElasticRepository elasticRepository) {
        this.mailSenderService = mailSenderService;
        this.elasticRepository = elasticRepository;
    }

    /**
     * This method is called when the user enters an email address and click the envelope.
     * It calls the getByIds method of ElasticRespository in order to retrive the courses.
     * It then calls the sendCourses method of MailSenderService.
     *
     * @param address     encoded email(s) address(es)
     * @param courseId    selected course(s) id(s)
     * @return a ResponseEntity
     */
    @RequestMapping(
            value = "/courses/mail/send",
            method = RequestMethod.POST,
            produces = "application/json"
    )
    public ResponseEntity sendMail(@RequestParam List<String> address, @RequestParam List<String> courseId) {

        if (courseId.isEmpty()) {
            String errorMessage = "No courses provided for email";
            LOGGER.error(errorMessage);
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorDTO(errorMessage));
        }

        if (address.isEmpty()) {
            String errorMessage = "No addresses provided for email";
            LOGGER.error(errorMessage);
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorDTO(errorMessage));
        }

        try {
            List<Course> courses = this.elasticRepository.getByIds(Course.class, "courses", "course", courseId)
                    .collect(Collectors.toList());

            mailSenderService.sendCourses(address, courses);

            return ResponseEntity.ok().build();
        } catch (MailException e) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorDTO(e.getMessage()));
        }
    }
}
