/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 01/04/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.elastic.controller;

import org.junit.Before;
import org.junit.Test;

import java.util.Arrays;
import java.util.List;

import static io.restassured.RestAssured.given;
import static org.hamcrest.Matchers.*;

/**
 * Elastic Controller Test
 *
 * @author ARHS Spikeseed
 */
public class ElasticControllerTest {

    private String url;

    @Before
    public void setup() {
        this.url = "http://skpt-dev.divpqaubuk.eu-west-1.elasticbeanstalk.com/moocs/recommendations/courses";
    }

    @Test
    public void searchByLanguage() {
        List<String> languages = Arrays.asList("en");
        String motherTongue = "";

        given()
                .param("language", languages)
                .param("motherTongue", motherTongue).
        when()
                .get(this.url + "/language").
        then()
                .statusCode(200)
                .contentType("application/json")
                .body("size()", equalTo(5))
                .body("", everyItem(anyOf(
                        hasEntry(is("languages"), hasItem(is("http://publications.europa.eu/resource/authority/language/ENG"))),
                        hasEntry(is("description"), containsString("English")),
                        hasEntry(is("name"), containsString("English"))
                )));
    }

    @Test
    public void searchByLanguageErrorLanguageCode() {
        List<String> languages = Arrays.asList("errorCode");
        String motherTongue = "";

        given()
                .param("language", languages)
                .param("motherTongue", motherTongue).
        when()
                .get(this.url + "/language").
        then()
                .statusCode(400)
                .contentType("application/json")
                .body("message", equalTo("Could not convert language code \'" + languages.get(0) + "\' to uri"));
    }

    @Test
    public void searchByLanguageErrorMotherTongueCode() {
        List<String> languages = Arrays.asList();
        String motherTongue = "errorCode";

        given()
                .param("language", languages)
                .param("motherTongue", motherTongue).
        when()
                .get(this.url + "/language").
        then()
                .statusCode(400)
                .contentType("application/json")
                .body("message", equalTo("Could not convert language code \'" + motherTongue + "\' to uri"));
    }

    @Test
    public void searchByIntegration() {
        String[] languages = {
                "http://publications.europa.eu/resource/authority/language/ENG",
                "http://publications.europa.eu/resource/authority/language/FRA"
        };

        List<String> hostCountryLanguage = Arrays.asList("en");
        List<String> understoodLanguages = Arrays.asList("fr");

        given()
                .param("hostCountryLanguage", hostCountryLanguage)
                .param("understoodLanguages", understoodLanguages).
        when()
                .get(this.url + "/integration").
        then()
                .statusCode(200)
                .contentType("application/json")
                .body("size()", equalTo(5))
                .body("", everyItem(anyOf(
                        hasEntry(is("languages"), hasItem(isIn(languages))),
                        hasEntry(is("subtitles"), hasItem(isIn(languages)))
                )));
    }

    @Test
    public void searchByIntegrationErrorHostCountyLanguageCode() {
        List<String> hostCountryLanguage = Arrays.asList("errorCode");
        List<String> understoodLanguages = Arrays.asList();

        given()
                .param("hostCountryLanguage", hostCountryLanguage)
                .param("understoodLanguages", understoodLanguages).
        when()
                .get(this.url + "/integration").
        then()
                .statusCode(400)
                .contentType("application/json")
                .body("message", equalTo("Could not convert language code \'" + hostCountryLanguage.get(0) + "\' to uri"));
    }

    @Test
    public void searchByIntegrationErrorUnderstoodLanguagesCode() {
        List<String> hostCountryLanguage = Arrays.asList();
        List<String> understoodLanguages = Arrays.asList("errorCode");

        given()
                .param("hostCountryLanguage", hostCountryLanguage)
                .param("understoodLanguages", understoodLanguages).
        when()
                .get(this.url + "/integration").
        then()
                .statusCode(400)
                .contentType("application/json")
                .body("message", equalTo("Could not convert language code \'" + understoodLanguages.get(0) + "\' to uri"));
    }

    @Test
    public void searchByUndertakeEducation() {
        List<String> educationLevel = Arrays.asList("http://data.europa.eu/esco/ConceptScheme/at-ed-level/isced-1");
        List<String> studyFields = Arrays.asList(
                "http://data.europa.eu/esco/isced-f/01",
                "http://data.europa.eu/esco/isced-f/011",
                "http://data.europa.eu/esco/isced-f/0110",
                "http://data.europa.eu/esco/isced-f/0111",
                "http://data.europa.eu/esco/isced-f/0112",
                "http://data.europa.eu/esco/isced-f/0113",
                "http://data.europa.eu/esco/isced-f/0114",
                "http://data.europa.eu/esco/isced-f/0119",
                "http://data.europa.eu/esco/isced-f/018",
                "http://data.europa.eu/esco/isced-f/0188"
        );
        List<String> understoodLanguages = Arrays.asList("en");

        given()
                .param("educationLevel", educationLevel)
                .param("studyField", studyFields)
                .param("understoodLanguages", understoodLanguages).
        when()
                .get(this.url + "/study").
        then()
                .statusCode(200)
                .contentType("application/json")
                .body("size()", equalTo(5))
                .body("", everyItem(anyOf(
                        hasEntry(is("educationLevel"), isIn(educationLevel)),
                        hasEntry(is("isceds"), hasItem(isIn(studyFields))),
                        hasEntry(is("languages"), hasItem(is("http://publications.europa.eu/resource/authority/language/ENG"))),
                        hasEntry(is("subtitles"), hasItem(is("http://publications.europa.eu/resource/authority/language/ENG")))
                )));
    }

    @Test
    public void searchByUndertakeEducationErrorUnderstoodLanguages() {
        List<String> educationLevel = Arrays.asList();
        List<String> studyField = Arrays.asList();
        List<String> understoodLanguages = Arrays.asList("errorCode");

        given()
                .param("educationLevel", educationLevel)
                .param("studyField", studyField)
                .param("understoodLanguages", understoodLanguages).
        when()
                .get(this.url + "/study").
        then()
                .statusCode(400)
                .contentType("application/json")
                .body("message", equalTo("Could not convert language code \'" + understoodLanguages.get(0) + "\' to uri"));
    }

}
