/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 12/4/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.api;

import eu.europa.empl.skpt.api.dto.AttachmentDto;
import eu.europa.empl.skpt.api.dto.AttachmentToDtoConverter;
import eu.europa.empl.skpt.api.exception.AttachmentEncodingException;
import eu.europa.empl.skpt.api.exception.AttachmentIOException;
import eu.europa.empl.skpt.api.exception.BadRequestException;
import eu.europa.empl.skpt.api.exception.InternalServerErrorException;
import eu.europa.empl.skpt.api.exception.NotFoundException;
import eu.europa.empl.skpt.data.domain.attachment.Attachment;
import eu.europa.empl.skpt.service.attachment.AttachmentSaveResult;
import eu.europa.empl.skpt.service.attachment.IAttachmentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

import static eu.europa.empl.skpt.common.Error.ATTACHMENT_ENCODING_EXCEPTION;
import static eu.europa.empl.skpt.common.Error.ATTACHMENT_IO_EXCEPTION;

// CLOVER:OFF

/**
 * Controller handling requests related to attachments
 *
 * @author ARHS Spikeseed
 */
@RestController
public class AttachmentController {

    private final IAttachmentService attachmentService;
    private final AttachmentToDtoConverter attachmentToDtoConverter;

    /**
     * Constructor
     *
     * @param attachmentService        attachment service
     * @param attachmentToDtoConverter attachment converter
     */
    @Autowired
    public AttachmentController(IAttachmentService attachmentService, AttachmentToDtoConverter attachmentToDtoConverter) {
        this.attachmentService = attachmentService;
        this.attachmentToDtoConverter = attachmentToDtoConverter;
    }

    /**
     * Retrieve a {@link Attachment} identified by the given identifier
     *
     * @param id the {@link Attachment} identifier.
     * @return The {@link Attachment} identified by the given identifier.
     */
    @RequestMapping("/api/attachment/{id}")
    public ResponseEntity<InputStreamResource> getById(@PathVariable String id) {

        try {
            Attachment attachment = this.attachmentService.findByIdentifier(id)
                    .orElseThrow(() -> new NotFoundException(String.format("Attachment with id [%s] not found", id)));

            HttpHeaders headers = new HttpHeaders();
            headers.set(HttpHeaders.CONTENT_DISPOSITION,
                    "attachment; filename*=UTF-8''" + URLEncoder.encode(attachment.getName(), StandardCharsets.UTF_8.name()) + "");
            headers.setContentLength(attachment.getSize());
            headers.setContentType(MediaType.parseMediaType(attachment.getMediaType()));

            InputStreamResource inputStreamResource = new InputStreamResource(this.attachmentService.read(attachment.getIdentifier()));

            return new ResponseEntity<>(inputStreamResource, headers, HttpStatus.OK);
        } catch (UnsupportedEncodingException e) {
            throw new AttachmentEncodingException("Error during the URL encoding of the attachment", ATTACHMENT_ENCODING_EXCEPTION);
        }
    }

    /**
     * Upload an attachment.
     *
     * @param file the file to save
     * @return an attachment
     */
    @RequestMapping(value = "/api/attachment", method = RequestMethod.POST, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public AttachmentDto upload(@RequestParam("file") MultipartFile file) {
        try (InputStream is = file.getInputStream()) {

            AttachmentSaveResult result = this.attachmentService
                    .save(is, file.getOriginalFilename(), file.getContentType(), file.getSize());

            if (result.hasErrors()) {
                throw new BadRequestException("Attachment submitted is invalid.", result.getErrors());
            }

            return result.getAttachment().map(this.attachmentToDtoConverter::convert)
                    .orElseThrow(() -> new InternalServerErrorException("Attachment was saved without errors but not returned"));
        } catch (IOException e) {
            throw new AttachmentIOException("Error during the upload of the attachment", ATTACHMENT_IO_EXCEPTION);
        }
    }
}
