/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.api;

import eu.europa.empl.skpt.api.dto.UnproxyCountryConverter;
import eu.europa.empl.skpt.api.exception.NotFoundException;
import eu.europa.empl.skpt.data.domain.country.Country;
import eu.europa.empl.skpt.service.country.ICountryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

// CLOVER:OFF

/**
 * Controller for the {@link Country} entity.
 *
 * @author ARHS Spikeseed
 */
@RestController
public class CountryController {

    private final ICountryService countryService;
    private final UnproxyCountryConverter unproxyCountryConverter;

    /**
     * Constructor.
     *
     * @param countryService          the {@link Country} service
     * @param unproxyCountryConverter unproxy country controller
     */
    @Autowired
    public CountryController(ICountryService countryService, UnproxyCountryConverter unproxyCountryConverter) {
        this.countryService = countryService;
        this.unproxyCountryConverter = unproxyCountryConverter;
    }

    /**
     * Retrieve a collection of {@link Country}.
     *
     * @param resultLanguages the languages to search on
     * @param page            zero-based page index.
     * @param pageSize        the size of the page to be returned.
     * @param sortField       the properties to sort by, must not be {@literal null} or empty.
     * @param sortDirection   the direction of the {@link Sort} to be specified, can be {@literal null}.
     * @param q               the query string search parameter.
     * @return The ordered and paginated collection of countries.
     */
    @Cacheable(value = "api-countries")
    @RequestMapping("/api/countries")
    public Iterable<Country> getAll(@RequestParam(required = false, defaultValue = "") String[] resultLanguages,
                                    @RequestParam(required = false, defaultValue = "0") int page,
                                    @RequestParam(required = false, defaultValue = "10") int pageSize,
                                    @RequestParam(required = false, defaultValue = "id") String sortField,
                                    @RequestParam(required = false, defaultValue = "ASC") Sort.Direction sortDirection,
                                    @RequestParam(required = false) String q) {
        return this.countryService.search(q, resultLanguages, PageRequest.of(page, pageSize, sortDirection, sortField))
                .map(unproxyCountryConverter::convert);
    }

    /**
     * Retrieve a country identified by the given alpha-3 code.
     *
     * @param code      the alpha-3 code country code
     * @param languages the languages returned.
     * @return The country identified by the given alpha-3 code.
     */
    @Cacheable(value = "api-country")
    @RequestMapping("/api/country/{code}")
    public Country getCountryByAlpha3Code(@PathVariable String code,
            @RequestParam(required = false, defaultValue = "") String[] languages) {
        return this.countryService.findByAlphaCode3(code, languages).map(unproxyCountryConverter::convert).orElseThrow(() ->
                new NotFoundException(String.format("Country with code [%s] not found", code)));
    }
}
