/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.api;

import eu.europa.empl.skpt.api.dto.UnproxyEscoConverter;
import eu.europa.empl.skpt.api.exception.NotFoundException;
import eu.europa.empl.skpt.data.domain.esco.EscoClassification;
import eu.europa.empl.skpt.data.domain.esco.EscoLocalConcept;
import eu.europa.empl.skpt.service.esco.IEscoLocalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

// CLOVER:OFF

/**
 * Controller for the {@link EscoLocalConcept} entity.
 *
 * @author ARHS Spikeseed
 */
@RestController
public class EscoController {

    private final IEscoLocalService escoService;
    private final UnproxyEscoConverter unproxyEscoConverter;

    /**
     * Contructor.
     *
     * @param escoService          the {@link EscoLocalConcept} service.
     * @param unproxyEscoConverter unproxy esco converter
     */
    @Autowired
    public EscoController(IEscoLocalService escoService, UnproxyEscoConverter unproxyEscoConverter) {
        this.escoService = escoService;
        this.unproxyEscoConverter = unproxyEscoConverter;
    }

    /**
     * Retrieve a {@link EscoLocalConcept} identified by the given identifier
     *
     * @param id        the {@link EscoLocalConcept} identifier.
     * @param languages the languages returned.
     * @return The {@link EscoLocalConcept} identified by the given identifier.
     */
    @Cacheable(value = "api-esco")
    @RequestMapping("/api/esco/{id}")
    public EscoLocalConcept getById(@PathVariable Long id,
            @RequestParam(required = false, defaultValue = "") String[] languages) {
        return this.escoService.findById(id, languages)
                .map(unproxyEscoConverter::convert)
                .orElseThrow(() -> new NotFoundException(String.format("ESCO with id [%s] not found", id)));
    }

    /**
     * Retrieve a collection of {@link EscoLocalConcept} of the given {@link EscoClassification}.
     *
     * @param resultLanguages the languages to search on
     * @param classification  the ESCO classification.
     * @param page            zero-based page index.
     * @param pageSize        the size of the page to be returned.
     * @param sortField       the properties to sort by, must not be {@literal null} or empty.
     * @param sortDirection   the direction of the {@link Sort} to be specified, can be {@literal null}.
     * @param q               the query string search parameter.
     * @return The ordered and paginated collection of {@link EscoLocalConcept}.
     */
    @Cacheable(value = "api-escos")
    @RequestMapping("/api/escos/{classification}")
    public Iterable<EscoLocalConcept> search(
            @PathVariable EscoClassification classification,
            @RequestParam(required = false, defaultValue = "") String[] resultLanguages,
            @RequestParam(required = false, defaultValue = "0") int page,
            @RequestParam(required = false, defaultValue = "10") int pageSize,
            @RequestParam(required = false, defaultValue = "id") String sortField,
            @RequestParam(required = false, defaultValue = "ASC") Sort.Direction sortDirection,
            @RequestParam(required = false) String q) {
        return this.escoService.search(classification, q, resultLanguages, PageRequest.of(page, pageSize,
                sortDirection, sortField)).map(unproxyEscoConverter::convert);
    }

}
