/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.api;

import eu.europa.empl.skpt.api.dto.UnproxyIscedFoetConverter;
import eu.europa.empl.skpt.api.exception.NotFoundException;
import eu.europa.empl.skpt.data.domain.isced.IscedFoet;
import eu.europa.empl.skpt.service.isced.IIscedFoetService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

// CLOVER:OFF

/**
 * Controller for the {@link IscedFoet} entity.
 *
 * @author ARHS Spikeseed
 */
@RestController
public class IscedFoetController {

    private final IIscedFoetService iscedFoetService;
    private final UnproxyIscedFoetConverter unproxyIscedFoetConverter;

    /**
     * Contructor.
     *
     * @param iscedFoetService          the {@link IscedFoet} service.
     * @param unproxyIscedFoetConverter unproxy isced foet converter
     */
    @Autowired
    public IscedFoetController(IIscedFoetService iscedFoetService, UnproxyIscedFoetConverter unproxyIscedFoetConverter) {
        this.iscedFoetService = iscedFoetService;
        this.unproxyIscedFoetConverter = unproxyIscedFoetConverter;
    }

    /**
     * Retrieve a {@link IscedFoet} identified by the given identifier.
     *
     * @param id        the {@link IscedFoet} identifier.
     * @param languages the languages returned.
     * @return The {@link IscedFoet} identified by the given identifier.
     */
    @Cacheable(value = "api-isced")
    @RequestMapping("/api/isced/{id}")
    public IscedFoet getById(@PathVariable Long id,
            @RequestParam(required = false, defaultValue = "") String[] languages) {
        return this.iscedFoetService.findById(id, languages)
                .map(unproxyIscedFoetConverter::convert)
                .orElseThrow(() -> new NotFoundException(String.format("ISCED FoET with id [%s] not found", id)));
    }

    /**
     * Retrieve a collection of {@link IscedFoet}.
     *
     * @param resultLanguages the languages to search on
     * @param page            zero-based page index.
     * @param pageSize        the size of the page to be returned.
     * @param sortField       the properties to sort by, must not be {@literal null} or empty.
     * @param sortDirection   the direction of the {@link Sort} to be specified, can be {@literal null}.
     * @param q               the query string search parameter.
     * @return The ordered and paginated collection of {@link IscedFoet}.
     */
    @Cacheable(value = "api-isceds")
    @RequestMapping("/api/isceds")
    public Iterable<IscedFoet> search(@RequestParam(required = false, defaultValue = "") String[] resultLanguages,
                                      @RequestParam(required = false, defaultValue = "0") int page,
                                      @RequestParam(required = false, defaultValue = "10") int pageSize,
                                      @RequestParam(required = false, defaultValue = "id") String sortField,
                                      @RequestParam(required = false, defaultValue = "ASC") Sort.Direction sortDirection,
                                      @RequestParam(required = false) String q) {
        return this.iscedFoetService.search(q, resultLanguages, PageRequest.of(page, pageSize, sortDirection, sortField))
                .map(unproxyIscedFoetConverter::convert);
    }
}
