/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.api;

import eu.europa.empl.skpt.api.dto.LanguageDto;
import eu.europa.empl.skpt.api.dto.LanguageToLanguageDtoConverter;
import eu.europa.empl.skpt.api.dto.UnproxyLanguageConverter;
import eu.europa.empl.skpt.api.exception.NotFoundException;
import eu.europa.empl.skpt.data.domain.language.Language;
import eu.europa.empl.skpt.service.language.ILanguageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

// CLOVER:OFF

/**
 * Controller for the {@link Language} entity.
 *
 * @author ARHS Spikeseed
 */
@RestController
public class LanguageController {

    private final ILanguageService languageService;
    private final UnproxyLanguageConverter unproxyLanguageConverter;
    private final LanguageToLanguageDtoConverter languageToLanguageDtoConverter;

    /**
     * Contructor.
     *
     * @param languageService          the {@link Language} service.
     * @param unproxyLanguageConverter unproxy language converter
     */
    @Autowired
    public LanguageController(ILanguageService languageService, UnproxyLanguageConverter unproxyLanguageConverter) {
        this.languageService = languageService;
        this.unproxyLanguageConverter = unproxyLanguageConverter;
        this.languageToLanguageDtoConverter = new LanguageToLanguageDtoConverter();
    }

    /**
     * Retrieve a collection of {@link Language}.
     *
     * @param resultLanguages the languages returned.
     * @param searchLanguages the languages to search on
     * @param page            zero-based page index.
     * @param pageSize        the size of the page to be returned.
     * @param sortField       the properties to sort by, must not be {@literal null} or empty.
     * @param sortDirection   the direction of the {@link Sort} to be specified, can be {@literal null}.
     * @param q               the query string search parameter.
     * @return The ordered and paginated collection of {@link Language}.
     */
    @Cacheable(value = "api-languages")
    @RequestMapping("/api/languages")
    public Iterable<Language> search(@RequestParam(required = false, defaultValue = "") String[] resultLanguages,
                                     @RequestParam(required = false, defaultValue = "") Set<String> searchLanguages,
                                     @RequestParam(required = false, defaultValue = "0") int page,
                                     @RequestParam(required = false, defaultValue = "10") int pageSize,
                                     @RequestParam(required = false, defaultValue = "id") String sortField,
                                     @RequestParam(required = false, defaultValue = "ASC") Sort.Direction sortDirection,
                                     @RequestParam(required = false) String q) {
        return this.languageService.search(q, searchLanguages, resultLanguages, PageRequest.of(page, pageSize, sortDirection, sortField))
                .map(unproxyLanguageConverter::convert);
    }

    /**
     * Retrieve a {@link Language} identified by the given ISO code 1.
     *
     * @param code      the {@link Language} ISO code 1.
     * @param languages the languages returned.
     * @return The {@link Language} identified by the given ISO code 1.
     */
    @Cacheable(value = "api-language")
    @RequestMapping("/api/language/{code}")
    public Language getLanguageByIsoCode1(@PathVariable String code,
                                          @RequestParam(required = false, defaultValue = "") String[] languages) {
        return this.languageService.findByIsoCode1(code, languages)
                .map(unproxyLanguageConverter::convert)
                .orElseThrow(() -> new NotFoundException(String.format("Language with code [%s] not found", code)));
    }

    /**
     * Retrieve a {@link Language} with the application languages
     * @param page the pagination parameters
     * @param pageSize        the size of the page to be returned.
     * @param sortField       the properties to sort by, must not be {@literal null} or empty.
     * @param sortDirection   the direction of the {@link Sort} to be specified, can be {@literal null}.
     * @param selectorType    the type of request
     * @return The ordered and paginated collection of {@link Language}.
     */
    @RequestMapping("/api/languages/supported")
    public Iterable<LanguageDto> getSupportedLanguages(
            @RequestParam(required = false, defaultValue = "0") int page,
            @RequestParam(required = false, defaultValue = "10") int pageSize,
            @RequestParam(required = false, defaultValue = "id") String sortField,
            @RequestParam(required = false, defaultValue = "ASC") Sort.Direction sortDirection,
            @RequestParam(required = false, defaultValue = "primary") String selectorType) {
        Iterable<Language> input = null;

        if ("primary".equals(selectorType)) {
            input = this.languageService.findByPrimarySupported(PageRequest.of(page, pageSize,
                    sortDirection, sortField));
        } else if ("secondary".equals(selectorType)) {
            input = this.languageService.findBySecondarySupported(PageRequest.of(page, pageSize,
                    sortDirection, sortField));
        } else {
            return null;
        }

        return StreamSupport.stream(input.spliterator(), false)
                .map(this.languageToLanguageDtoConverter::convert)
                .collect(Collectors.toList());
    }

}
