/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.api;

import eu.europa.empl.skpt.api.dto.UnproxyNaceConverter;
import eu.europa.empl.skpt.api.exception.NotFoundException;
import eu.europa.empl.skpt.data.domain.nace.Nace;
import eu.europa.empl.skpt.service.nace.INaceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

// CLOVER:OFF

/**
 * Controller for the {@link Nace} entity.
 *
 * @author ARHS Spikeseed
 */
@RestController
public class NaceController {

    private final INaceService naceService;
    private final UnproxyNaceConverter unproxyNaceConverter;

    /**
     * Constructor.
     *
     * @param naceService          the {@link Nace} service.
     * @param unproxyNaceConverter unproxy nace converter
     */
    @Autowired
    public NaceController(INaceService naceService, UnproxyNaceConverter unproxyNaceConverter) {
        this.naceService = naceService;
        this.unproxyNaceConverter = unproxyNaceConverter;
    }

    /**
     * Retrieve a {@link Nace} identified by the given identifier.
     *
     * @param id        the {@link Nace} identifier.
     * @param languages the languages returned.
     * @return The {@link Nace} identified by the given identifier.
     */
    @Cacheable(value = "api-nace")
    @RequestMapping("/api/nace/{id}")
    public Nace getById(@PathVariable Long id,
            @RequestParam(required = false, defaultValue = "") String[] languages) {
        return this.naceService.findById(id, languages)
                .map(unproxyNaceConverter::convert)
                .orElseThrow(() -> new NotFoundException(String.format("NACE with id [%s] not found", id)));
    }

    /**
     * Retrieve a collection of {@link Nace}.
     *
     * @param resultLanguages the languages of the results.
     * @param page            zero-based page index.
     * @param pageSize        the size of the page to be returned.
     * @param sortField       the properties to sort by, must not be {@literal null} or empty.
     * @param sortDirection   the direction of the {@link Sort} to be specified, can be {@literal null}.
     * @param q               the query string search parameter.
     * @return The ordered and paginated collection of {@link Nace}.
     */
    @Cacheable(value = "api-naces")
    @RequestMapping("/api/naces")
    public Iterable<Nace> search(@RequestParam(required = false, defaultValue = "") String[] resultLanguages,
                                 @RequestParam(required = false, defaultValue = "0") int page,
                                 @RequestParam(required = false, defaultValue = "10") int pageSize,
                                 @RequestParam(required = false, defaultValue = "id") String sortField,
                                 @RequestParam(required = false, defaultValue = "ASC") Sort.Direction sortDirection,
                                 @RequestParam(required = false) String q) {
        return this.naceService.search(q, resultLanguages, PageRequest.of(page, pageSize, sortDirection, sortField))
                .map(unproxyNaceConverter::convert);
    }
}
