/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.api.common;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import org.springframework.data.domain.Page;
import org.springframework.util.Assert;

import java.util.Collections;
import java.util.List;

/**
 * Wraps a Page so that it can be returned as JSON
 *
 * @param <T> the wrapped result type
 *
 * @author ARHS Spikeseed
 */
@JsonPropertyOrder({"number",
        "numberOfElements",
        "size",
        "totalPages",
        "totalElements",
        "content"
})
public class PageWrapper<T> {

    private final Page page;

    /**
     * Syntax sugar to create a PageWrapper object.
     *
     * @param page the Page to wrap
     * @param <D>  D
     * @return a new PageWrapper object
     */
    public static <D> PageWrapper<D> of(Page<D> page) {
        Assert.notNull(page, "'page' is null");
        return new PageWrapper<>(page);
    }

    /**
     * Constructor
     *
     * @param page the Page to wrap
     */
    public PageWrapper(Page<T> page) {
        Assert.notNull(page, "'page' is null");
        this.page = page;
    }

    // CLOVER:OFF

    /**
     * Return the page content.
     *
     * @return the page content
     * @see org.springframework.data.domain.Page#getContent()
     */
    public List<T> getContent() {
        return Collections.unmodifiableList(page.getContent());
    }

    /**
     * Return the current page number.
     *
     * @return the number of the current page
     * @see org.springframework.data.domain.Page#getNumber()
     */
    public int getNumber() {
        return page.getNumber();
    }

    /**
     * Return the size of the current page.
     *
     * @return the size of the current page
     * @see org.springframework.data.domain.Page#getSize()
     */
    public int getSize() {
        return page.getSize();
    }

    /**
     * Return the total number of pages.
     *
     * @return the total number of pages
     * @see org.springframework.data.domain.Page#getTotalPages()
     */
    public int getTotalPages() {
        return page.getTotalPages();
    }

    /**
     *
     * Return the total amount of elements.
     *
     * @return the total amount of elements
     * @see org.springframework.data.domain.Page#getTotalElements()
     */
    public long getTotalElements() {
        return page.getTotalElements();
    }

    /**
     * Return the number of elements in this page.
     *
     * @return the number of elements in this page
     * @see org.springframework.data.domain.Page#getNumberOfElements()
     */
    public long getNumberOfElements() {
        return page.getNumberOfElements();
    }

}
