/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 05/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.api.exception;

import eu.europa.empl.skpt.common.Error;
import org.apache.tomcat.util.http.fileupload.impl.FileSizeLimitExceededException;
import org.apache.tomcat.util.http.fileupload.impl.SizeLimitExceededException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.multipart.MaxUploadSizeExceededException;
import org.springframework.web.multipart.MultipartException;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import java.util.*;

import static eu.europa.empl.skpt.common.Error.INTERNAL_SERVER_ERROR;

/**
 * Error handler.
 *
 * @author ARHS Spikeseed
 */
@ControllerAdvice
public class ErrorHandler extends ResponseEntityExceptionHandler {

    private static final Logger ERROR_HANDLER_LOGGER = LoggerFactory.getLogger(ErrorHandler.class);

    @Override
    protected ResponseEntity<Object> handleExceptionInternal(Exception ex, Object body, HttpHeaders headers, HttpStatus status,
                                                             WebRequest request) {

        if (HttpStatus.INTERNAL_SERVER_ERROR.equals(status)) {
            ERROR_HANDLER_LOGGER.error("An unexpected exception occurred", ex);
        } else {
            ERROR_HANDLER_LOGGER.debug("Exception occurred", ex);
        }

        return super.handleExceptionInternal(ex, body, headers, status, request);
    }

    /**
     * Handles custom exceptions
     *
     * @param ex      the exception
     * @param request the web request
     * @return the response entity to return
     * @throws NoSuchMethodException if occurs
     */
    @ExceptionHandler({
            ApiException.class,
            RuntimeException.class
    })
    public final ResponseEntity<Object> handleCustomException(Exception ex, WebRequest request) throws NoSuchMethodException {

        HttpHeaders headers = new HttpHeaders();
        HttpStatus status = getHttpStatus(ex);
        ErrorHandlerResponseBody body = getErrorHandlerResponseBody(ex, status);

        return handleExceptionInternal(ex, body, headers, status, request);
    }

    /**
     * Handles multipart exception
     *
     * @param ex      the exception
     * @param request the web request
     * @return the response entity to return
     * @throws NoSuchMethodException if occurs
     */
    @ExceptionHandler({
            MultipartException.class
    })
    public final ResponseEntity<Object> handleMultipartException(MultipartException ex, WebRequest request) throws NoSuchMethodException {

        Throwable rootCause = ex.getMostSpecificCause();
        if (rootCause instanceof FileSizeLimitExceededException
                || ex instanceof MaxUploadSizeExceededException) {
            return handleCustomException(new BadRequestException(ex, new ArrayList(EnumSet.of(Error.FILE_EXCEEDS_MAX_SIZE))),
                    request);
        }

        if (rootCause instanceof SizeLimitExceededException) {
            return handleCustomException(new BadRequestException(ex, new ArrayList(EnumSet.of(Error.REQUEST_EXCEEDS_MAX_SIZE))),
                    request);
        }

        return handleCustomException(ex, request);
    }

    private ErrorHandlerResponseBody getErrorHandlerResponseBody(Exception ex, HttpStatus status) {
        if (ex instanceof ApiException) {
            ApiException apiException = (ApiException) ex;
            if (apiException.hasErrors()) {
                List<ErrorResponseBody> errorResponseBodyList = new ArrayList<>();
                for (Error error : apiException.getErrors()) {
                    errorResponseBodyList.add(new ErrorResponseBody(error.getId(), error.getDescription()));
                }
                return new ErrorHandlerResponseBody(status.value(), errorResponseBodyList);
            }
        }

        ErrorResponseBody errorResponseBody = new ErrorResponseBody(INTERNAL_SERVER_ERROR.getId(),
                INTERNAL_SERVER_ERROR.getDescription());
        return new ErrorHandlerResponseBody(status.value(), Collections.singletonList(errorResponseBody));
    }

    private HttpStatus getHttpStatus(Exception ex) throws NoSuchMethodException {
        if (ex.getClass().isAnnotationPresent(ResponseStatus.class)) {

            Object defaultValue = ResponseStatus.class.getMethod("value").getDefaultValue();

            HttpStatus value = ex.getClass().getAnnotation(ResponseStatus.class).value();
            HttpStatus code = ex.getClass().getAnnotation(ResponseStatus.class).code();

            return Objects.equals(defaultValue, value) ? code : value;
        }
        return HttpStatus.INTERNAL_SERVER_ERROR;
    }

}
