/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 16/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.api.export;

import eu.europa.empl.skpt.api.exception.ExportIOException;
import eu.europa.empl.skpt.api.exception.ExportURISyntaxException;
import eu.europa.empl.skpt.common.Error;
import eu.europa.empl.skpt.service.export.IExportService;
import eu.europa.empl.skpt.service.export.model.ParseJsonResult;
import eu.europa.empl.skpt.service.export.model.profile.PersonalInfo;
import eu.europa.empl.skpt.service.export.model.profile.Profile;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.util.Optional;

import static eu.europa.empl.skpt.common.Error.EXPORT_WORD_IO_EXCEPTION;
import static eu.europa.empl.skpt.common.Error.EXPORT_WORD_URI_SYNTAX_EXCEPTION;

// CLOVER:OFF

/**
 * Controller handling export requests.
 *
 * @author ARHS Spikeseed
 */
@RestController
public class ExportController {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportController.class);

    @Autowired
    private IExportService exportService;

    private static final String HEADER_VALUE_START = "attachment; filename*=UTF-8''";

    /**
     * Exports the provided document in XML.
     *
     * @param response HTTP response
     * @param json     json
     * @param dlToken  download token
     * @return the exported document in XML
     * @throws IOException if occurs
     */
    @RequestMapping(value = "/api/export/xml", produces = MediaType.APPLICATION_XML_VALUE,
            method = RequestMethod.POST)
    public HttpEntity<byte[]> exportXml(HttpServletResponse response,
            @RequestBody String json,
            @RequestParam("DL-TOKEN") String dlToken) {

        ParseJsonResult parseJsonResult = exportService.parseJson(json);
        Optional<Profile> profileOptional = parseJsonResult.getProfile();

        this.checkIfDlTokenIsNotBlank(response, dlToken);

        ResponseEntity responseEntity = this.checkIfProfileIsPresent(profileOptional, true);
        if (responseEntity != null) {
            return responseEntity;
        }

        Profile profile = profileOptional.get();
        PersonalInfo personalInfo = profile.getPersonalInfo();

        String fileName = getFileName(personalInfo, "xml");

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {

            exportService.toXmlWithAttachments(os, profile);

            byte[] byteArray = os.toByteArray();

            HttpHeaders headers = new HttpHeaders();
            headers.set(HttpHeaders.CONTENT_DISPOSITION,
                    HEADER_VALUE_START + URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()) + "");
            headers.setContentLength(byteArray.length);

            return new ResponseEntity<>(byteArray, headers, HttpStatus.OK);
        } catch (IOException e) {
            LOGGER.error("Error during the Xml export", e);
            throw new ExportIOException("Error during the Xml export", Error.EXPORT_PDF_IO_EXCEPTION);
        }
    }

    /**
     * Exports the provided document in PDF.
     *
     * @param response HTTP response
     * @param json     json
     * @param dlToken  download token
     * @param lang     the language to produce the PDF in
     * @return the exported document in PDF
     * @throws IOException if occurs
     */
    @RequestMapping(value = "/api/export/pdf", produces = MediaType.APPLICATION_PDF_VALUE,
            method = RequestMethod.POST)
    public HttpEntity<byte[]> exportPdf(HttpServletResponse response,
            @RequestBody String json,
            @ModelAttribute("DL-TOKEN") String dlToken, @RequestParam("lang") String lang) {

        ParseJsonResult parseJsonResult = exportService.parseJson(json);
        Optional<Profile> profileOptional = parseJsonResult.getProfile();

        this.checkIfDlTokenIsNotBlank(response, dlToken);

        ResponseEntity responseEntity = this.checkIfProfileIsPresent(profileOptional);
        if (responseEntity != null) {
            return responseEntity;
        }

        Profile profile = profileOptional.get();
        PersonalInfo personalInfo = profile.getPersonalInfo();

        String fileName = getFileName(personalInfo, "pdf");

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {

            exportService.toPdf(os, profile, lang, false);

            byte[] byteArray = os.toByteArray();

            HttpHeaders headers = new HttpHeaders();
            headers.set(HttpHeaders.CONTENT_DISPOSITION,
                    HEADER_VALUE_START + URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()) + "");
            headers.setContentLength(byteArray.length);

            return new ResponseEntity<>(byteArray, headers, HttpStatus.OK);
        } catch (IOException e) {
            LOGGER.error("Error during the Pdf export", e);
            throw new ExportIOException("Error during the Pdf export", Error.EXPORT_PDF_IO_EXCEPTION);
        }
    }

    /**
     * Exports the provided document in docx (MS Word)
     *
     * @param response HTTP response
     * @param json     json
     * @param dlToken  download token
     * @param lang     the language to produce the docx file in
     * @return the exported document in docx format
     * @throws IOException        if occurs
     * @throws URISyntaxException URISyntaxException thrown when accessing URI
     */
    @RequestMapping(value = "/api/export/word", produces = "application/vnd.openxmlformats-officedocument.wordprocessingml.document")
    public HttpEntity<byte[]> exportWord(
            HttpServletResponse response,
            @RequestBody String json,
            @ModelAttribute("DL-TOKEN") String dlToken,
            @RequestParam("lang") String lang) {

        ParseJsonResult parseJsonResult = exportService.parseJson(json);
        Optional<Profile> profileOptional = parseJsonResult.getProfile();

        this.checkIfDlTokenIsNotBlank(response, dlToken);

        ResponseEntity responseEntity = this.checkIfProfileIsPresent(profileOptional);
        if (responseEntity != null) {
            return responseEntity;
        }

        Profile profile = profileOptional.get();
        PersonalInfo personalInfo = profile.getPersonalInfo();

        String fileName = getFileName(personalInfo, "docx");

        String pictureId = personalInfo.getProfilePic() == null
                ? null : personalInfo.getProfilePic().getId();

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            exportService.createDocxFile(os, fileName, profile, lang, pictureId);
            byte[] byteArray = os.toByteArray();

            HttpHeaders headers = new HttpHeaders();
            headers.set(HttpHeaders.CONTENT_DISPOSITION,
                    HEADER_VALUE_START + URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()) + "");
            headers.setContentLength(byteArray.length);

            return new ResponseEntity<>(byteArray, headers, HttpStatus.OK);
        } catch (IOException e) {
            LOGGER.error("Error during the Word export", e);
            throw new ExportIOException("Error during the Word export", EXPORT_WORD_IO_EXCEPTION);
        } catch (URISyntaxException e) {
            LOGGER.error("Error with the syntax of the URI for the Word export", e);
            throw new ExportURISyntaxException("Error with the syntax of the URI for the Word export", EXPORT_WORD_URI_SYNTAX_EXCEPTION);
        }
    }

    /**
     * Exports the provided document in xlsx (MS Excel)
     *
     * @param response HTTP response
     * @param json json
     * @param dlToken dlToken
     * @param lang lang
     * @return the exported document in xlsx format
     */
    @RequestMapping(value = "/api/export/excel", produces = "application/vnd.ms-excel")
    public HttpEntity<byte[]> exportExcel(
            HttpServletResponse response,
            @RequestBody String json,
            @ModelAttribute("DL-TOKEN") String dlToken,
            @RequestParam("lang") String lang) {

        ParseJsonResult parseJsonResult = exportService.parseJson(json);
        Optional<Profile> profileOptional = parseJsonResult.getProfile();

        this.checkIfDlTokenIsNotBlank(response, dlToken);

        ResponseEntity responseEntity = this.checkIfProfileIsPresent(profileOptional);
        if (responseEntity != null) {
            return responseEntity;
        }

        Profile profile = profileOptional.get();

        String fileName = getFileName(profile.getPersonalInfo(), "xlsx");

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            exportService.createXlsxFile(os, profile, lang);
            byte[] byteArray = os.toByteArray();

            HttpHeaders headers = new HttpHeaders();
            headers.set(HttpHeaders.CONTENT_DISPOSITION,
                    HEADER_VALUE_START + URLEncoder.encode(fileName, StandardCharsets.UTF_8.name()) + "");
            headers.setContentLength(byteArray.length);

            return new ResponseEntity<>(byteArray, headers, HttpStatus.OK);
        } catch (IOException e) {
            LOGGER.error("Error during the Excel export", e);
            throw new ExportIOException("Error during the Excel export", EXPORT_WORD_IO_EXCEPTION);
        }
    }

    private String getFileName(PersonalInfo personalInfo, String extension) {
        return exportService.getFileName(personalInfo.getFamilyName(), personalInfo.getFirstName(),
                LocalDate.now(), extension);
    }

    /**
     * Check if the dlToken is not blank
     *
     * @param response HTTP response
     * @param dlToken  download token
     * @return HTTP response
     */
    private HttpServletResponse checkIfDlTokenIsNotBlank(HttpServletResponse response, String dlToken) {
        if (StringUtils.isNotBlank(dlToken)) {
            Cookie cookie = new Cookie("DL-TOKEN", dlToken);
            cookie.setSecure(true);
            cookie.setPath("/");
            cookie.setHttpOnly(false);
            response.addCookie(cookie);
        }
        return response;
    }

    /**
     * Check if profileOptional is present
     *
     * @param profileOptional profileOptional from the json
     * @return response entity
     */
    private ResponseEntity checkIfProfileIsPresent(Optional<Profile> profileOptional) {
        return checkIfProfileIsPresent(profileOptional, false);
    }

    /**
     * Check if profile is present
     *
     * @param profileOptional profile from the json
     * @param checkOutcome    indicates if the outcome must be checked
     * @return response entity
     */
    private ResponseEntity checkIfProfileIsPresent(Optional<Profile> profileOptional, boolean checkOutcome) {
        if (!profileOptional.isPresent()) {
            return new ResponseEntity<>(new byte[] {}, HttpStatus.BAD_REQUEST);
        }
        if (profileOptional.get().getPersonalInfo() == null
                || StringUtils.isBlank(profileOptional.get().getPersonalInfo().getFamilyName())) {
            return new ResponseEntity<>(new byte[] {}, HttpStatus.BAD_REQUEST);
        }
        if (StringUtils.isBlank(profileOptional.get().getPersonalInfo().getFirstName())) {
            return new ResponseEntity<>(new byte[] {}, HttpStatus.BAD_REQUEST);
        }
        if (checkOutcome && (profileOptional.get().getOutcome().getStatistics() == null
                || profileOptional.get().getOutcome().getStored() == null)) {
            return new ResponseEntity<>(new byte[] {}, HttpStatus.BAD_REQUEST);
        }
        return null;
    }
}
