/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 12/4/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.config.impl;

import eu.europa.empl.skpt.config.IApplicationProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

// CLOVER:OFF

/**
 * Holds the application properties injected by the Spring mechanism.
 *
 * @author ARHS Spikeseed
 */
@Component
public class ApplicationProperties implements IApplicationProperties {

    private static final String ATTACHMENT_DIR_KEY = "skpt.attachment.dir";
    private static final String ATTACHMENT_DIR_KEY_VAR = "${" + ATTACHMENT_DIR_KEY + "}";

    private static final String ATTACHMENT_ACCEPTED_MEDIA_TYPES_KEY = "skpt.attachment.accepted.media.types";
    private static final String ATTACHMENT_ACCEPTED_MEDIA_TYPES_KEY_VAR = "${" + ATTACHMENT_ACCEPTED_MEDIA_TYPES_KEY + "}";

    private static final String ATTACHMENT_ACCEPTED_EXTENSIONS_KEY = "skpt.attachment.accepted.extensions";
    private static final String ATTACHMENT_ACCEPTED_EXTENSIONS_KEY_VAR = "${" + ATTACHMENT_ACCEPTED_EXTENSIONS_KEY + "}";

    private static final String ATTACHMENT_MAX_SIZE_KEY = "skpt.attachment.max.size";
    private static final String ATTACHMENT_MAX_SIZE_KEY_VAR = "${" + ATTACHMENT_MAX_SIZE_KEY + "}";

    private static final String ATTACHMENT_WINDOW_KEY = "skpt.attachment.window";
    private static final String ATTACHMENT_MAX_WINDOW_VAR = "${" + ATTACHMENT_WINDOW_KEY + "}";

    private final File attachmentDir;
    private final List<String> attachmentAcceptedMediaTypes;
    private final List<String> attachmentAcceptedExtensions;
    private final long attachmentMaxSize;
    private final long attachmentWindow;

    /**
     * Constructor
     *
     * @param attachmentDir                the property identified by the key 'skpt.attachment.dir'
     * @param attachmentAcceptedMediaTypes the property identified by the key 'skpt.attachment.accepted.media.types'
     * @param attachmentAcceptedExtensions the property identified by the key 'skpt.attachment.accepted.extensions'
     * @param attachmentMaxSize            the property identified by the key 'skpt.attachment.max.size'
     * @param attachmentWindow             the property identified by the key 'skpt.attachment.window'
     */
    @Autowired
    public ApplicationProperties(@Value(ATTACHMENT_DIR_KEY_VAR) File attachmentDir,
            @Value(ATTACHMENT_ACCEPTED_MEDIA_TYPES_KEY_VAR) List<String> attachmentAcceptedMediaTypes,
            @Value(ATTACHMENT_ACCEPTED_EXTENSIONS_KEY_VAR) List<String> attachmentAcceptedExtensions,
            @Value(ATTACHMENT_MAX_SIZE_KEY_VAR) long attachmentMaxSize,
            @Value(ATTACHMENT_MAX_WINDOW_VAR) long attachmentWindow) {
        this.attachmentDir = attachmentDir;
        this.attachmentAcceptedMediaTypes = attachmentAcceptedMediaTypes;
        this.attachmentAcceptedExtensions = attachmentAcceptedExtensions;
        this.attachmentMaxSize = attachmentMaxSize;
        this.attachmentWindow = attachmentWindow;
    }

    @Override
    public File getAttachmentDir() {
        return attachmentDir;
    }

    @Override
    public List<String> getAttachmentAcceptedMediaTypes() {
        return attachmentAcceptedMediaTypes != null ? new ArrayList<>(attachmentAcceptedMediaTypes) : Collections.emptyList();
    }

    @Override
    public List<String> getAttachmentAcceptedExtensions() {
        return attachmentAcceptedExtensions != null ? new ArrayList<>(attachmentAcceptedExtensions) : Collections.emptyList();
    }

    @Override
    public long getAttachmentMaxSize() {
        return attachmentMaxSize;
    }

    @Override
    public long getAttachmentWindow() {
        return attachmentWindow;
    }

    /**
     * post construct
     */
    @PostConstruct
    public void postConstruct() {

        if (this.attachmentDir == null) {
            throw new IllegalArgumentException("A value for '" + ATTACHMENT_DIR_KEY + "' must be specified");
        }

        if (!this.attachmentDir.exists()) {
            throw new IllegalArgumentException(
                    "The directory specified for '" + ATTACHMENT_DIR_KEY + "' does not exist: " + this.attachmentDir.getPath());
        }

        String valueSpecifiedForAttachmentDirKey = "The value specified for '" + ATTACHMENT_DIR_KEY;

        if (!this.attachmentDir.isDirectory()) {
            throw new IllegalArgumentException(
                    valueSpecifiedForAttachmentDirKey + "' is not a directory: " + this.attachmentDir.getPath());
        }

        if (this.attachmentMaxSize <= 0) {
            throw new IllegalArgumentException(
                    valueSpecifiedForAttachmentDirKey + "' must be greater than 0: " + this.attachmentMaxSize);
        }

        if (this.attachmentWindow <= 0) {
            throw new IllegalArgumentException(
                    valueSpecifiedForAttachmentDirKey + "' must be greater than 0: " + this.attachmentWindow);
        }

    }
}
