/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 15/02/2018
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.data.domain.stats;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import eu.europa.empl.skpt.data.serializer.ProfileStatisticDeserializer;
import lombok.Data;

// CLOVER:OFF

/**
 * The profile entity used for the statistics.
 *
 * @author ARHS Spikeseed
 */
@Data
@JsonDeserialize(using = ProfileStatisticDeserializer.class)
public class ProfileStatistic {

    private String hostCountry;
    private List<String> organizationTypes;
    private String gender;
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd")
    private Date dateOfBirth;
    private Integer age;
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd")
    private Date dateOfArrival;
    private List<String> nationalityCountries;
    private List<String> expectations;
    private String educationLevel;
    private List<String> fieldOfStudy;
    private List<String> languages;
    private List<String> escoOccupations;
    private List<String> escoSkills;

    /**
     * Return static {@link ProfileStatisticBuilder} instance
     *
     * @return ProfileStatisticBuilder instance
     */
    public static ProfileStatisticBuilder builder() {
        return new ProfileStatisticBuilder();
    }

    /**
     * The builder class for {@link ProfileStatistic}
     *
     * @author ARHS Spikeseed
     */
    public static class ProfileStatisticBuilder {

        private String hostCountry;
        private List<String> organizationTypes;
        private String gender;
        private Calendar dateOfBirth;
        private Integer age;
        private Calendar dateOfArrival;
        private List<String> nationalityCountries;
        private List<String> expectations;
        private String educationLevel;
        private List<String> fieldsOfStudy;
        private List<String> languages;
        private List<String> escoOccupations;
        private List<String> escoSkills;

        /**
         * Build a {@link ProfileStatistic} instance using the {@link ProfileStatisticBuilder} attributes
         *
         * @return ProfileStatistic
         */
        public ProfileStatistic build() {
            ProfileStatistic profileStatistic = new ProfileStatistic();
            profileStatistic.setHostCountry(this.hostCountry);
            profileStatistic.setOrganizationTypes(this.organizationTypes);
            profileStatistic.setGender(this.gender);
            Date birthDate = this.dateOfBirth == null ? null : this.dateOfBirth.getTime();
            profileStatistic.setDateOfBirth(birthDate);
            profileStatistic.setAge(this.age);
            Date arrivalDate = this.dateOfArrival == null ? null : this.dateOfArrival.getTime();
            profileStatistic.setDateOfArrival(arrivalDate);
            profileStatistic.setNationalityCountries(this.nationalityCountries);
            profileStatistic.setExpectations(this.expectations);
            profileStatistic.setFieldOfStudy(this.fieldsOfStudy);
            profileStatistic.setEducationLevel(this.educationLevel);
            profileStatistic.setLanguages(this.languages);
            profileStatistic.setEscoOccupations(this.escoOccupations);
            profileStatistic.setEscoSkills(this.escoSkills);
            return profileStatistic;
        }

        /**
         * Set the host country.
         *
         * @param hostCountry host country
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withHostCountry(String hostCountry) {
            this.hostCountry = hostCountry;
            return this;
        }

        /**
         * Set the organisation types.
         *
         * @param organizationTypes organisation type {@link List}
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withOrganizaionTypes(List<String> organizationTypes) {
            this.organizationTypes = organizationTypes;
            return this;
        }

        /**
         * Set gender.
         *
         * @param gender the gender
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withGender(String gender) {
            this.gender = gender;
            return this;
        }

        /**
         * Set the date of birth and age with the given date.
         *
         * @param dateOfBirth date of birth
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withDateOfBirth(Calendar dateOfBirth) {

            if (dateOfBirth != null) {
                dateOfBirth.set(Calendar.HOUR_OF_DAY, 0);
                dateOfBirth.set(Calendar.MINUTE, 0);
                dateOfBirth.set(Calendar.SECOND, 0);
                dateOfBirth.set(Calendar.MILLISECOND, 0);
                this.dateOfBirth = dateOfBirth;
                this.age = Calendar.getInstance().get(Calendar.YEAR) - dateOfBirth.get(Calendar.YEAR);
            }
            return this;
        }

        /**
         * Set the date of arrival.
         *
         * @param dateOfArrival the date of arrival
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withArrivalDate(Calendar dateOfArrival) {
            if (dateOfArrival != null) {
                this.dateOfArrival = dateOfArrival;
                this.dateOfArrival.set(Calendar.HOUR_OF_DAY, 0);
                this.dateOfArrival.set(Calendar.MINUTE, 0);
                this.dateOfArrival.set(Calendar.SECOND, 0);
                this.dateOfArrival.set(Calendar.MILLISECOND, 0);
            }
            return this;
        }

        /**
         * Set the nationalities country
         *
         * @param nationalityCountries the nationalities {@link List}
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withNationalitiesCountry(List<String> nationalityCountries) {
            this.nationalityCountries = nationalityCountries;
            return this;
        }

        /**
         * Set the expectations.
         *
         * @param expectations the expectations {@link List}
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withExpectations(List<String> expectations) {
            this.expectations = expectations;
            return this;
        }

        /**
         * Set education level.
         *
         * @param educationLevel the education level.
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withEducationLevel(String educationLevel) {
            this.educationLevel = educationLevel;
            return this;
        }

        /**
         * Set the fields of study.
         *
         * @param fieldsOfStudy the fields of study {@link List}
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withFieldsOfStudy(List<String> fieldsOfStudy) {
            this.fieldsOfStudy = fieldsOfStudy;
            return this;
        }

        /**
         * Set the languages.
         *
         * @param languages the languages {@link List}
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withLanguages(List<String> languages) {
            this.languages = languages;
            return this;
        }

        /**
         * Set the ESCO occupations.
         *
         * @param escoOccupations the ESCO occuaptions {@link List}
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withEscoOccupations(List<String> escoOccupations) {
            this.escoOccupations = escoOccupations;
            return this;
        }

        /**
         * Set the ESCO skills.
         *
         * @param escoSkills the ESCO skills {@link List}
         * @return the {@link ProfileStatisticBuilder}
         */
        public ProfileStatisticBuilder withEscoSkills(List<String> escoSkills) {
            this.escoSkills = escoSkills;
            return this;
        }

    }
}
