/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.data.repository.country;

import eu.europa.empl.skpt.data.domain.country.Country;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.repository.query.Param;

import java.util.Optional;
import java.util.Set;

/**
 * Repository retrieving {@link Country} entities
 *
 * @author ARHS Spikeseed
 */
public interface ICountryRepository extends PagingAndSortingRepository<Country, Long> {

    /**
     * Returns a {@link Page} of {@link Country} entities meeting the paging restriction provided in the
     * {@code Pageable} object.
     *
     * @param language  the search language
     * @param pageable  the pagination parameters
     * @return a page of entities
     */
    @Query("SELECT DISTINCT c FROM Country c JOIN c.values cv WHERE cv.language = :language")
    Page<Country> findByLanguage(@Param("language") String language, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link Country} entities meeting the paging restriction provided in the
     * {@code Pageable} object.
     *
     * @param pageable the pagination parameters
     * @param value    parameter matched against {@link eu.europa.empl.skpt.data.domain.country.CountryValue#value}
     * @return a page of entities
     */
    @Query("SELECT DISTINCT c FROM Country c JOIN c.values cv WHERE cv.value LIKE concat('%', :value, '%')")
    Page<Country> findByValueIgnoreCase(@Param("value") String value, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link Country} entities meeting the paging restriction provided in the
     * {@code Pageable} object.
     *
     * @param pageable  the pagination parameters
     * @param value     parameter matched against {@link eu.europa.empl.skpt.data.domain.country.CountryValue#value}
     * @param languages the languages for which the value will be matched against
     * @return a page of entities
     */
    @Query("SELECT DISTINCT c FROM Country c JOIN c.values cv WHERE cv.language IN :langs"
            + " AND cv.value LIKE concat('%', :value, '%')")
    Page<Country> findByValueIgnoreCaseAndLanguages(@Param("langs") Set<String> languages, @Param("value") String
            value, Pageable pageable);
    /**
     * Returns a {@link Optional} of a {@link Country} entity by its ISO 3166 A3 code.
     *
     * @param code the ISO 3166 A3 country code
     * @return a country
     */
    Optional<Country> findByAlphaCode3(String code);
}
