/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.data.repository.esco;

import eu.europa.empl.skpt.data.domain.esco.EscoClassification;
import eu.europa.empl.skpt.data.domain.esco.EscoLocalConcept;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.repository.query.Param;

import java.util.Optional;
import java.util.Set;

/**
 * Repository retrieving {@link EscoLocalConcept} entities
 *
 * @author ARHS Spikeseed
 */
public interface IEscoRepository extends PagingAndSortingRepository<EscoLocalConcept, Long> {

    /**
     * Returns the {@link EscoLocalConcept} mathing the unique identifier specified.
     *
     * @param id the unique identifier
     * @return an {@link Optional} of {@link EscoLocalConcept}
     */
    Optional<EscoLocalConcept> findById(Long id);

    /**
     * Returns a {@link Page} of {@link EscoLocalConcept} entities meeting the paging restriction provided in the
     * {@code Pageable} object.
     *
     * @param pageable the pagination parameters
     * @param value    parameter matched against {@link eu.europa.empl.skpt.data.domain.esco.EscoValue#value}
     * @return a page of entities
     */
    @Query("SELECT DISTINCT e FROM EscoLocalConcept e JOIN e.values ev WHERE ev.value LIKE concat('%', :value, '%')")
    Page<EscoLocalConcept> findByValueIgnoreCase(@Param("value") String value, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link EscoLocalConcept} entities filtered by {@link EscoClassification} meeting the paging
     * restriction provided in the {@code Pageable} object.
     *
     * @param classification the ESCO classification
     * @param pageable       the pagination parameters
     * @return a page of entities
     */
    Page<EscoLocalConcept> findByClassification(EscoClassification classification, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link EscoLocalConcept} entities filtered by {@link EscoClassification} meeting the paging
     * restriction provided in the {@code Pageable} object.
     *
     * @param classification the ESCO classification
     * @param pageable       the pagination parameters
     * @param value          parameter matched against {@link eu.europa.empl.skpt.data.domain.esco.EscoValue#value}
     * @return a page of entities
     */
    @Query("SELECT DISTINCT e FROM EscoLocalConcept e JOIN e.values ev WHERE e.classification = :classification"
            + " AND ev.value LIKE concat('%', :value, '%')")
    Page<EscoLocalConcept> findByClassificationAndValueIgnoreCase(@Param("classification") EscoClassification classification,
            @Param("value") String value, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link EscoLocalConcept} entities filtered by {@link EscoClassification} and language meeting the
     * paging restriction provided in the {@code Pageable} object.
     *
     * @param classification the ESCO classification
     * @param language       the language
     * @param pageable       the pagination parameters
     * @return a page of entities
     */
    @Query("SELECT DISTINCT e FROM EscoLocalConcept e JOIN e.values ev WHERE e.classification = :classification"
            + " AND ev.language = :language")
    Page<EscoLocalConcept> findByClassificationAndLanguage(@Param("classification") EscoClassification classification,
            @Param("language") String language, Pageable pageable);


    /**
     * Returns a {@link Page} of {@link EscoLocalConcept} entities filtered by {@link EscoClassification} and language meeting the
     * paging restriction provided in the {@code Pageable} object.
     *
     * @param classification the ESCO classification
     * @param languages      the languages for which the value will be matched against
     * @param pageable       the pagination parameters
     * @param value          parameter matched against {@link eu.europa.empl.skpt.data.domain.esco.EscoValue#value}
     * @return a page of entities
     */
    @Query("SELECT DISTINCT e FROM EscoLocalConcept e JOIN e.values ev WHERE e.classification = :classification "
            + "AND ev.language IN :langs AND ev.value LIKE concat('%', :value, '%')")
    Page<EscoLocalConcept> findByClassificationAndValueIgnoreCaseAndLanguages(@Param("classification") EscoClassification classification,
            @Param("langs") Set<String> languages,
            @Param("value") String value, Pageable pageable);
}
