/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.data.repository.language;

import eu.europa.empl.skpt.data.domain.language.Language;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.repository.query.Param;

import java.util.Optional;
import java.util.Set;

/**
 * Repository retrieving {@link Language} entities
 *
 * @author ARHS Spikeseed
 */
public interface ILanguageRepository extends PagingAndSortingRepository<Language, Long> {

    /**
     * Returns a {@link Page} of {@link Language} entities meeting the paging restriction provided in the
     * {@code Pageable} object.
     *
     * @param language  the search language
     * @param pageable  the pagination parameters
     * @return a page of entities
     */
    @Query("SELECT DISTINCT l FROM Language l JOIN l.values lv WHERE lv.language = :language")
    Page<Language> findByLanguage(@Param("language") String language, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link Language} entities  containing the specify languages,
     * translated in the specified translation languages.
     * @param languagesToSearch The set of languages to search.
     * @param pageable the pagination parameters
     * @return a page of entities
     */
    Page<Language> findByIsoCode1In(Set<String> languagesToSearch, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link Language} entities meeting the paging restriction provided in the
     * {@code Pageable} object.
     *
     * @param pageable the pagination parameters
     * @param value    parameter matched against {@link eu.europa.empl.skpt.data.domain.language.LanguageValue#value}
     * @return a page of entities
     */
    @Query("SELECT DISTINCT l FROM Language l JOIN l.values lv WHERE lv.value LIKE concat('%', :value, '%')")
    Page<Language> findByValueIgnoreCase(@Param("value") String value, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link Language} entities meeting the paging restriction provided in the
     * {@code Pageable} object.
     *
     * @param value         parameter matched against {@link eu.europa.empl.skpt.data.domain.language.LanguageValue#value}
     * @param language      the primary language
     * @param pageable      the pagination parameters
     * @return a page of entities
     */
    @Query("SELECT lv.language "
            + "FROM LanguageValue lv "
            + "WHERE lv.value LIKE concat('%', :value, '%') "
            + "AND lv.language = :language ")
    Page<Language> findByValueIgnoreCaseAndLanguages(@Param("language") Set<String> language,
                                                     @Param("value") String value,
                                                     Pageable pageable);

    /**
     * Returns a {@link Page} of {@link Language} entities filtered by {@link Language} ISO 639-1 code.
     *
     * @param code the ISO 639-1 language code
     * @return a page of entities
     */
    Optional<Language> findByIsoCode1(String code);

    /**
     * Returns a {@link Page} of {@link Language} entities filtered by {@link Language} ISO 639-1 code meeting the
     * paging restriction provided in the {@code Pageable} object.
     *
     * @param code     the ISO 639-2 language code
     * @param pageable the pagination parameters
     * @return a page of entities
     */
    Page<Language> findByIsoCode2(String code, Pageable pageable);

    /**
     * Returns a {@link Page} of {@link Language} entities filtered by {@link Language} ISO 639-1 code meeting the
     * paging restriction provided in the {@code Pageable} object.
     *
     * @param code     the ISO 639-3 language code
     * @param pageable the pagination parameters
     * @return a page of entities
     */
    Page<Language> findByIsoCode3(String code, Pageable pageable);

    /**
     * Returns the primary supported languages
     * @param pageable the pagination parameters
     * @param primarySupported true or false
     * @return a page of entities
     */
    Page<Language> findByPrimarySupported(Boolean primarySupported, Pageable pageable);

    /**
     * Returns the secondary supported languages
     * @param pageable the pagination parameters
     * @param secondarySupported true or false
     * @return a page of entities
     */
    Page<Language> findBySecondarySupported(Boolean secondarySupported, Pageable pageable);
}
