package eu.europa.empl.skpt.data.serializer;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import eu.europa.empl.skpt.data.domain.stats.ProfileStatistic;
import eu.europa.empl.skpt.service.country.ICountryService;
import eu.europa.empl.skpt.service.esco.IEscoApiService;
import eu.europa.empl.skpt.service.isced.IIscedFoetService;
import eu.europa.empl.skpt.service.label.ILabelService;
import eu.europa.empl.skpt.service.language.ILanguageService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.stream.Collectors;

/**
 * The {@link ProfileStatistic} deserializer class.
 *
 * @author ARHS Spikeseed.
 */
@Component
public class ProfileStatisticDeserializer extends StdDeserializer<ProfileStatistic> {

    /**
     * English language value
     */
    public static final String EN_LANG = "en";
    /**
     * Skill classification
     */
    public static final String SKILL = "SKILL";
    /**
     * Occupation classification
     */
    public static final String OCCUPATION = "OCCUPATION";
    @Autowired
    private ILabelService labelService;
    @Autowired
    private ICountryService countryService;
    @Autowired
    private ILanguageService languageService;
    @Autowired
    private IEscoApiService escoApiService;
    @Autowired
    private IIscedFoetService iscedFoetService;

    /**
     * {@link ProfileStatisticDeserializer} constructor calling parent constructor.
     */
    protected ProfileStatisticDeserializer() {
        super(ProfileStatistic.class);
    }

    private Calendar extractDateNode(JsonNode dateNode) {
        if (dateNode == null || "{}".equals(dateNode.toString()) || !hasYear(dateNode)) {
            return null;
        }

        Calendar cal = Calendar.getInstance();
        cal.set(dateNode.get("year").asInt(), Calendar.JANUARY, 1);
        if (hasDay(dateNode)) {
            cal.set(Calendar.DAY_OF_MONTH, dateNode.get("day").asInt());
        }
        if (hasMonth(dateNode)) {
            cal.set(Calendar.MONTH, dateNode.get("month").asInt());
        }
        return cal;
    }

    private boolean hasDay(JsonNode dateNode) {
        return !"".equals(dateNode.get("day").textValue());
    }

    private boolean hasMonth(JsonNode dateNode) {
        return !"".equals(dateNode.get("month").textValue());
    }

    private boolean hasYear(JsonNode dateNode) {
        return !"".equals(dateNode.get("year").textValue());
    }

    private String getLabel(String labelKey) {
        if (StringUtils.isNotEmpty(labelKey)) {
            return this.labelService.getLabelTranslationByKey(labelKey, EN_LANG)
                    .orElse(labelKey);
        }
        return "";
    }

    private String getPrefixedNodeLabel(JsonNode nodeToTranslate, String prefixKey) {
        return getPrefixedAndSuffixedNodeLabel(nodeToTranslate, prefixKey, "");
    }

    private String getPrefixedAndSuffixedNodeLabel(JsonNode nodeToTranslate, String prefixKey, String suffixKey) {
        String key = "";
        if (nodeToTranslate != null && !nodeToTranslate.isNull() && !nodeToTranslate.textValue().isEmpty()) {
            if (StringUtils.isNotEmpty(prefixKey)) {
                prefixKey = prefixKey + ".";
            }
            if (StringUtils.isNotEmpty(suffixKey)) {
                suffixKey = "." + suffixKey;
            }
            key = prefixKey + nodeToTranslate.textValue() + suffixKey;
        }
        return getLabel(key);
    }

    private String getCountryNodeLabel(JsonNode nodeToTranslate) {
        if (nodeToTranslate != null && !nodeToTranslate.isNull()) {
            return this.countryService.findByAlphaCode3(nodeToTranslate.textValue(), new String[] { EN_LANG })
                    .map(c -> c.getValues().get(EN_LANG).getValue())
                    .orElse(nodeToTranslate.textValue());
        }
        return "";
    }

    private String getLanguageNodeLabel(JsonNode nodeToTranslate) {
        if (nodeToTranslate != null && !nodeToTranslate.isNull()) {
            return this.languageService.findByIsoCode1(nodeToTranslate.textValue(), new String[] { EN_LANG })
                    .map(c -> c.getValues().get(EN_LANG).getValue())
                    .orElse(nodeToTranslate.textValue());
        }
        return "";
    }

    private String getIscedNodeLabel(JsonNode nodeToTranslate) {
        if (nodeToTranslate != null && !nodeToTranslate.isNull()) {
            return this.iscedFoetService.findById(nodeToTranslate.asLong(), new String[] { EN_LANG })
                    .map(c -> c.getValues().get(EN_LANG).getValue())
                    .orElse(nodeToTranslate.textValue());
        }
        return "";
    }

    private String getEscoNodeLabel(JsonNode nodeToTranslate, String classification) {
        if (nodeToTranslate != null && !nodeToTranslate.isNull()) {
            return this.escoApiService.findById(nodeToTranslate.textValue(), classification)
                    .map(c -> c.getPreferredLabel().get(EN_LANG))
                    .orElse(nodeToTranslate.textValue());
        }
        return "";
    }

    /**
     * Define the deserialization rule for the {@link ProfileStatistic} and return the {@link ProfileStatistic} instance.
     *
     * @param jsonParser             the {@link JsonParser} used for the deserialization
     * @param deserializationContext the {@link DeserializationContext} used fr the deserialization.
     * @return the deserialized {@link ProfileStatistic}
     * @throws IOException             thrown in case of deserialization error.
     */
    @Override public ProfileStatistic deserialize(JsonParser jsonParser, DeserializationContext deserializationContext)
            throws IOException {
        JsonNode profileNode = jsonParser.getCodec().readTree(jsonParser);
        ProfileStatistic.ProfileStatisticBuilder builder = new ProfileStatistic.ProfileStatisticBuilder();

        JsonNode personalInfoNode = profileNode.get("personalInfo");
        builder.withGender(getPrefixedNodeLabel(personalInfoNode.get("gender"), "profile.general.gender"))
                .withDateOfBirth(extractDateNode(personalInfoNode.get("birthDate")))
                .withArrivalDate(extractDateNode(personalInfoNode.get("dateOfArrival")))
                .withHostCountry(getCountryNodeLabel(personalInfoNode.get("contactInfo").get("country")));

        String firstNationality = getCountryNodeLabel(personalInfoNode.get("nationality"));
        if (StringUtils.isNotEmpty(firstNationality)) {
            List<String> nationalities = new ArrayList<>();
            nationalities.add(firstNationality);
            JsonNode otherNationalitiesNode = personalInfoNode.get("nationalities");
            if (otherNationalitiesNode != null) {
                nationalities.addAll(otherNationalitiesNode.findValues("name")
                        .stream()
                        .map(this::getCountryNodeLabel)
                        .collect(Collectors.toList())
                );
            }
            builder.withNationalitiesCountry(nationalities);
        }

        JsonNode expectationsNode = profileNode.get("expectations");
        List<String> expectations = expectationsNode.findValues("doNext")
                .stream()
                .map(expCode -> getPrefixedAndSuffixedNodeLabel(expCode, "profile.expectations", "label"))
                .collect(Collectors.toList());
        builder.withExpectations(expectations);

        JsonNode skillsIdentificationNode = profileNode.get("skillsIdentification");
        JsonNode languageSkillsNode = skillsIdentificationNode.get("languageSkills");
        String motherTongue = getLanguageNodeLabel(languageSkillsNode.get("motherTongue"));
        if (StringUtils.isNotEmpty(motherTongue)) {
            List<String> languages = new ArrayList<>();
            languages.add(motherTongue);
            JsonNode otherMontherTonguesNode = languageSkillsNode.get("otherMotherTongue");
            if (otherMontherTonguesNode != null) {
                languages.addAll(otherMontherTonguesNode.findValues("name")
                        .stream()
                        .map(this::getLanguageNodeLabel)
                        .collect(Collectors.toList())
                );
            }
            builder.withLanguages(languages);
        }

        JsonNode educationTrainingNode = skillsIdentificationNode.get("educationTraining");
        String firstStudyField = getIscedNodeLabel(educationTrainingNode.get("studyField"));
        List<String> studyFields = new ArrayList<>();
        if (StringUtils.isNotEmpty(firstStudyField)) {
            studyFields.add(firstStudyField);
            JsonNode studyFieldsNode = educationTrainingNode.get("studyFields");
            if (studyFieldsNode != null) {
                studyFields.addAll(studyFieldsNode.findValues("name")
                        .stream()
                        .map(this::getIscedNodeLabel)
                        .collect(Collectors.toList())
                );
            }
        }
        Boolean otherStudyFields = educationTrainingNode.get("otherField") == null
                ? false : educationTrainingNode.get("otherField").booleanValue();
        if (otherStudyFields) {
            JsonNode otherStudyFieldsNode = educationTrainingNode.get("otherStudyFields");
            if (otherStudyFieldsNode != null) {
                studyFields.addAll(otherStudyFieldsNode.findValues("name")
                        .stream()
                        .map(this::getIscedNodeLabel)
                        .collect(Collectors.toList())
                );
            }
        }
        builder.withFieldsOfStudy(studyFields);

        builder.withEducationLevel(getPrefixedNodeLabel(educationTrainingNode.get("highestLevel"),
                "profile.skills_id.education_training.level"));

        JsonNode professionalSkillsNode = skillsIdentificationNode.get("professionalSkills");
        JsonNode experiencesNode = professionalSkillsNode.get("experiences");
        List<String> occupations = experiencesNode.findValues("occupation")
                .stream()
                .map(expCode -> getEscoNodeLabel(expCode, OCCUPATION))
                .collect(Collectors.toList());
        builder.withEscoOccupations(occupations);

        List<String> skills = experiencesNode.findValues("skill")
                .stream()
                .map(expCode -> getEscoNodeLabel(expCode, SKILL))
                .collect(Collectors.toList());
        List<JsonNode> otherSkills = experiencesNode.findValues("otherSkills");
        skills.addAll(
                otherSkills.stream()
                        .flatMap(s -> s.findValues("name").stream())
                        .map(labelCode -> getEscoNodeLabel(labelCode, SKILL))
                        .collect(Collectors.toList()));
        builder.withEscoSkills(skills);

        JsonNode outcomeNode = profileNode.get("outcome");
        JsonNode organisationsNode = outcomeNode.get("organisations");
        List<String> organisationsType = organisationsNode.findValues("type")
                .stream()
                .map(orgType -> getPrefixedNodeLabel(orgType, "profile.outcome.details_issuing_organisation.type"))
                .collect(Collectors.toList());
        builder.withOrganizaionTypes(organisationsType);
        return builder.build();
    }
}
