/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 12/4/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.attachment.impl;

import eu.europa.empl.skpt.common.Error;
import eu.europa.empl.skpt.config.IApplicationProperties;
import eu.europa.empl.skpt.data.domain.attachment.Attachment;
import eu.europa.empl.skpt.data.repository.attachment.IAttachmentRepository;
import eu.europa.empl.skpt.service.attachment.AttachmentSaveResult;
import eu.europa.empl.skpt.service.attachment.IAttachmentService;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.time.Instant;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Stream;

/**
 * Default implementation of {@link IAttachmentService}.
 *
 * @author ARHS Spikeseed
 */
@Service
public class AttachmentService implements IAttachmentService {

    private static final Logger LOGGER = LoggerFactory.getLogger(AttachmentService.class);
    private static final String ID_CANNOT_BE_NULL = "'id' cannot be null";

    private final IAttachmentRepository attachmentRepository;
    private final IApplicationProperties applicationProperties;

    /**
     * Constructor
     *
     * @param attachmentRepository  attachment repository
     * @param applicationProperties application properties
     */
    @Autowired
    public AttachmentService(IAttachmentRepository attachmentRepository, IApplicationProperties applicationProperties) {
        this.attachmentRepository = attachmentRepository;
        this.applicationProperties = applicationProperties;
    }

    // CLOVER:OFF

    @Override
    public Optional<Attachment> findById(Long id) {
        Assert.notNull(id, ID_CANNOT_BE_NULL);
        return attachmentRepository.findById(id);
    }

    @Override
    public Optional<Attachment> findByIdentifier(String id) {
        Assert.notNull(id, ID_CANNOT_BE_NULL);
        return attachmentRepository.findByIdentifier(id);
    }

    // CLOVER:ON

    @Override
    public AttachmentSaveResult save(InputStream is, String name, String type, long size) {
        Assert.notNull(is, "'is' cannot be null");
        Assert.hasText(name, "'name' cannot be null or empty");
        Assert.hasText(type, "'type' cannot be null or empty");
        Assert.isTrue(size >= 0, "'size' must be a positive integer");

        String extension = StringUtils.substringAfterLast(name, ".");

        List<Error> errors = validateAttachment(type, extension, size);
        if (!errors.isEmpty()) {
            return new AttachmentSaveResult(errors);
        }

        String identifier = newIdentifier();
        String path = newPath(identifier, extension);
        File pathOnDisk = new File(applicationProperties.getAttachmentDir(), path);

        try {
            FileUtils.forceMkdirParent(pathOnDisk);
            try (OutputStream os = new FileOutputStream(pathOnDisk)) {
                IOUtils.copy(is, os);
            }
        } catch (IOException e) {
            // CLOVER:OFF
            LOGGER.error(e.getMessage());
            throw new RuntimeException(e);
            // CLOVER:ON
        }

        Attachment attachment = new Attachment();

        attachment.setIdentifier(identifier);
        attachment.setName(name);
        attachment.setSize(size);
        attachment.setMediaType(type);
        attachment.setPath(path);
        attachment.setCreatedDate(Instant.now());
        attachment.setUploadTimestamp(Instant.now());

        return new AttachmentSaveResult(attachmentRepository.save(attachment));

    }

    private List<Error> validateAttachment(String type, String extension, long size) {

        List<Error> errors = new ArrayList<>();

        if (type == null || !applicationProperties.getAttachmentAcceptedMediaTypes().contains(type)) {
            errors.add(Error.ATTACHMENT_WRONG_MEDIA_TYPE);
        }

        if (extension == null || !applicationProperties.getAttachmentAcceptedExtensions().contains(extension.toLowerCase())) {
            errors.add(Error.ATTACHMENT_WRONG_EXTENSION);
        }

        if (size > applicationProperties.getAttachmentMaxSize()) {
            errors.add(Error.ATTACHMENT_EXCEEDS_MAX_SIZE);
        }

        return errors;
    }

    private String newIdentifier() {
        return UUID.randomUUID().toString();
    }

    private String newPath(String identifier, String extension) {
        StringBuilder sb = new StringBuilder();
        sb.append(LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));
        sb.append(File.separatorChar);
        sb.append(identifier);
        sb.append('.');
        sb.append(extension);
        return sb.toString();
    }

    @Override
    public InputStream read(String id) {
        Assert.notNull(id, ID_CANNOT_BE_NULL);

        Attachment attachment = attachmentRepository.findByIdentifier(id)
                .orElseThrow(() -> new IllegalArgumentException(String.format("An attachment with id [%s] cannot be found", id)));

        File pathOnDisk = new File(applicationProperties.getAttachmentDir(), attachment.getPath());

        try {
            return new FileInputStream(pathOnDisk);
        } catch (FileNotFoundException e) {
            LOGGER.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

    @Override
    @Scheduled(cron = "${skpt.attachment.cron}")
    @Transactional
    public void removeOutdatedAttachments() {

        LOGGER.debug("Cron job triggered: removeOutdatedAttachments");

        try (Stream<Attachment> attachmentStream = attachmentRepository.findByUploadTimestampLessThan(
                Instant.now().minusSeconds(applicationProperties.getAttachmentWindow()))) {
            attachmentStream.forEach(this::delete);
        }

        LOGGER.debug("Cron job ended: removeOutdatedAttachments");
    }

    private void delete(Attachment attachment) {
        File pathOnDisk = new File(applicationProperties.getAttachmentDir(), attachment.getPath());

        try {
            FileUtils.forceDelete(pathOnDisk);
            LOGGER.debug("File deleted [{}]", pathOnDisk);
        } catch (FileNotFoundException e) {
            // file has already been deleted
            LOGGER.warn("Unable to find the following file on disk [{}]. Ignoring exception and considering file as deleted.", pathOnDisk,
                    e);
        } catch (IOException e) {
            // CLOVER:OFF
            LOGGER.error("Unable to delete file on disk [{}]", pathOnDisk, e);
            return; // return without deleting attachment from DB
            // CLOVER:ON
        }

        attachmentRepository.delete(attachment);

    }

}
