/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 17/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export;

import eu.europa.empl.skpt.service.export.model.ParseJsonResult;
import eu.europa.empl.skpt.service.export.model.profile.Profile;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URISyntaxException;
import java.time.LocalDate;

/**
 * Service handling logic for the export feature.
 *
 * @author ARHS Spikeseed
 */
public interface IExportService {

    /**
     * Returns the file name of a saved profile, i.e. LASTNAME_FIRSTNAME_YYYYMMDD.extension
     *
     * @param lastName  the last name of the interviewee, can be null
     * @param firstName the first name of the interviewee, can be null
     * @param date      the date at which the profile was created
     * @param extension the file extension
     * @return the file name
     */
    String getFileName(String lastName, String firstName, LocalDate date, String extension);

    /**
     * Transforms the profile into an XML that includes attachment encoded in base64.
     *
     * @param os      the output stream to write the XML to
     * @param profile the profile
     */
    void toXmlWithAttachments(OutputStream os, Profile profile);

    /**
     * Transforms the profile into WORD
     *
     * @param os        the output stream to write the WORD to
     * @param profile   the profile
     * @param lang      the language to use to print out the WORD
     */
    void toWordXML(OutputStream os, Profile profile, String lang);

    /**
     * Transforms the profile into PDF
     *
     * @param os       the output stream to write the PDF to
     * @param profile  the profile
     * @param lang     the language to use to print out the PDF
     * @param outXslfo the flag to indicate the XML file containing the XSLFO tags of the transformation should be returned or not
     */
    void toPdf(OutputStream os, Profile profile, String lang, boolean outXslfo);

    /**
     * Save the profile's picture
     *
     * @param imagePath path of the picture destination
     * @param id        id of the picture
     */
    void saveProfilePicture(String imagePath, String id);

    /**
     * Creates the docx file
     *
     * @param byteArrayOutputStream Stream used to store the docx file
     * @param fileName  Name of the docx file
     * @param profile   Profile containing the data
     * @param lang      Language to use
     * @param pictureId Id of the picture to display
     * @throws IOException        Thrown when accessing IO
     * @throws URISyntaxException Thrown when accessing URI
     */
    void createDocxFile(ByteArrayOutputStream byteArrayOutputStream, String fileName, Profile profile, String lang,
                        String pictureId) throws IOException, URISyntaxException;
    /**
     * Parses the JSON profile.
     *
     * @param profileJson the profile as JSON
     * @return the parsed profile
     */
    ParseJsonResult parseJson(String profileJson);

    /**
     * Create the xlsx file
     *
     * @param byteArrayOutputStream Stream used to store the docx file
     * @param profile profile
     * @param language Language to use
     * @throws IOException Thrown when accessing IO
     */
    void createXlsxFile(ByteArrayOutputStream byteArrayOutputStream, Profile profile, String language)
            throws IOException;
}
