/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export;

import eu.europa.empl.skpt.common.Error;
import eu.europa.empl.skpt.data.domain.attachment.Attachment;
import eu.europa.empl.skpt.data.domain.country.CountryValue;
import eu.europa.empl.skpt.data.domain.isced.IscedFoetValue;
import eu.europa.empl.skpt.data.domain.language.LanguageValue;
import eu.europa.empl.skpt.data.domain.nace.NaceValue;
import eu.europa.empl.skpt.service.attachment.AttachmentSaveResult;
import eu.europa.empl.skpt.service.attachment.IAttachmentService;
import eu.europa.empl.skpt.service.country.ICountryService;
import eu.europa.empl.skpt.service.esco.IEscoApiService;
import eu.europa.empl.skpt.service.isced.IIscedFoetService;
import eu.europa.empl.skpt.service.label.ILabelService;
import eu.europa.empl.skpt.service.language.ILanguageService;
import eu.europa.empl.skpt.service.nace.INaceService;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Optional;

/**
 * Helper class exposing methods accessible from an XSL document.
 *
 * @author ARHS Spikeseed
 */
@Service
@Transactional
public class XslDataService {

    private static final Logger LOGGER = LoggerFactory.getLogger(XslDataService.class);

    private static final String EMPTY_STRING = "";
    private final ICountryService countryService;
    private final IEscoApiService escoService;
    private final IIscedFoetService iscedFoetService;
    private final ILanguageService languageService;
    private final INaceService naceService;
    private final ILabelService labelService;
    private final IAttachmentService attachmentService;

    /**
     * Constructor
     *
     * @param countryService    country service
     * @param escoService       ESCO service
     * @param iscedFoetService  ISCED FOET service
     * @param languageService   language service
     * @param naceService       NACE service
     * @param labelService      label service
     * @param attachmentService attachment service
     */
    @Autowired
    public XslDataService(ICountryService countryService, IEscoApiService escoService, IIscedFoetService iscedFoetService,
            ILanguageService languageService, INaceService naceService, ILabelService labelService, IAttachmentService attachmentService) {
        this.countryService = countryService;
        this.escoService = escoService;
        this.iscedFoetService = iscedFoetService;
        this.languageService = languageService;
        this.naceService = naceService;
        this.labelService = labelService;
        this.attachmentService = attachmentService;
    }

    /**
     * Returns the translation of the label identified by the specified key
     *
     * @param key  the key identifying the label
     * @param lang the language to return the translation in
     * @return the translation of the label identified by the specified key
     */
    public String getLabel(String key, String lang) {
        if (StringUtils.isBlank(key) || StringUtils.isBlank(lang)) {
            return EMPTY_STRING;
        }

        return this.labelService.getLabelTranslationByKey(key, lang).orElse(EMPTY_STRING);
    }

    /**
     * Returns the country label
     *
     * @param code the country code
     * @param lang the language code
     * @return the country label
     */
    public String getCountryLabel(String code, String lang) {
        if (StringUtils.isBlank(code) || StringUtils.isBlank(lang)) {
            return EMPTY_STRING;
        }

        return this.countryService.findByAlphaCode3(code, new String[] { lang })
                .map(c -> {
                    CountryValue countryValue = c.getValues().get(lang);
                    if (countryValue != null) {
                        return countryValue.getValue();
                    }
                    return null;
                })
                .orElse(EMPTY_STRING);
    }

    /**
     * Returns the ESCO label
     *
     * @param id                the ESCO id
     * @param classification    the classification of the concept (i.e. SKILL, OCCUPATION...)
     * @param lang              the language code
     * @return the ESCO label
     */
    public String getEscoLabel(String id, String classification, String lang) {
        if (StringUtils.isBlank(id) || StringUtils.isBlank(lang)) {
            return EMPTY_STRING;
        }

        return this.escoService.findById(id, classification)
                .map(c -> {
                    String preferredLabel = c.getPreferredLabel().get(lang);
                    if (preferredLabel != null) {
                        return preferredLabel;
                    }
                    return null;
                })
                .orElse(EMPTY_STRING);
    }

    /**
     * Returns the ISCED FOET label
     *
     * @param id   the ISCED FOET id
     * @param lang the language code
     * @return the ISCED FOET label
     */
    public String getIscedFoetLabel(String id, String lang) {
        if (StringUtils.isBlank(id) || StringUtils.isBlank(lang)) {
            return EMPTY_STRING;
        }

        return this.iscedFoetService.findById(Long.valueOf(id), new String[] { lang })
                .map(c -> {
                    IscedFoetValue iscedFoetValue = c.getValues().get(lang);
                    if (iscedFoetValue != null) {
                        return iscedFoetValue.getValue();
                    }
                    return null;
                })
                .orElse(EMPTY_STRING);
    }

    /**
     * Returns the language label
     *
     * @param code the language code
     * @param lang the language code
     * @return the language label
     */
    public String getLanguageLabel(String code, String lang) {
        if (StringUtils.isBlank(code) || StringUtils.isBlank(lang)) {
            return EMPTY_STRING;
        }

        return this.languageService.findByIsoCode1(code, new String[] { lang })
                .map(c -> {
                    LanguageValue languageValue = c.getValues().get(lang);
                    if (languageValue != null) {
                        return languageValue.getValue();
                    }
                    return null;
                })
                .orElse(EMPTY_STRING);
    }

    /**
     * Returns the NACE label
     *
     * @param id   the NACE id
     * @param lang the language code
     * @return the NACE label
     */
    public String getNaceLabel(String id, String lang) {
        if (StringUtils.isBlank(id) || StringUtils.isBlank(lang)) {
            return EMPTY_STRING;
        }

        return this.naceService.findById(Long.valueOf(id), new String[] { lang })
                .map(c -> {
                    NaceValue naceValue = c.getValues().get(lang);
                    if (naceValue != null) {
                        return naceValue.getValue();
                    }
                    return null;
                })
                .orElse(EMPTY_STRING);
    }

    /**
     * Returns an attachment encoded in base64
     *
     * @param id the unique identifier
     * @return an attachment encoded in base64
     */
    public String getAttachmentAsBase64(String id) {
        Optional<Attachment> attachment = attachmentService.findByIdentifier(id);

        if (attachment.isPresent()) {
            // Using RCF 2045 to encode in Base64, see https://www.w3.org/TR/xmlschema-2/#base64Binary
            try (InputStream is = attachmentService.read(id);
                 ByteArrayOutputStream os = new ByteArrayOutputStream();
                 OutputStream osBase64 = Base64.getMimeEncoder().wrap(os)) {
                IOUtils.copy(is, osBase64);
                return os.toString(StandardCharsets.UTF_8.name());
            } catch (IOException e) {
                // CLOVER:OFF
                LOGGER.error(e.getMessage());
                throw new RuntimeException(e);
                // CLOVER:ON
            }
        }

        return null;
    }

    /**
     * Creates a new attachment and returns its ID.
     *
     * @param name the name of the attachment to create
     * @param type the MIME type of the attachment
     * @param size the size of the attachment
     * @param data the content of the attachment encoded in base64
     * @return the unique identifier of the newly created attachment
     */
    public String createAttachmentFromBase64(String name, String type, long size, String data) {
        // Using RCF 2045 to decode from Base64, see https://www.w3.org/TR/xmlschema-2/#base64Binary
        try (InputStream is = Base64.getMimeDecoder().wrap(new ByteArrayInputStream(data.getBytes(StandardCharsets.UTF_8)))) {

            AttachmentSaveResult saveResult = attachmentService.save(is, name, type, size);

            if (saveResult.hasErrors()) {
                Error error = saveResult.getErrors().get(0);
                String message = "Cannot create attachment: " + error.getId() + " - " + error.getDescription();
                LOGGER.error(message);
                throw new RuntimeException(message);
            }

            return saveResult.getAttachment().orElseThrow(() -> new RuntimeException("No attachment created")).getIdentifier();

        } catch (IOException e) {
            LOGGER.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }
}
