/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 17/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export.impl;

import eu.europa.empl.skpt.service.attachment.IAttachmentService;
import eu.europa.empl.skpt.service.export.IExportService;
import eu.europa.empl.skpt.service.export.IJsonParserService;
import eu.europa.empl.skpt.service.export.ITransformerService;
import eu.europa.empl.skpt.service.export.model.ExcelHelper;
import eu.europa.empl.skpt.service.export.model.ParseJsonResult;
import eu.europa.empl.skpt.service.export.model.profile.Profile;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.fop.apps.FOPException;
import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;
import org.apache.poi.ooxml.POIXMLProperties;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.*;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Comparator;
import java.util.UUID;
import java.util.stream.Stream;

/**
 * Default implementation of {@link eu.europa.empl.skpt.service.export.IExportService}
 *
 * @author ARHS Spikeseed
 */
@Service
public class ExportService implements IExportService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportService.class);

    @Value("${export.excel.creator:EU Skills Profile Tool}")
    private String excelCreator;

    private static final String XSD_RESOURCE_PATH = "/eu/europa/empl/skpt/profile.xsd";
    private static final String XSL_PDF_RESOURCE_PATH = "/eu/europa/empl/skpt/pdf.xsl";
    private static final String XSL_WORD_RESOURCE_PATH = "/eu/europa/empl/skpt/word.xsl";
    private static final String XSL_ATTACHMENT_RESOURCE_PATH = "/eu/europa/empl/skpt/attachment.xsl";
    private static final String OS_CANNOT_BE_NULL = "'os' cannot be null";
    private static final String PROFILE_CANNOT_BE_NULL = "'profile' cannot be null";
    private static final String DOCUMENT_XML = "document.xml";
    private static final String IMAGE = "word" + File.separator + "media" + File.separator + "image1.png";

    private final String tempDir;

    private final IAttachmentService attachmentService;
    private final FopFactory fopFactory;
    private final ZipDocumentService zipDocumentService;
    private final IJsonParserService jsonParserService;
    private final Marshaller marshaller;
    private final ITransformerService transformerService;
    private final ExcelHelper excelHelper;

    /**
     * Constructor
     *
     * @param attachmentService  attachment service
     * @param zipDocumentService zip document service
     * @param tempDir            path to tempDir
     * @param jsonParserService  json parser service
     * @param fopFactory         fop factory
     * @param marshaller         marshaller
     * @param transformerService transformer service
     * @param excelHelper excel helper
     */
    @Autowired
    public ExportService(IAttachmentService attachmentService,
                         ZipDocumentService zipDocumentService, @Value("${export.word.temp.dir}") String tempDir,
                         IJsonParserService jsonParserService, FopFactory fopFactory,
                         Marshaller marshaller, ITransformerService transformerService,
                         ExcelHelper excelHelper) {
        this.attachmentService = attachmentService;
        this.jsonParserService = jsonParserService;
        this.marshaller = marshaller;
        this.transformerService = transformerService;
        this.fopFactory = fopFactory;
        this.zipDocumentService = zipDocumentService;
        this.tempDir = tempDir;
        this.excelHelper = excelHelper;
    }

    @Override
    public String getFileName(String lastName, String firstName, LocalDate date, String extension) {
        Assert.notNull(date, "'date' cannot be null");
        Assert.hasText(extension, "'extension' cannot be null or blank");

        StringBuilder fileName = new StringBuilder();

        if (StringUtils.isNotBlank(lastName)) {
            fileName.append(lastName);
        }

        if (StringUtils.isNotBlank(firstName)) {
            if (fileName.length() > 0) {
                fileName.append('_');
            }
            fileName.append(firstName);
        }

        if (fileName.length() > 0) {
            fileName.append('_');
        }
        fileName.append(date.format(DateTimeFormatter.ofPattern("YYYYMMdd")));
        fileName.append('.');
        fileName.append(extension);

        return fileName.toString();
    }

    private void toXml(OutputStream os, Profile profile) {
        Assert.notNull(os, OS_CANNOT_BE_NULL);
        Assert.notNull(profile, PROFILE_CANNOT_BE_NULL);

        try {
            marshaller.marshal(profile, os);
        } catch (JAXBException e) {
            LOGGER.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

    @Override
    public void toXmlWithAttachments(OutputStream os, Profile profile) {
        Assert.notNull(os, OS_CANNOT_BE_NULL);
        xsltTransformProfile(new StreamResult(os), profile, "", XSL_ATTACHMENT_RESOURCE_PATH, true);
    }

    @Override
    public void toWordXML(OutputStream os, Profile profile, String lang) {
        Assert.notNull(os, OS_CANNOT_BE_NULL);
        Assert.hasText(lang, "'lang' cannot be null nor empty");
        xsltTransformProfile(new StreamResult(os), profile, lang, XSL_WORD_RESOURCE_PATH, true);
    }

    private void xsltTransformProfile(Result res, Profile profile, String lang, String xsltSheetName, boolean prettyPrint) {
        Assert.notNull(profile, PROFILE_CANNOT_BE_NULL);

        Source xslt = new StreamSource(getClass().getResource(xsltSheetName).toString());
        try {
            transformerService.transform(profile, xslt, lang, prettyPrint, res);
        } catch (IOException | JAXBException | TransformerException e) {
            LOGGER.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

    @Override
    public void toPdf(OutputStream os, Profile profile, String lang, boolean printXMLFo) {
        Assert.notNull(os, OS_CANNOT_BE_NULL);
        Assert.hasText(lang, "'lang' cannot be null nor empty");
        try {
            Fop fop = fopFactory.newFop(MimeConstants.MIME_PDF, os);
            Result res = getXMLResult(os, fop, printXMLFo);
            xsltTransformProfile(res, profile, lang, XSL_PDF_RESOURCE_PATH, printXMLFo);
        } catch (FOPException e) {
            LOGGER.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

    @Override
    public void saveProfilePicture(String imagePath, String id) {
        if (id != null) {
            try (OutputStream stream = new FileOutputStream(imagePath)) {
                byte[] data = IOUtils.toByteArray(attachmentService.read(id));
                stream.write(data);
            } catch (Exception e) {
                // CLOVER:OFF
                LOGGER.error(e.getMessage());
                // CLOVER:ON
            }
        }
    }

    /**
     * Create the right Result implementation depending on the outXslfo param. If true the transformation will produce a PDF file,
     * otherwise an XML file containing the XSLFO tags of the transformation.
     *
     * @param out        the {@link OutputStream} that will contains the transformation result
     * @param fop        the {@link Fop} factory that will operates the transformation
     * @param printXMLFo the flag that indicate if the transformation should produce XSLFO or PDF.
     * @return the {@link Result} implementation that will be used for the transformation.
     * @throws FOPException exception thrown if setting up the DefaultHandler fails
     */
    public Result getXMLResult(OutputStream out, Fop fop, boolean printXMLFo) throws FOPException {
        return printXMLFo ? new StreamResult(out) : new SAXResult(fop.getDefaultHandler());
    }

    @Override
    public void createDocxFile(ByteArrayOutputStream byteArrayOutputStream, String fileName, Profile profile, String lang,
            String pictureId) throws IOException {

        String profileDirectory = this.tempDir + File.separator
                + "profile_" + System.currentTimeMillis() + "_" + UUID.randomUUID();
        String profilePath = profileDirectory + File.separator + "word" + File.separator + DOCUMENT_XML;

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {

            FileUtils.copyDirectory(
                    new File(getClass().getResource("/word").toURI()),
                    new File(profileDirectory));

            // generate the document.xml
            toWordXML(os, profile, lang);

            Path documentXml = Paths.get(profilePath);
            Files.write(documentXml, os.toByteArray());

            saveProfilePicture(profileDirectory + File.separator + IMAGE, pictureId);

            // generate the zip file
            zipDocumentService.zipDocument(byteArrayOutputStream, Paths.get(profileDirectory));
        } catch (IOException | URISyntaxException e) {
            LOGGER.error("Error creating docx", e);
        } finally {
            try (Stream<Path> walk = Files.walk(Paths.get(profileDirectory))) {
                walk.sorted(Comparator.reverseOrder())
                        .map(Path::toFile)
                        .forEach(File::delete);
            }
        }
    }

    @Override
    public ParseJsonResult parseJson(String profileJson) {
        return jsonParserService.parseJson(profileJson);
    }

    @Override
    public void createXlsxFile(ByteArrayOutputStream byteArrayOutputStream, Profile profile, String language)
            throws IOException {
        XSSFWorkbook workbook = new XSSFWorkbook();

        workbook = profile.toExcel(excelHelper, workbook, language);

        for (Sheet sheet : workbook) {
            for (int i = 0; i < sheet.getRow(1).getLastCellNum(); i++) {
                sheet.autoSizeColumn(i);
            }
        }

        POIXMLProperties poixmlProperties = workbook.getProperties();
        POIXMLProperties.CoreProperties coreProperties = poixmlProperties.getCoreProperties();
        coreProperties.setCreator(excelCreator);
        POIXMLProperties.ExtendedProperties extendedProperties = poixmlProperties.getExtendedProperties();
        extendedProperties.getUnderlyingProperties().setCompany(excelCreator);

        workbook.write(byteArrayOutputStream);
        workbook.close();
    }

}
