/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 26/06/2018
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export.impl;

import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.util.JAXBSource;
import javax.xml.transform.*;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import eu.europa.empl.skpt.service.export.ITransformerService;
import eu.europa.empl.skpt.service.export.XslDataService;
import eu.europa.empl.skpt.service.export.model.profile.Profile;

/**
 * Default implementation of {@link ITransformerService}
 *
 * @author ARHS Spikeseed
 */
@Service
public class TransformerService implements ITransformerService {
    private static final String DATA_SERVICE = "dataService";
    private final CustomErrorListener errorListener = new CustomErrorListener();

    private final TransformerFactory transformerFactory;
    private final XslDataService xslDataService;
    private final Marshaller marshaller;

    /**
     * Transforms the xml result according to the xslt
     *
     * @param transformerFactory transformer factory
     * @param xslDataService     xsl dataservice
     * @param marshaller         marshaller
     */
    @Autowired
    public TransformerService(TransformerFactory transformerFactory, XslDataService xslDataService, Marshaller marshaller) {
        this.transformerFactory = transformerFactory;
        this.xslDataService = xslDataService;
        this.marshaller = marshaller;
    }

    @Override
    public void transform(Profile profile, Source xslt, String lang, boolean prettyPrint, Result res)
            throws TransformerException, JAXBException {

        Source src = new JAXBSource(marshaller, profile);
        transform(src, xslt, lang, prettyPrint, res);
    }

    private void transform(Source xmlSource, Source xslt, String lang, boolean prettyPrint, Result res) throws TransformerException {
        Transformer transformer = transformerFactory.newTransformer(xslt);
        transformer.setParameter(DATA_SERVICE, xslDataService);
        if (StringUtils.isNotEmpty(lang)) {
            transformer.setParameter("lang", lang);
        }
        if (prettyPrint) {
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
            transformer.setOutputProperty(OutputKeys.STANDALONE, "yes");
        }
        transformer.setErrorListener(errorListener);
        transformer.transform(xmlSource, res);
    }

    @Override
    public void transform(StreamSource streamSource, StreamResult streamResult, Source xsltSource) throws TransformerException {
        transform(streamSource, xsltSource, null, false, streamResult);
    }

    private static class CustomErrorListener implements ErrorListener {
        @Override
        public void warning(TransformerException exception) throws TransformerException {
            // No exception on warning.
        }

        @Override
        public void error(TransformerException exception) throws TransformerException {
            throw exception;
        }

        @Override
        public void fatalError(TransformerException exception) throws TransformerException {
            throw exception;
        }
    }
}