/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 6/9/2019
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export.model;

import eu.europa.empl.skpt.data.domain.IReferenceData;
import eu.europa.empl.skpt.data.domain.country.Country;
import eu.europa.empl.skpt.data.domain.country.CountryValue;
import eu.europa.empl.skpt.data.domain.esco.EscoApiConcept;
import eu.europa.empl.skpt.data.domain.isced.IscedFoet;
import eu.europa.empl.skpt.data.domain.isced.IscedFoetValue;
import eu.europa.empl.skpt.data.domain.language.Language;
import eu.europa.empl.skpt.data.domain.language.LanguageValue;
import eu.europa.empl.skpt.data.domain.nace.Nace;
import eu.europa.empl.skpt.data.domain.nace.NaceValue;
import eu.europa.empl.skpt.service.country.ICountryService;
import eu.europa.empl.skpt.service.esco.IEscoApiService;
import eu.europa.empl.skpt.service.isced.IIscedFoetService;
import eu.europa.empl.skpt.service.label.ILabelService;
import eu.europa.empl.skpt.service.language.ILanguageService;
import eu.europa.empl.skpt.service.nace.INaceService;
import java.util.Map;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * Excel helper
 *
 * @author ARHS Spikeseed
 */
@Component
public class ExcelHelper {
  
  private ILabelService labelService;
  private ICountryService countryService;
  private ILanguageService languageService;
  private INaceService naceService;
  private IEscoApiService escoService;
  private IIscedFoetService iscedFoetService;
  
  private CellStyle cellStyle;
  @Value("#{${excelSheetNameCharacterReplaceMap : {'/':'-' }}}")
  private Map<String, String> charReplaceMap;
  
  /**
   * Constructor.
   *
   * @param labelService label service
   * @param countryService country service
   * @param languageService language service
   * @param naceService nace service
   * @param escoService esco service
   * @param iscedFoetService isced foet service
   */
  @Autowired
  public ExcelHelper(
      ILabelService labelService,
      ICountryService countryService,
      ILanguageService languageService,
      INaceService naceService,
      IEscoApiService escoService,
      IIscedFoetService iscedFoetService) {
    this.labelService = labelService;
    this.countryService = countryService;
    this.languageService = languageService;
    this.naceService = naceService;
    this.escoService = escoService;
    this.iscedFoetService = iscedFoetService;
  }
  
  /**
   * Returns the translation of the key in the language (english by default).
   *
   * @param key key
   * @param language language
   * @param type type of translation
   * @return the translation
   */
  @SuppressWarnings("squid:S128")
  public String getTranslation(String key, String language, ExcelTranslationTypesEnum type) {
    if (StringUtils.isNotEmpty(key) && StringUtils.isNotEmpty(language)) {
      switch (type) {
        case YES_NO:
        case LEVEL_DIGITAL:
          key = LabelKey.getKeyById(key);
        case LABEL:
          return labelService.getLabelTranslationByKey(key, language, true).orElse(key);
        case COUNTRY:
          Optional<Country> optionalCountry = countryService.findByAlphaCode3(key, new String[]{language}, true);
          if (optionalCountry.isPresent()) {
            return ((CountryValue) extractValueInEnglishByDefault(optionalCountry.get(), language)).getValue();
          }
        case LANGUAGE:
          Optional<Language> optionalLanguage = languageService.findByIsoCode1(key, new String[]{language}, true);
          if (optionalLanguage.isPresent()) {
            return ((LanguageValue) extractValueInEnglishByDefault(optionalLanguage.get(), language)).getValue();
          }
        case NACE:
          if (NumberUtils.isCreatable(key)) {
            Optional<Nace> optionalNace = naceService.findById(Long.parseLong(key), new String[]{language}, true);
            if (optionalNace.isPresent()) {
              return ((NaceValue) extractValueInEnglishByDefault(optionalNace.get(), language)).getValue();
            }
          }
          break;
        case ESCO_OCCUPATION:
          Optional<EscoApiConcept> optionalEscoApiConceptOccupation = escoService.findById(key, "OCCUPATION");
          if (optionalEscoApiConceptOccupation.isPresent()) {
            return extractEscoApiConceptValyeInEngishByDefault(optionalEscoApiConceptOccupation.get(), language);
          }
        case ESCO_SKILL:
          Optional<EscoApiConcept> optionalEscoApiConceptSkill = escoService.findById(key, "SKILL");
          if (optionalEscoApiConceptSkill.isPresent()) {
            return extractEscoApiConceptValyeInEngishByDefault(optionalEscoApiConceptSkill.get(), language);
          }
        case ISCED_FOET:
          if (NumberUtils.isCreatable(key)) {
            Optional<IscedFoet> optionalIscedFoet =
                iscedFoetService.findById(Long.parseLong(key), new String[]{language}, true);
            if (optionalIscedFoet.isPresent()) {
              return ((IscedFoetValue) extractValueInEnglishByDefault(optionalIscedFoet.get(), language)).getValue();
            }
          }
      }
    }
    return key;
  }
  
  private Object extractValueInEnglishByDefault(IReferenceData iReferenceData, String language) {
    return iReferenceData.getValues().containsKey(language)
           ? iReferenceData.getValues().get(language) : iReferenceData.getValues().get("en");
  }
  
  private String extractEscoApiConceptValyeInEngishByDefault(EscoApiConcept escoApiConcept, String language) {
    return escoApiConcept.getPreferredLabel().containsKey(language)
           ? escoApiConcept.getPreferredLabel().get(language) : escoApiConcept.getPreferredLabel().get("en");
  }
  
  /**
   * Returns the translation of the key in the language (english by default).
   *
   * @param key key
   * @param language language
   * @return the translation
   */
  public String getTranslation(String key, String language) {
    return getTranslation(key, language, ExcelTranslationTypesEnum.LABEL);
  }
  
  /**
   * Returns the translation of the key in the language (english by default).
   *
   * @param key key
   * @param language language
   * @return the translation
   */
  public String getTranslation(Boolean key, String language) {
    return key == null ? "" : getTranslation(String.valueOf(key), language, ExcelTranslationTypesEnum.YES_NO);
  }
  
  /**
   * Create all the rows for the sheet.
   *
   * @param sheet sheet
   * @param sectionIndex index of the section
   * @param sectionName name of the section
   * @return a list of rows
   */
  public List<XSSFRow> createRows(XSSFSheet sheet, int sectionIndex, String sectionName) {
    return createRows(sheet, sectionIndex, sectionName, null);
  }
  
  /**
   * Create all the rows for the sheet.
   *
   * @param sheet sheet
   * @param sectionIndex index of the section
   * @param sectionName name of the section
   * @param subsectionName name of the subsection
   * @return a list of rows
   */
  public List<XSSFRow> createRows(XSSFSheet sheet, int sectionIndex, String sectionName, String subsectionName) {
    // row 0 : x. section name - subsection name
    // row 1 : information name
    // row 2 : profile's row
    List<XSSFRow> rows = new ArrayList<>();
    for (int i = 0; i < 3; i++) {
      XSSFRow row = sheet.createRow(i);
      row.setHeight((short) -1);
      rows.add(row);
    }
    
    // merge cells of the first row
    sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, 10));
    
    if (subsectionName == null) {
      createCell(0, rows.get(0), sectionIndex + ". " + sectionName);
    }
    else {
      createCell(0, rows.get(0), sectionIndex + ". " + sectionName + " - " + subsectionName);
    }
    
    return rows;
  }
  
  /**
   * Create a new sheet.
   *
   * @param workbook workbook
   * @param sectionIndex index of the section
   * @param subSectionName sub section name
   * @return the created sheet
   */
  public XSSFSheet createSheet(XSSFWorkbook workbook, int sectionIndex, String subSectionName) {
    cellStyle = workbook.createCellStyle();
    cellStyle.setWrapText(true);
    
    // translate the sheet name and create the sheet
    return workbook.createSheet(sectionIndex + ". " + cleanUpSheetName(subSectionName));
  }
  
  private String cleanUpSheetName(String subSectionName) {
    for (String charToReplace : charReplaceMap.keySet()) {
      subSectionName = subSectionName.replaceAll(charToReplace, charReplaceMap.get(charToReplace));
    }
    return subSectionName;
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param language language
   * @param headerKey key value of the header cell
   * @param primaryData primary data
   * @param secondaryData secondary data
   * @param isEmail is email
   */
  public void createColumn(
      int columnIndex, List<XSSFRow> rows, String language, String headerKey, String primaryData, List<IExcelCell> secondaryData,
      boolean isEmail) {
    createColumn(columnIndex, rows, language, headerKey, primaryData, secondaryData, ExcelTranslationTypesEnum.LABEL, isEmail);
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param language language
   * @param headerKey key value of the header cell
   * @param primaryData primary data
   * @param secondaryData secondary data
   */
  public void createColumn(
      int columnIndex, List<XSSFRow> rows, String language, String headerKey, String primaryData, List<IExcelCell> secondaryData) {
    createColumn(columnIndex, rows, language, headerKey, primaryData, secondaryData, ExcelTranslationTypesEnum.LABEL, false);
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param language language
   * @param headerKey key value of the header cell
   * @param primaryData primary data
   * @param secondaryData secondary data
   * @param translationType translation type
   */
  public void createColumn(
      int columnIndex, List<XSSFRow> rows, String language, String headerKey, String primaryData, List<IExcelCell> secondaryData,
      ExcelTranslationTypesEnum translationType) {
    createColumn(columnIndex, rows, language, headerKey, primaryData, secondaryData, translationType, false);
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param language language
   * @param headerKey key value of the header cell
   * @param data data
   */
  public void createColumn(int columnIndex, List<XSSFRow> rows, String language, String headerKey, Object data) {
    createColumn(columnIndex, rows, language, headerKey, data != null ? String.valueOf(data) : null, null,
                 ExcelTranslationTypesEnum.LABEL, false);
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param language language
   * @param headerKey key value of the header cell
   * @param data data
   * @param translationType translation type
   */
  public void createColumn(
      int columnIndex, List<XSSFRow> rows, String language, String headerKey, Object data,
      ExcelTranslationTypesEnum translationType) {
    createColumn(columnIndex, rows, language, headerKey, data != null ? String.valueOf(data) : null, null, translationType, false);
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param language language
   * @param headerKey key value of the header cell
   * @param primaryData primary data
   * @param secondaryData secondary data
   * @param translationType translation type
   * @param isEmail is email
   */
  @SuppressWarnings("squid:S00107")
  public void createColumn(
      int columnIndex,
      List<XSSFRow> rows,
      String language,
      String headerKey,
      String primaryData,
      List<IExcelCell> secondaryData,
      ExcelTranslationTypesEnum translationType,
      boolean isEmail) {
    
    StringBuilder stringBuilder = new StringBuilder();
    if (primaryData != null) {
      stringBuilder.append(
          getTranslation(primaryData, language, translationType));
    }
    if (secondaryData != null) {
      for (IExcelCell excelCell : secondaryData) {
        if (excelCell.toExcelString(isEmail) != null) {
          stringBuilder.append(", ").append(
              getTranslation(excelCell.toExcelString(isEmail), language, translationType));
        }
      }
    }
    createColumn(columnIndex, rows, getTranslation(headerKey, language).replace("<br />", "\n"), stringBuilder.toString());
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param header value of the header cell
   */
  public void createColumn(int columnIndex, List<XSSFRow> rows, String header) {
    createColumn(columnIndex, rows, header, "");
  }
  
  /**
   * Create a new column in the sheet.
   *
   * @param columnIndex index of the column
   * @param rows rows of the sheet
   * @param header value of the header cell
   * @param data value of the profile cell
   */
  public void createColumn(int columnIndex, List<XSSFRow> rows, String header, String data) {
    createCell(columnIndex, rows.get(1), header);
    createCell(columnIndex, rows.get(2), data);
  }
  
  /**
   * Create a new cell in the row.
   *
   * @param columnIndex index of the column
   * @param row row
   * @param value value
   */
  private void createCell(int columnIndex, XSSFRow row, String value) {
    XSSFCell cell = row.createCell(columnIndex);
    cell.setCellValue(value);
    cell.setCellStyle(cellStyle);
  }
  
  /**
   * Add a new line of data in the last cell of the row
   *
   * @param rows rows
   * @param value value
   */
  public void addNewLineToLastCell(List<XSSFRow> rows, String value) {
    XSSFRow row = rows.get(rows.size() - 1);
    XSSFCell cell = row.getCell(row.getLastCellNum() - 1);
    cell.setCellValue(cell.getStringCellValue() + "\n" + value);
  }
  
  @Getter
  private enum LabelKey {
    
    TRUE("true", "common.yes"),
    FALSE("false", "common.no"),
    LABEL_0("0", "profile.skills_id.digital_skills.rating.none"),
    LABEL_1("1", "profile.skills_id.digital_skills.rating.guidance"),
    LABEL_2("2", "profile.skills_id.digital_skills.rating.beginner"),
    LABEL_3("3", "profile.skills_id.digital_skills.rating.intermediate"),
    LABEL_4("4", "profile.skills_id.digital_skills.rating.expert");
    
    private String id;
    private String key;
    
    LabelKey(String id, String key) {
      this.id = id;
      this.key = key;
    }
    
    public static String getKeyById(String numberId) {
      for (ExcelHelper.LabelKey labelKey : values()) {
        if (labelKey.getId().equals(numberId)) {
          return labelKey.getKey();
        }
      }
      return numberId;
    }
    
  }
  
}
