/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 2/05/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.service.export.model.profile;

import eu.europa.empl.skpt.service.export.model.ExcelHelper;
import eu.europa.empl.skpt.service.export.model.ExcelTranslationTypesEnum;
import eu.europa.empl.skpt.service.export.model.IExcelCell;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import javax.xml.bind.annotation.XmlType;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

// CLOVER:OFF

/**
 * Education and Training
 *
 * @author ARHS Spikeseed
 */
@XmlType(propOrder = {
        "attended",
        "last",
        "fullTimeWeek",
        "fullTimeMonth",
        "fullTimeYear",
        "partTimeWeek",
        "partTimeMonth",
        "partTimeYear",
        "highestLevel",
        "otherExplanation",
        "studyField",
        "studyFields",
        "otherField",
        "otherStudyFields",
        "studyFieldWeek",
        "studyFieldMonth",
        "studyFieldYear",
        "educationType",
        "priorLearning",
        "certificate",
        "certificates"
})
@Getter
@Setter
public class EducationTraining {

    private Boolean attended;
    private String last;
    private int fullTimeWeek;
    private int fullTimeMonth;
    private int fullTimeYear;
    private int partTimeWeek;
    private int partTimeMonth;
    private int partTimeYear;
    private String highestLevel;
    private String otherExplanation;
    private String studyField;
    private List<Name> studyFields;
    private Boolean otherField;
    private List<Name> otherStudyFields;
    private int studyFieldWeek;
    private int studyFieldMonth;
    private int studyFieldYear;
    private EducationType educationType;
    private Boolean priorLearning;
    private Boolean certificate;
    private List<Certificate> certificates;

    public List<Name> getStudyFields() {
        return studyFields == null ? Collections.emptyList() : Collections.unmodifiableList(studyFields);
    }

    public List<Certificate> getCertificates() {
        return certificates == null ? Collections.emptyList() : Collections.unmodifiableList(certificates);
    }

    public List<Name> getOtherStudyFields() {
        return otherStudyFields == null ? Collections.emptyList() : Collections.unmodifiableList(otherStudyFields);
    }

    /**
     * Adapt the class to generate an excel file.
     *
     * @param excelHelper excel helper
     * @param workbook excel workbook
     * @param sectionIndex section index
     * @param sectionName section name
     * @param language language
     * @return true if the section literacy skills is displayed
     */
    public boolean toExcel(ExcelHelper excelHelper, XSSFWorkbook workbook, int sectionIndex, String sectionName, String language) {
        String subsectionName = excelHelper.getTranslation("profile.skills_id.education_training.section.name", language);
        XSSFSheet sheet = excelHelper.createSheet(workbook, sectionIndex, subsectionName);

        List<XSSFRow> rows = excelHelper.createRows(sheet, sectionIndex, sectionName, subsectionName);

        int columnIndex = 0;
        excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.attended.label",
                attended, ExcelTranslationTypesEnum.YES_NO);
        if (BooleanUtils.isTrue(attended)) {
            excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.last.label",
                    last);
            String weeksTranslation = excelHelper.getTranslation(
                    "profile.skills_id.professional_skills.professional_experience.duration.week", language);
            String monthsTranslation = excelHelper.getTranslation(
                    "profile.skills_id.professional_skills.professional_experience.duration.month", language);
            String yearsTranslation = excelHelper.getTranslation(
                    "profile.skills_id.professional_skills.professional_experience.duration.year", language);
            String attendFullTime = fullTimeWeek + " " + weeksTranslation + " " + fullTimeMonth + " " + monthsTranslation
                    + " " + fullTimeYear + " " + yearsTranslation;
            excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.full_time.label",
                    attendFullTime);
            String attendPartTime = partTimeWeek + " " + weeksTranslation + " " + partTimeMonth + " " + monthsTranslation
                    + " " + partTimeYear + " " + yearsTranslation;
            excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.part_time.label",
                    attendPartTime);
            excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.highest_level.label",
                    highestLevel != null && !highestLevel.isEmpty() ? "profile.skills_id.education_training.level." + highestLevel : "");

            if (!"primary_education".equals(highestLevel)) {
                List<Name> excelStudyFields = new ArrayList<>(studyFields);
                excelStudyFields.addAll(otherStudyFields);
                String excelStudyFieldsDuration = studyFieldWeek + " " + weeksTranslation + " " + studyFieldMonth + " " + monthsTranslation
                        + " " + studyFieldYear + " " + yearsTranslation;
                excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.study_field.label",
                        studyField, excelStudyFields.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()),
                        ExcelTranslationTypesEnum.ISCED_FOET);
                excelHelper.addNewLineToLastCell(rows, excelStudyFieldsDuration);
                excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.education_type.label",
                        educationType.toExcel(excelHelper, language));
                excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.prior_learning.label",
                        priorLearning, ExcelTranslationTypesEnum.YES_NO);
                excelHelper.createColumn(columnIndex++, rows, language, "profile.skills_id.education_training.certificate.label",
                        certificate, ExcelTranslationTypesEnum.YES_NO);

                if (BooleanUtils.isTrue(certificate) && certificates != null) {
                    for (int i = 0; i < certificates.size(); i++) {
                        excelHelper.createColumn(columnIndex++, rows,
                                excelHelper.getTranslation("profile.skills_id.education_training.certificate.section.name", language)
                                        + " " + (i + 1));
                        excelHelper.createColumn(columnIndex++, rows, language,
                                "profile.skills_id.education_training.certificate.title.label",
                                certificates.get(i).getTitle());
                        excelHelper.createColumn(columnIndex++, rows, language,
                                "profile.skills_id.education_training.certificate.name.label",
                                certificates.get(i).getName());
                        excelHelper.createColumn(columnIndex++, rows, language,
                                "profile.skills_id.education_training.certificate.country.label",
                                certificates.get(i).getCountry(), ExcelTranslationTypesEnum.COUNTRY);
                        excelHelper.createColumn(columnIndex++, rows, language,
                                "profile.skills_id.education_training.certificate.year.label",
                                certificates.get(i).getYear());
                    }
                }
            }
        }

        return attended == null || !attended || highestLevel == null || highestLevel.isEmpty() || "primary_education".equals(highestLevel);
    }

}
