/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 20/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export.model.profile;

import eu.europa.empl.skpt.service.export.model.ExcelHelper;
import eu.europa.empl.skpt.service.export.model.ExcelTranslationTypesEnum;
import eu.europa.empl.skpt.service.export.model.IExcelCell;
import lombok.Getter;
import lombok.Setter;
import org.apache.poi.xssf.usermodel.XSSFRow;

import javax.xml.bind.annotation.XmlType;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

// CLOVER:OFF

/**
 * Expectation
 *
 * @author ARHS Spikeseed
 */
@XmlType(propOrder = {
        "doNext",
        "language",
        "otherImproveLanguage",
        "undertakeEducation",
        "undertakeEducationTimeCommitment",
        "educationType",
        "otherType",
        "undertakeEducationStudyField",
        "otherUndertakeEducationStudyFields",
        "otherStudyField",
        "otherExpectedStudyFields",
        "findWork",
        "findWorkTimeCommitment",
        "activityField",
        "otherActivityFields",
        "occupation",
        "otherOccupations",
        "otherTimeCommitment",
        "other"
})
@Getter
@Setter
public class Expectation {

    private String doNext;
    private String language;
    private List<Name> otherImproveLanguage;
    private String undertakeEducation;
    private String undertakeEducationTimeCommitment;
    private String educationType;
    private String otherType;
    private String undertakeEducationStudyField;
    private List<Name> otherUndertakeEducationStudyFields;
    private Boolean otherStudyField;
    private List<Name> otherExpectedStudyFields;
    private String findWork;
    private String findWorkTimeCommitment;
    private String activityField;
    private List<Name> otherActivityFields;
    private String occupation;
    private List<Name> otherOccupations;
    private String otherTimeCommitment;
    private String other;

    public List<Name> getOtherImproveLanguage() {
        return otherImproveLanguage == null ? Collections.emptyList() : Collections.unmodifiableList(otherImproveLanguage);
    }
    public List<Name> getOtherUndertakeEducationStudyFields() {
        return otherUndertakeEducationStudyFields == null
                ? Collections.emptyList() : Collections.unmodifiableList(otherUndertakeEducationStudyFields);
    }

    public List<Name> getOtherActivityFields() {
        return otherActivityFields == null ? Collections.emptyList() : Collections.unmodifiableList(otherActivityFields);
    }

    public List<Name> getOtherOccupations() {
        return otherOccupations == null ? Collections.emptyList() : Collections.unmodifiableList(otherOccupations);
    }

    public List<Name> getOtherExpectedStudyFields() {
        return otherExpectedStudyFields == null ? Collections.emptyList() : Collections.unmodifiableList(otherExpectedStudyFields);
    }

    /**
     * Adapt the class to generate an excel file.
     *
     * @param excelHelper excel helper
     * @param columnIndex column index
     * @param rows rows
     * @param expectationIndex expectation index
     * @param language language
     * @return the column index
     */
    public int toExcel(ExcelHelper excelHelper, int columnIndex, List<XSSFRow> rows, int expectationIndex, String language) {
        excelHelper.createColumn(columnIndex++, rows,
                excelHelper.getTranslation("profile.expectations.sub_section.title", language) + " " + expectationIndex);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.do_next.label",
                "profile.expectations." + doNext + ".label");

        columnIndex = buildImproveLanguageExpectation(excelHelper, columnIndex, rows, language);
        columnIndex = buildUndertakeEducationExpectation(excelHelper, columnIndex, rows, language);
        columnIndex = buildFindWorkExpectation(excelHelper, columnIndex, rows, language);
        columnIndex = buildOtherExpectation(excelHelper, columnIndex, rows, language);

        return columnIndex;
    }

    private int buildImproveLanguageExpectation(
            ExcelHelper excelHelper, int columnIndex, List<XSSFRow> rows, String language) {
        if ("improve_language".equals(doNext)) {
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.improve_language.rowLabel",
                    this.language, otherImproveLanguage.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()),
                    ExcelTranslationTypesEnum.LANGUAGE);
        }
        return columnIndex;
    }

    private int buildUndertakeEducationExpectation(
            ExcelHelper excelHelper, int columnIndex, List<XSSFRow> rows, String language) {
        if ("undertake_education".equals(doNext)) {
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.undertake_education.time_commitment.label",
                    undertakeEducationTimeCommitment != null && !undertakeEducationTimeCommitment.isEmpty()
                            ? "profile.expectations." + undertakeEducationTimeCommitment : "");
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.undertake_education.education_type.label",
                    educationType != null && !educationType.isEmpty()
                            ? "profile.expectations.undertake_education.education_type." + educationType : "");

            if ("other".equals(educationType)) {
                excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.undertake_education.education_type.other",
                        otherType);
            }

            List<Name> excelOtherUndertakeEducationStudyFields = new ArrayList<>();
            excelOtherUndertakeEducationStudyFields.addAll(otherUndertakeEducationStudyFields);
            excelOtherUndertakeEducationStudyFields.addAll(otherExpectedStudyFields);
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.undertake_education.study_field.label",
                    undertakeEducationStudyField,
                    excelOtherUndertakeEducationStudyFields.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()),
                    ExcelTranslationTypesEnum.ISCED_FOET);
        }
        return columnIndex;
    }

    private int buildFindWorkExpectation(
            ExcelHelper excelHelper, int columnIndex, List<XSSFRow> rows, String language) {
        if ("find_work".equals(doNext)) {
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.find_work.rowLabel",
                    findWork != null && !findWork.isEmpty()
                            ? "profile.expectations.work." + findWork : "");
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.undertake_education.time_commitment.label",
                    findWorkTimeCommitment != null && !findWorkTimeCommitment.isEmpty()
                            ? "profile.expectations." + findWorkTimeCommitment : "");
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.find_work.activity_field.label",
                    activityField, otherActivityFields.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()),
                    ExcelTranslationTypesEnum.NACE);
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.find_work.occupation.label",
                    occupation, otherOccupations.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()),
                    ExcelTranslationTypesEnum.ESCO_OCCUPATION);
        }
        return columnIndex;
    }

    private int buildOtherExpectation(
            ExcelHelper excelHelper, int columnIndex, List<XSSFRow> rows, String language) {
        if ("other".equals(doNext)) {
            excelHelper.createColumn(columnIndex++, rows, language, "profile.expectations.other.label",
                    other);
        }
        return columnIndex;
    }

}
