/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 17/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export.model.profile;

import eu.europa.empl.skpt.service.export.model.ExcelHelper;
import eu.europa.empl.skpt.service.export.model.ExcelTranslationTypesEnum;
import eu.europa.empl.skpt.service.export.model.IExcelCell;
import lombok.Getter;
import lombok.Setter;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlType;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

// CLOVER:OFF

/**
 * Personal Info
 *
 * @author ARHS Spikeseed
 */
@XmlType(propOrder = {
        "profilePic",
        "familyName",
        "firstName",
        "firstNames",
        "gender",
        "birthDate",
        "countryOfBirth",
        "nationality",
        "nationalities",
        "maritalStatus",
        "otherMaritalStatuses",
        "identificationDocuments",
        "contactInfo",
        "dateOfArrival",
        "residentialStatus",
        "otherResidencyStatuses",
        "workPermitStatus",
        "travelling",
        "minor15",
        "minor16To18",
        "adult",
        "elderly",
        "sick"

})
@Getter
@Setter
public class PersonalInfo {

    @XmlElement(name = "attachment")
    private Attachment profilePic;

    private String familyName;
    private String firstName;
    private List<Name> firstNames;
    private String gender;
    private Date birthDate;
    private String countryOfBirth;
    private String nationality;
    private List<Name> nationalities;
    private String maritalStatus;
    private List<Name> otherMaritalStatuses;
    private List<PersonalIdentificationDocument> identificationDocuments;

    private ContactInfo contactInfo;

    private Date dateOfArrival;
    private String residentialStatus;
    private List<Name> otherResidencyStatuses;
    private String workPermitStatus;
    private String travelling;
    private int minor15;
    private int minor16To18;
    private int adult;
    private int elderly;
    private int sick;

    public List<Name> getFirstNames() {
        return firstNames == null ? Collections.emptyList() : Collections.unmodifiableList(firstNames);
    }

    public List<Name> getNationalities() {
        return nationalities == null ? Collections.emptyList() : Collections.unmodifiableList(nationalities);
    }

    public List<Name> getOtherMaritalStatuses() {
        return otherMaritalStatuses == null ? Collections.emptyList() : Collections.unmodifiableList(otherMaritalStatuses);
    }

    public List<PersonalIdentificationDocument> getIdentificationDocuments() {
        return identificationDocuments == null ? Collections.emptyList() : Collections.unmodifiableList(identificationDocuments);
    }

    public List<Name> getOtherResidencyStatuses() {
        return otherResidencyStatuses  == null ? Collections.emptyList() : Collections.unmodifiableList(otherResidencyStatuses);
    }

    /**
     * Adapt the class to generate an excel file.
     *
     * @param excelHelper excel helper
     * @param workbook excel workbook
     * @param sectionIndex section index
     * @param language language
     */
    public void toExcel(ExcelHelper excelHelper, XSSFWorkbook workbook, int sectionIndex, String language) {
        String sectionName = excelHelper.getTranslation("step.personal", language);
        generalInfoToExcel(excelHelper, workbook, sectionIndex, sectionName, language);
        contactInfo.toExcel(excelHelper, workbook, sectionIndex, sectionName, language);
        migrationInfoToExcel(excelHelper, workbook, sectionIndex, sectionName, language);
    }

    private void generalInfoToExcel(ExcelHelper excelHelper, XSSFWorkbook workbook, int sectionIndex, String sectionName, String language) {
        String subsectionName = excelHelper.getTranslation("profile.general.section.name", language);
        XSSFSheet sheet = excelHelper.createSheet(workbook, sectionIndex, subsectionName);

        List<XSSFRow> rows = excelHelper.createRows(sheet, sectionIndex, sectionName, subsectionName);

        int columnIndex = 0;
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.name.family",
                familyName);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.name.first",
                firstName, firstNames.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()));
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.gender.label",
                gender != null && !gender.isEmpty() ? "profile.general.gender." + gender : "");
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.birth.date",
                birthDate.toExcelString(false));
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.birth.country",
                countryOfBirth, ExcelTranslationTypesEnum.COUNTRY);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.nationality",
                nationality, nationalities.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()),
                ExcelTranslationTypesEnum.COUNTRY);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.marital.status",
                maritalStatus, otherMaritalStatuses.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()));

        for (int i = 0; i < identificationDocuments.size(); i++) {
            columnIndex = identificationDocuments.get(i).toExcel(excelHelper, columnIndex, rows, i + 1, language);
        }
    }

    private void migrationInfoToExcel(
            ExcelHelper excelHelper, XSSFWorkbook workbook, int sectionIndex, String sectionName, String language) {
        String subsectionName = excelHelper.getTranslation("profile.migration.section.name", language);
        XSSFSheet sheet = excelHelper.createSheet(workbook, sectionIndex, subsectionName);

        List<XSSFRow> rows = excelHelper.createRows(sheet, sectionIndex, sectionName, subsectionName);

        int columnIndex = 0;
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.country.reception_arrival_date",
                dateOfArrival.toExcelString(false));
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.residence.status",
                residentialStatus != null && !residentialStatus.isEmpty() ? "profile.general.residence.statuses." + residentialStatus : "",
                otherResidencyStatuses.stream().map(elm -> (IExcelCell) elm).collect(Collectors.toList()));
        excelHelper.createColumn(columnIndex++, rows, language, "profile.general.work.permit.label",
                workPermitStatus != null && !workPermitStatus.isEmpty() ? "profile.general.work.permit." + workPermitStatus : "");
        excelHelper.createColumn(columnIndex++, rows, language, "profile.personal.migration.travelling.label",
                travelling, ExcelTranslationTypesEnum.YES_NO);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.personal.migration.travelling.minor_15",
                minor15);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.personal.migration.travelling.minor_16_to_18",
                minor16To18);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.personal.migration.travelling.adult",
                adult);
        excelHelper.createColumn(columnIndex++, rows, language, "profile.personal.migration.travelling.elderly",
                elderly);
        excelHelper.createColumn(columnIndex, rows, language, "profile.personal.migration.travelling.sick",
                sick);
    }

}
