/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.label.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import eu.europa.empl.skpt.data.domain.label.Label;
import eu.europa.empl.skpt.data.domain.label.LabelValue;
import eu.europa.empl.skpt.data.repository.label.ILabelRepository;
import eu.europa.empl.skpt.service.label.ILabelService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Stream;

/**
 * Default implementation of {@link eu.europa.empl.skpt.service.label.ILabelService}
 *
 * @author ARHS Spikeseed
 */
@Service
public class LabelService implements ILabelService {

    private static final Logger LOGGER = LoggerFactory.getLogger(LabelService.class);

    private final ILabelRepository labelRepository;
    private final ObjectMapper objectMapper;

    /**
     * Constructor
     *
     * @param labelRepository the label repository
     * @param objectMapper    object mapper
     */
    @Autowired
    public LabelService(ILabelRepository labelRepository, ObjectMapper objectMapper) {
        this.labelRepository = labelRepository;
        this.objectMapper = objectMapper;
    }

    @Override
    @Transactional
    public void asJson(OutputStream os, String lang) {
        Assert.notNull(os, "'os' cannot be null");
        Assert.hasText(lang, "'lang' cannot be null or empty");

        ObjectNode root = objectMapper.createObjectNode();

        try (Stream<Label> labelStream = labelRepository.findAllAsStream()) {
            labelStream.forEach(label -> {
                LabelValue labelValue = label.getValues().get(lang);
                if (labelValue != null) {
                    String[] keys = StringUtils.split(label.getKey(), ".");
                    if (keys != null) {
                        this.buildPathAndNode(root, keys, labelValue);
                    }
                }
            });
            objectMapper.writeValue(os, root);
        } catch (IOException e) {
            LOGGER.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }

    private void buildPathAndNode(ObjectNode root, String[] keys, LabelValue labelValue) {
        ObjectNode currentNode = root;
        StringBuilder path = new StringBuilder();
        for (int i = 0; i < keys.length; i++) {
            String key = keys[i];
            path.append("." + key);

            if (i == keys.length - 1) {
                if (currentNode.has(key) && currentNode.get(key).isObject()) {
                    String message = "The following path is conflicting: " + path;
                    LOGGER.error(message);
                    throw new RuntimeException(message);
                }
                currentNode.put(key, labelValue.getTranslation());
            } else if (currentNode.has(key)) {
                currentNode = (ObjectNode) currentNode.get(key);
            } else {
                currentNode.set(key, objectMapper.createObjectNode());
                currentNode = (ObjectNode) currentNode.get(key);
            }

        }
    }

    @Override
    public Optional<String> getLabelTranslationByKey(String key, String lang, boolean defaultEnglish) {
        Assert.hasText(key, "'key' cannot be null or empty");
        Assert.hasText(lang, "'lang' cannot be null or empty");

        Optional<Label> label = labelRepository.findByKey(key);

        if (label.isPresent()) {
            Map<String, LabelValue> values = label.get().getValues();
            if (defaultEnglish) {
                return values.containsKey(lang)
                        ? Optional.of(values.get(lang).getTranslation()) : Optional.of(values.get("en").getTranslation());
            } else if (values.containsKey(lang) && values.get(lang) != null) {
                return Optional.of(values.get(lang).getTranslation());
            }
        }

        return Optional.empty();
    }

    @Override
    public Optional<String> getLabelTranslationByKey(String key, String lang) {
        return getLabelTranslationByKey(key, lang, false);
    }

}
