/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.service.language.impl;

import eu.europa.empl.skpt.data.domain.language.Language;
import eu.europa.empl.skpt.data.repository.language.ILanguageRepository;
import eu.europa.empl.skpt.service.language.ILanguageService;
import eu.europa.empl.skpt.service.tool.LanguageFilter;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Optional;
import java.util.Set;

/**
 * Implementation of the service for the {@link Language} entity.
 *
 * @author ARHS Spikeseed
 */
@Service
public class LanguageService implements ILanguageService {

    private final ILanguageRepository languageRepository;
    private final LanguageFilter languageFilter;

    /**
     * Contructor.
     *
     * @param languageRepository the {@link Language} repository.
     * @param languageFilter    language filter
     */
    @Autowired
    public LanguageService(ILanguageRepository languageRepository, LanguageFilter languageFilter) {
        this.languageRepository = languageRepository;
        this.languageFilter = languageFilter;
    }

    @Override
    public Page<Language> search(String searchString, Set<String> searchLanguages, String[] resultLanguages, Pageable pageable) {
        if (StringUtils.isNotBlank(searchString)) {
            return getLanguagesForSpecificSearch(searchString, searchLanguages, resultLanguages, pageable);
        }
        return getLanguagesForEmptySearch(searchLanguages, resultLanguages, pageable);
    }

    private Page<Language> getLanguagesForSpecificSearch(
            String searchString, Set<String> searchLanguages, String[] resultLanguages, Pageable pageable) {
        Page<Language> languagesPage;

        if (searchLanguages == null || searchLanguages.isEmpty()) {
            languagesPage = this.languageRepository.findByValueIgnoreCase(searchString, pageable);
        } else {
            languagesPage = this.languageRepository.findByValueIgnoreCaseAndLanguages(searchLanguages, searchString, pageable);
        }
        languagesPage = (Page<Language>) languageFilter.filter(languagesPage, resultLanguages);
        return languagesPage;
    }

    private Page<Language> getLanguagesForEmptySearch(Set<String> searchLanguages, String[] resultLanguages, Pageable pageable) {
        Page<Language> languagesPage;

        if (!CollectionUtils.isEmpty(searchLanguages)) {
            languagesPage = this.languageRepository.findByIsoCode1In(searchLanguages, pageable);
        } else {
            languagesPage = this.languageRepository.findAll(pageable);
        }
        languagesPage = (Page<Language>) languageFilter.filter(languagesPage, resultLanguages);
        return languagesPage;
    }

    @Override
    public Optional<Language> findByIsoCode1(String code, String[] languages, boolean defaultEnglish) {
        Optional<Language> optionalLanguage = this.languageRepository.findByIsoCode1(code);

        if (optionalLanguage.isPresent()) {
            Language language = (Language) languageFilter.filter(optionalLanguage.get(), languages);
            if (defaultEnglish && !language.getValues().containsKey(languages[0])) { // always have only one language for this case
                language = (Language) languageFilter.filter(optionalLanguage.get(), new String[]{"en"});
            }
            optionalLanguage = Optional.of(language);
        }

        return optionalLanguage;
    }

    @Override
    public Optional<Language> findByIsoCode1(String code, String[] languages) {
        return findByIsoCode1(code, languages, false);
    }

    // CLOVER:OFF

    @Override
    public Page<Language> findByPrimarySupported(Pageable pageable) {
        return this.languageRepository.findByPrimarySupported(true, pageable);
    }

    @Override
    public Page<Language> findBySecondarySupported(Pageable pageable) {
        return this.languageRepository.findBySecondarySupported(true, pageable);
    }
}
