package eu.europa.empl.skpt.service.mail.impl;

import eu.europa.empl.skpt.api.exception.NotFoundException;
import eu.europa.empl.skpt.service.mail.IMailService;
import org.bouncycastle.util.encoders.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.mail.MailException;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.mail.javamail.MimeMessagePreparator;
import org.springframework.stereotype.Service;
import org.thymeleaf.util.StringUtils;

import java.util.List;

/**
 * Report service
 *
 * @author ARHS Spikeseed
 */
@Service
public class MailService implements IMailService {

    private static final Logger LOGGER = LoggerFactory.getLogger(MailService.class);

    private final JavaMailSender mailSender;

    /**
     * Constructor
     *
     * @param mailSender mail sender
     */
    @Autowired
    public MailService(JavaMailSender mailSender) {
        this.mailSender = mailSender;
    }

    @Override
    public void sendMail(String to, String subject, String htmlContent, List<MailAttachment> attachments)
            throws RuntimeException {

        if (StringUtils.isEmpty(to)) {
            String errorMessage = "No recipient found in the configurations.";
            LOGGER.error(errorMessage);
            throw new NotFoundException(errorMessage);
        }

        if (StringUtils.isEmpty(subject)) {
            String errorMessage = "No mail subject found in the configurations.";
            LOGGER.error(errorMessage);
            throw new NotFoundException(errorMessage);
        }

        if (StringUtils.isEmpty(htmlContent)) {
            String errorMessage = "No content is passed for the resport mail";
            LOGGER.error(errorMessage);
            throw new NotFoundException(errorMessage);
        }

        MimeMessagePreparator messagePreparator = mimeMessage -> {
            MimeMessageHelper messageHelper = new MimeMessageHelper(mimeMessage, true);
            messageHelper.setTo(to);
            messageHelper.setSubject(subject);

            messageHelper.setText(htmlContent, true);

            for (MailAttachment attachment : attachments) {
                messageHelper.addAttachment(attachment.getFileName(),
                        new ByteArrayResource(Base64.decode(attachment.getBase64Data())));
            }
        };

        try {
            mailSender.send(messagePreparator);
        } catch (MailException e) {
            LOGGER.error("Error when sending report", e);
            throw new IllegalStateException(e);
        }
    }

}
