/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 29/12/2016
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.data.repository.esco;

import eu.europa.empl.skpt.data.domain.esco.EscoClassification;
import eu.europa.empl.skpt.data.domain.esco.EscoLocalConcept;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

/**
 * Test class for {@link eu.europa.empl.skpt.data.repository.esco.IEscoRepository}.
 *
 * @author ARHS Spikeseed
 */
@RunWith(SpringRunner.class)
@DataJpaTest
@ActiveProfiles("unitTest")
@Sql("esco-repository-test.sql")
public class EscoRepositoryTest {

    @Autowired
    private IEscoRepository repository;

    @Test
    public void findById() {

        EscoLocalConcept firstEsco = repository.findAll(PageRequest.of(0, 10)).getContent().stream().findFirst().get();

        Optional<EscoLocalConcept> esco = this.repository.findById(11000L);
        Assert.assertNotNull(esco);
        Assert.assertFalse(esco.isPresent());

        esco = this.repository.findById(firstEsco.getId());
        Assert.assertNotNull(esco);
        Assert.assertTrue(esco.isPresent());
        Assert.assertEquals(firstEsco.getId(), esco.get().getId());

    }

    @Test
    public void findAll() {
        Page<EscoLocalConcept> escoPage = this.repository.findAll(PageRequest.of(0, 10, Sort.Direction.ASC, "id"));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(10, escoPage.getTotalElements());
    }

    @Test
    public void findByValueIgnoreCase() {
        Page<EscoLocalConcept> escoPage = this.repository.findByValueIgnoreCase("economists", PageRequest.of(0, 10, Sort.Direction.ASC, "id"));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(1, escoPage.getContent().size());

        escoPage = this.repository.findByValueIgnoreCase("ECONOMISTS", PageRequest.of(0, 10, Sort.Direction.ASC, "id"));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(1, escoPage.getContent().size());
    }

    @Test
    public void findByClassification() {
        Page<EscoLocalConcept> escoPage = this.repository.findByClassification(EscoClassification.OCCUPATIONS, PageRequest.of(0, 10));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(10, escoPage.getTotalElements());
    }

    @Test
    public void findByClassificationAndValueIgnoreCase() {
        Page<EscoLocalConcept> escoPage = this.repository
                .findByClassificationAndValueIgnoreCase(EscoClassification.OCCUPATIONS, "economists", PageRequest.of(0, 10));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(1, escoPage.getContent().size());

        escoPage = this.repository
                .findByClassificationAndValueIgnoreCase(EscoClassification.OCCUPATIONS, "ECONOMISTS", PageRequest.of(0, 10));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(1, escoPage.getContent().size());
    }

    @Test
    public void findByClassificationAndLanguage() {
        Page<EscoLocalConcept> escoPage = this.repository.findByClassificationAndLanguage(EscoClassification.OCCUPATIONS, "en", PageRequest.of(0, 10));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(10, escoPage.getTotalElements());
    }

    @Test
    public void findByClassificationAndValueIgnoreCaseAndLanguages() {

        Set<String> langs = new HashSet<>();
        langs.add("fr");

        Page<EscoLocalConcept> escoPage = this.repository
                .findByClassificationAndValueIgnoreCaseAndLanguages(EscoClassification.OCCUPATIONS, langs, "Aquaculteurs et ouvriers de l'aquaculture", PageRequest.of(0, 10));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(1, escoPage.getContent().size());

        escoPage = this.repository
                .findByClassificationAndValueIgnoreCaseAndLanguages(EscoClassification.OCCUPATIONS, langs, "Aquaculture workers", PageRequest.of(0, 10));
        Assert.assertNotNull(escoPage);
        Assert.assertEquals(0, escoPage.getContent().size());
    }

}
