/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 13/07/2018
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.data.serializer;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.ObjectMapper;
import eu.europa.empl.skpt.data.domain.country.Country;
import eu.europa.empl.skpt.data.domain.country.CountryValue;
import eu.europa.empl.skpt.data.domain.esco.EscoApiConcept;
import eu.europa.empl.skpt.data.domain.isced.IscedFoet;
import eu.europa.empl.skpt.data.domain.isced.IscedFoetValue;
import eu.europa.empl.skpt.data.domain.language.Language;
import eu.europa.empl.skpt.data.domain.language.LanguageValue;
import eu.europa.empl.skpt.data.domain.stats.ProfileStatistic;
import eu.europa.empl.skpt.service.country.impl.CountryService;
import eu.europa.empl.skpt.service.esco.impl.EscoApiService;
import eu.europa.empl.skpt.service.isced.impl.IscedFoetService;
import eu.europa.empl.skpt.service.label.impl.LabelService;
import eu.europa.empl.skpt.service.language.impl.LanguageService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.when;

/**
 * ProfileStatisticDeserializerTest
 *
 * @author ARHS Spikeseed
 */
@RunWith(MockitoJUnitRunner.class)
public class ProfileStatisticDeserializerTest {

    @Mock
    private LabelService labelService;
    @Mock
    private CountryService countryService;
    @Mock
    private LanguageService languageService;
    @Mock
    private IscedFoetService iscedFoetService;
    @Mock
    private EscoApiService escoApiService;

    @InjectMocks
    private ProfileStatisticDeserializer profileStatisticDeserializer;

    private ObjectMapper objectMapper;

    @Before
    public void init() {
        objectMapper = new ObjectMapper();

        when(labelService.getLabelTranslationByKey("profile.general.gender.male", "en"))
                .thenReturn(Optional.of("Male"));
        when(labelService.getLabelTranslationByKey("profile.expectations.improve_language.label", "en"))
                .thenReturn(Optional.of("Improve language skills"));
        when(labelService.getLabelTranslationByKey("profile.expectations.follow_course.label", "en"))
                .thenReturn(Optional.of("Follow integration course"));
        when(labelService.getLabelTranslationByKey("profile.expectations.undertake_education.label", "en"))
                .thenReturn(Optional.of("Undertake education and / or training"));
        when(labelService.getLabelTranslationByKey("profile.expectations.find_work.label", "en"))
                .thenReturn(Optional.of("Find work"));
        when(labelService.getLabelTranslationByKey("profile.skills_id.education_training.level.higher_education", "en"))
                .thenReturn(Optional.of("Higher Education / Training"));
        when(labelService.getLabelTranslationByKey("profile.outcome.details_issuing_organisation.type.ngo", "en"))
                .thenReturn(Optional.of("NGO"));
        when(labelService.getLabelTranslationByKey("profile.outcome.details_issuing_organisation.type.pes", "en"))
                .thenReturn(Optional.of("PES"));

        when(countryService.findByAlphaCode3("BEL", new String[] { "en" }))
                .thenReturn(Optional.of(generateCountry("BEL", "Belgium")));

        when(languageService.findByIsoCode1("fr", new String[] { "en" }))
                .thenReturn(Optional.of(generateLanguage("fr", "French")));
        when(languageService.findByIsoCode1("en", new String[] { "en" }))
                .thenReturn(Optional.of(generateLanguage("en", "English")));

        when(iscedFoetService.findById(new Long(2), new String[] { "en" }))
                .thenReturn(Optional.of(generateIscedFoet("2", "Literacy and numeracy")));
        when(iscedFoetService.findById(new Long(1), new String[] { "en" }))
                .thenReturn(Optional.of(generateIscedFoet("1", "Basic programmes and qualifications")));
        when(iscedFoetService.findById(new Long(0), new String[] { "en" }))
                .thenReturn(Optional.of(generateIscedFoet("Other field of study", "Other field of study")));

        when(escoApiService.findById("http://data.europa.eu/esco/occupation/4cf7be91-fed9-47a7-9ca9-e74c7eb6becb", "OCCUPATION"))
                .thenReturn(Optional.of(
                        generateEsco("http://data.europa.eu/esco/occupation/4cf7be91-fed9-47a7-9ca9-e74c7eb6becb", "OCCUPATION", "3D printing technician")));
        when(escoApiService.findById("http://data.europa.eu/esco/occupation/50af07f0-7a75-424e-a66d-5a9deea10f4c", "OCCUPATION"))
                .thenReturn(Optional.of(
                        generateEsco("http://data.europa.eu/esco/occupation/50af07f0-7a75-424e-a66d-5a9deea10f4c", "OCCUPATION", "ICT accessibility tester")));
        when(escoApiService.findById("http://data.europa.eu/esco/skill/6e53fd99-b646-4327-9580-ac062ab21188", "SKILL"))
                .thenReturn(Optional.of(generateEsco("http://data.europa.eu/esco/skill/6e53fd99-b646-4327-9580-ac062ab21188", "OCCUPATION", "3D lighting")));
        when(escoApiService.findById("http://data.europa.eu/esco/skill/2afb2b59-c9a3-4cf3-b1dd-1a2fad51e583", "SKILL"))
                .thenReturn(Optional.of(
                        generateEsco("http://data.europa.eu/esco/skill/2afb2b59-c9a3-4cf3-b1dd-1a2fad51e583", "OCCUPATION", "3D printing process")));
        when(escoApiService.findById("http://data.europa.eu/esco/skill/3e7516dc-0f7c-4686-8b2f-fade4e5d21be", "SKILL"))
                .thenReturn(Optional.of(generateEsco("http://data.europa.eu/esco/skill/3e7516dc-0f7c-4686-8b2f-fade4e5d21be", "OCCUPATION", "3D texturing")));
        when(escoApiService.findById("http://data.europa.eu/esco/skill/eb0e5615-1575-4a86-a1a2-7d39595033c5", "SKILL"))
                .thenReturn(Optional.of(generateEsco("http://data.europa.eu/esco/skill/eb0e5615-1575-4a86-a1a2-7d39595033c5", "OCCUPATION", "ABAP")));
    }

    private Country generateCountry(String alphaCode3, String countryEn) {
        Country country = new Country();

        country.setAlphaCode3(alphaCode3);

        CountryValue countryValue = new CountryValue();
        countryValue.setLanguage("en");
        countryValue.setValue(countryEn);
        countryValue.setCountry(country);

        Map<String, CountryValue> map = new HashMap<>();
        map.put("en", countryValue);

        country.setValues(map);

        return country;
    }

    private Language generateLanguage(String isoCode, String languageEn) {
        Language language = new Language();

        language.setIsoCode1(isoCode);

        LanguageValue languageValue = new LanguageValue();
        languageValue.setLanguage("en");
        languageValue.setValue(languageEn);
        languageValue.setLang(language);

        Map<String, LanguageValue> map = new HashMap<>();
        map.put("en", languageValue);

        language.setValues(map);

        return language;
    }

    private IscedFoet generateIscedFoet(String key, String iscedFoetEn) {
        IscedFoet iscedFoet = new IscedFoet();

        iscedFoet.setKey(key);

        IscedFoetValue iscedFoetValue = new IscedFoetValue();
        iscedFoetValue.setLanguage("en");
        iscedFoetValue.setValue(iscedFoetEn);
        iscedFoetValue.setIscedFoet(iscedFoet);

        Map<String, IscedFoetValue> map = new HashMap<>();
        map.put("en", iscedFoetValue);

        iscedFoet.setValues(map);

        return iscedFoet;
    }

    private EscoApiConcept generateEsco(String classId, String classification, String escoEn) {
        EscoApiConcept escoApiConcept = new EscoApiConcept();

        escoApiConcept.setClassName(classification);
        escoApiConcept.setClassId(classId);

        Map<String, String> map = new HashMap<>();
        map.put("en", escoEn);

        escoApiConcept.setPreferredLabel(map);

        return escoApiConcept;
    }

    @Test
    public void deserializeWithAFullProfile() throws IOException {
        String fullProfile = new String(Files.readAllBytes(
                Paths.get("./src/test/resources/eu/europa/empl/skpt/data/serializer/jsonProfile.json")));

        JsonParser jsonParser = objectMapper.getFactory().createParser(fullProfile);
        DeserializationContext deserializationContext = objectMapper.getDeserializationContext();
        ProfileStatistic deserializedProfile = profileStatisticDeserializer.deserialize(jsonParser, deserializationContext);
        Calendar dateOfBirth = Calendar.getInstance();
        dateOfBirth.set(Calendar.DAY_OF_MONTH, 20);
        dateOfBirth.set(Calendar.MONTH, 8);
        dateOfBirth.set(Calendar.YEAR, 1987);
        dateOfBirth.set(Calendar.HOUR_OF_DAY, 0);
        dateOfBirth.set(Calendar.MINUTE, 0);
        dateOfBirth.set(Calendar.SECOND, 0);
        ProfileStatistic expectedProfile = new ProfileStatistic.ProfileStatisticBuilder()
                .withHostCountry("Belgium")
                .withOrganizaionTypes(Arrays.asList("NGO", "PES"))
                .withGender("Male")
                .withDateOfBirth(dateOfBirth)
                .withArrivalDate(dateOfBirth)
                .withNationalitiesCountry(Arrays.asList("Belgium"))
                .withExpectations(Arrays.asList("Improve language skills",
                        "Follow integration course",
                        "Undertake education and / or training", "Find work"))
                .withEducationLevel("Higher Education / Training")
                .withFieldsOfStudy(Arrays.asList("Literacy and numeracy",
                        "Basic programmes and qualifications",
                        "Other field of study"))
                .withLanguages(Arrays.asList("French", "English"))
                .withEscoOccupations(Arrays.asList("3D printing technician", "ICT accessibility tester"))
                .withEscoSkills(Arrays.asList("3D lighting", "3D lighting", "3D printing process", "3D texturing", "3D texturing", "ABAP"))
                .build();

        assertEquals(expectedProfile, deserializedProfile);
    }

    @Test
    public void deserializeWithAnEmptyProfile() throws IOException {
        String emptyProfile = new String(Files.readAllBytes(
                Paths.get("./src/test/resources/eu/europa/empl/skpt/data/serializer/jsonEmptyProfile.json")));

        JsonParser jsonParser = objectMapper.getFactory().createParser(emptyProfile);
        DeserializationContext deserializationContext = objectMapper.getDeserializationContext();
        ProfileStatistic deserializedProfile = profileStatisticDeserializer.deserialize(jsonParser, deserializationContext);

        ProfileStatistic expectedProfile = new ProfileStatistic.ProfileStatisticBuilder()
                .withExpectations(new ArrayList<>())
                .withFieldsOfStudy(new ArrayList<>())
                .withEscoOccupations(new ArrayList<>())
                .withEscoSkills(new ArrayList<>())
                .withOrganizaionTypes(new ArrayList<>())
                .withGender("")
                .withHostCountry("")
                .withEducationLevel("")
                .build();

        assertEquals(expectedProfile, deserializedProfile);
    }

    @Test
    public void deserializeWithAnOnlyYearProfile() throws IOException {
        String emptyProfile = new String(Files.readAllBytes(
                Paths.get("./src/test/resources/eu/europa/empl/skpt/data/serializer/jsonYearProfile.json")));

        JsonParser jsonParser = objectMapper.getFactory().createParser(emptyProfile);
        DeserializationContext deserializationContext = objectMapper.getDeserializationContext();
        ProfileStatistic deserializedProfile = profileStatisticDeserializer.deserialize(jsonParser, deserializationContext);

        Calendar dateOfBirth = Calendar.getInstance();
        dateOfBirth.set(Calendar.DAY_OF_MONTH, 1);
        dateOfBirth.set(Calendar.MONTH, 0);
        dateOfBirth.set(Calendar.YEAR, 1995);
        dateOfBirth.set(Calendar.HOUR_OF_DAY, 0);
        dateOfBirth.set(Calendar.MINUTE, 0);
        dateOfBirth.set(Calendar.SECOND, 0);

        Calendar dateOfArrival = Calendar.getInstance();
        dateOfArrival.set(Calendar.DAY_OF_MONTH, 1);
        dateOfArrival.set(Calendar.MONTH, 0);
        dateOfArrival.set(Calendar.YEAR, 2018);
        dateOfArrival.set(Calendar.HOUR_OF_DAY, 0);
        dateOfArrival.set(Calendar.MINUTE, 0);
        dateOfArrival.set(Calendar.SECOND, 0);

        ProfileStatistic expectedProfile = new ProfileStatistic.ProfileStatisticBuilder()
                .withDateOfBirth(dateOfBirth)
                .withArrivalDate(dateOfArrival)
                .withExpectations(new ArrayList<>())
                .withFieldsOfStudy(new ArrayList<>())
                .withEscoOccupations(new ArrayList<>())
                .withEscoSkills(new ArrayList<>())
                .withOrganizaionTypes(new ArrayList<>())
                .withGender("")
                .withHostCountry("")
                .withEducationLevel("")
                .build();

        assertEquals(expectedProfile, deserializedProfile);
    }

}
