/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 16/07/2018
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.service.attachment.impl;

import eu.europa.empl.skpt.common.Error;
import eu.europa.empl.skpt.config.IApplicationProperties;
import eu.europa.empl.skpt.data.repository.attachment.IAttachmentRepository;
import eu.europa.empl.skpt.service.attachment.AttachmentSaveResult;
import org.apache.commons.io.IOUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import static org.junit.Assert.*;
import static org.mockito.Mockito.when;

/**
 * Test class for {@link AttachmentService}.
 *
 * @author ARHS Spikeseed
 */
@RunWith(SpringRunner.class)
@DataJpaTest
@ActiveProfiles("unitTest")
@Sql(value = "attachment-test.sql")
public class AttachmentServiceTest {

    @Autowired
    private IAttachmentRepository attachmentRepository;

    @Mock
    private IApplicationProperties applicationProperties;

    private AttachmentService attachmentService;

    @Before
    public void init() {
        attachmentService = new AttachmentService(attachmentRepository, applicationProperties);

        List<String> acceptedMediaType = Arrays.asList("application/pdf", "image/jpeg", "image/png");
        when(applicationProperties.getAttachmentAcceptedMediaTypes())
                .thenReturn(acceptedMediaType);

        List<String> acceptedExtensions = Arrays.asList("pdf", "jpg", "jpeg", "png");
        when(applicationProperties.getAttachmentAcceptedExtensions())
                .thenReturn(acceptedExtensions);

        when(applicationProperties.getAttachmentMaxSize())
                .thenReturn(2500000L);
    }

    @Test
    public void save() throws IOException {
        InputStream is = Files.newInputStream(
                Paths.get("./src/test/resources/images/profile_picture.jpg"));
        String name = "blob.jpg";
        String type = "image/png";
        Long size = 14948L;
        AttachmentSaveResult attachmentSaveResult = attachmentService.save(is, name, type, size);

        assertNotEquals(Optional.empty(), attachmentSaveResult.getAttachment());
        assertEquals(Collections.emptyList(), attachmentSaveResult.getErrors());

    }

    @Test
    public void saveWithErrors() throws IOException {
        InputStream is = Files.newInputStream(
                Paths.get("./src/test/resources/images/profile_picture.jpg"));
        String name = "blob.sparta";
        String type = "jean/didier";
        Long size = Long.MAX_VALUE;
        AttachmentSaveResult attachmentSaveResult = attachmentService.save(is, name, type, size);

        assertEquals(Optional.empty(), attachmentSaveResult.getAttachment());
        List<Error> errors =
                Arrays.asList(Error.ATTACHMENT_WRONG_MEDIA_TYPE, Error.ATTACHMENT_WRONG_EXTENSION, Error.ATTACHMENT_EXCEEDS_MAX_SIZE);
        assertEquals(errors, attachmentSaveResult.getErrors());
    }

    @Test
    public void read() throws IOException {
        String path = "./src/test/resources/eu/europa/empl/skpt/service/attachment/impl/attachment.jpg";
        InputStream isExpected = Files.newInputStream(
                Paths.get(path));
        String id = "001";
        String imageToReadPath = path.replace("attachment.jpg", "");

        when(applicationProperties.getAttachmentDir())
                .thenReturn(new File(imageToReadPath));

        InputStream is = attachmentService.read(id);

        assertTrue(IOUtils.contentEquals(isExpected, is));
    }

    @Test(expected = IllegalArgumentException.class)
    public void readWithBadAttachmentId() {
        String id = "003";
        attachmentService.read(id);
    }

    @Test(expected = RuntimeException.class)
    public void readWithFileNotFoundException() {
        String id = "002";
        attachmentService.read(id);
    }

    @Test
    public void removeOutdatedAttachments() {
        String imageToDeletePath =
                getClass().getResource("/eu/europa/empl/skpt/service/attachment/impl/attachment.jpg").getPath();

        when(applicationProperties.getAttachmentDir())
                .thenReturn(new File(imageToDeletePath.replace("attachment.jpg", "")));

        attachmentService.removeOutdatedAttachments();

        File f = new File(imageToDeletePath);
        assertFalse(f.exists());
    }

}
