/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.service.country.impl;

import eu.europa.empl.skpt.data.domain.IReferenceData;
import eu.europa.empl.skpt.data.domain.country.Country;
import eu.europa.empl.skpt.data.repository.country.ICountryRepository;
import eu.europa.empl.skpt.service.tool.LanguageFilter;
import eu.europa.empl.skpt.tool.CountryGenerator;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.data.domain.*;

import java.util.List;
import java.util.Optional;

import static org.junit.Assert.*;
import static org.mockito.Mockito.when;

/**
 * Test class for the {@link CountryService} service.
 *
 * @author ARHS Spikeseed
 */
@RunWith(MockitoJUnitRunner.class)
public class CountryServiceTest {

    @Mock
    private ICountryRepository countryRepository;
    @Mock
    private LanguageFilter languageFilter;

    private CountryService countryService;

    private List<Country> allCountry;
    private List<Country> allCountryContainsLuxWord;
    private Country macaoCountry;
    private Pageable pageable;

    @Before
    public void setUp() {
        countryService = new CountryService(
                countryRepository, languageFilter);

        allCountry = CountryGenerator.generateAllCountryList();
        allCountryContainsLuxWord = CountryGenerator.generateAllCountryContainsLuxWordList();

        macaoCountry = allCountry.get(1);

        pageable = PageRequest.of(0, 20, Sort.Direction.ASC, "id");

        when(countryRepository.findAll(pageable))
                .thenReturn(new PageImpl<>(allCountry));

        when(countryRepository.findByValueIgnoreCase("lux", pageable))
                .thenReturn(new PageImpl<>(allCountryContainsLuxWord));

        when(countryRepository.findByAlphaCode3("MAC"))
                .thenReturn(Optional.of(macaoCountry));

        when(countryRepository.findByAlphaCode3("empty"))
                .thenReturn(Optional.ofNullable(null));

        Mockito.<Page<? extends IReferenceData>>
                when(languageFilter.filter(new PageImpl<>(allCountry), new String[0]))
                .thenReturn(new PageImpl<>(allCountry));

        Mockito.<Page<? extends IReferenceData>>
                when(languageFilter.filter(new PageImpl<>(allCountryContainsLuxWord), new String[0]))
                .thenReturn(new PageImpl<>(allCountryContainsLuxWord));

        when(languageFilter.filter(macaoCountry, new String[0]))
                .thenReturn(macaoCountry);
    }

    @Test
    public void findAll() {
        String searchString = null;
        String[] resultLanguages = {};
        List<Country> countries = this.countryService
                .search(searchString, resultLanguages, pageable)
                .getContent();

        assertNotNull(countries);
        assertEquals(allCountry, countries);
    }

    @Test
    public void findAllWithSearchString() {
        String searchString = "lux";
        String[] resultLanguages = {};
        List<Country> countries = this.countryService
                .search(searchString, resultLanguages, pageable)
                .getContent();

        assertNotNull(countries);
        assertEquals(allCountryContainsLuxWord, countries);
    }

    @Test
    public void findByAlphaCode3() {
        String code = "MAC";
        String[] resultLanguages = {};
        Optional<Country> country = this.countryService
                .findByAlphaCode3(code, resultLanguages);

        assertNotNull(country);
        assertTrue(country.isPresent());
        assertEquals(macaoCountry, country.get());
    }

    @Test
    public void findByAlphaCode3WithCountryNotPresent() {
        String code = "empty";
        String[] resultLanguages = {};
        Optional<Country> country = this.countryService
                .findByAlphaCode3(code, resultLanguages);

        assertNotNull(country);
        Assert.assertFalse(country.isPresent());
    }
}
