package eu.europa.empl.skpt.service.export.impl;

import eu.europa.empl.skpt.service.export.IExportService;
import eu.europa.empl.skpt.service.export.model.ParseJsonResult;
import eu.europa.empl.skpt.service.export.model.profile.Profile;
import eu.europa.empl.skpt.service.importation.IImportService;
import org.apache.commons.io.IOUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.ImageType;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.junit4.SpringRunner;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import static org.junit.Assert.*;

@RunWith(SpringRunner.class)
@SpringBootTest
@ActiveProfiles("integrationTest")
@Sql("export-service-integration-test.sql")
public class ExportServiceIntegrationTest {

    @Value("${export.word.temp.dir}")
    private String tmpPath;

    @Autowired
    private IExportService exportService;

    @Autowired
    private IImportService importService;

    private static final String BASE_FOLDER = "src/test/resources/eu/europa/empl/skpt/service/export/impl/";

    @Test
    public void generatePdf() throws Exception {
        checkXmlForPdfGeneration("profile.xml", "skpt.xml");
        checkPdfGeneration("profile.xml", "skpt.pdf");
    }

    @Test
    public void generatePdfForMaritalStatusOther() throws Exception {
        checkXmlForPdfGeneration("profile_marital_status_other.xml", "skpt_other_marital_status.xml");
        checkPdfGeneration("profile_marital_status_other.xml", "skpt_other_marital_status.pdf");
    }

    @Test
    public void generatePdfForExpectations() throws Exception {
        checkXmlForPdfGeneration("profile_expectations.xml", "skpt_expectations.xml");
        checkPdfGeneration("profile_expectations.xml", "skpt_expectations.pdf");
    }

    @Test
    public void generatePdfForExperiences() throws Exception {
        checkXmlForPdfGeneration("profile_experiences.xml", "skpt_experiences.xml");
        checkPdfGeneration("profile_experiences.xml", "skpt_experiences.pdf");
    }

    @Test
    public void generatePdfWithNoDigitalSkill() throws Exception {
        checkXmlForPdfGeneration("profile_no_digital_skills.xml", "skpt_no_digital_skills.xml");
        checkPdfGeneration("profile_no_digital_skills.xml", "skpt_no_digital_skills.pdf");
    }

    @Test
    public void generatePdfWithLiteracyskills() throws Exception {
        checkXmlForPdfGeneration("profile_low_education.xml", "skpt_low_education.xml");
        checkPdfGeneration("profile_low_education.xml", "skpt_low_education.pdf");
    }

    @Test
    public void generateDocx() throws Exception {
        checkXmlForDocxGeneration("profile.xml", "skpt_word.xml");
        checkDocxFileGeneration("profile.xml", "skpt.docx");
    }

    @Test
    public void generateDocxForMaritalStatusOther() throws Exception {
        checkXmlForDocxGeneration("profile_marital_status_other.xml", "skpt_word_other_marital_status.xml");
        checkDocxFileGeneration("profile_marital_status_other.xml", "skpt_other_marital_status.docx");
    }

    @Test
    public void generateDocxForExpectations() throws Exception {
        checkXmlForDocxGeneration("profile_expectations.xml", "skpt_word_expectations.xml");
        checkDocxFileGeneration("profile_expectations.xml", "skpt_expectations.docx");
    }

    @Test
    public void generateDocxForExperiences() throws Exception {
        checkXmlForDocxGeneration("profile_experiences.xml", "skpt_word_experiences.xml");
        checkDocxFileGeneration("profile_experiences.xml", "skpt_experiences.docx");
    }

    @Test
    public void generateDocxWithNoDigitalSkill() throws Exception {
        checkXmlForDocxGeneration("profile_no_digital_skills.xml", "skpt_word_no_digital_skills.xml");
        checkDocxFileGeneration("profile_no_digital_skills.xml", "skpt_no_digital_skills.docx");
    }

    @Test
    public void generateDocxWithLiteracyskills() throws Exception {
        checkXmlForDocxGeneration("profile_low_education.xml", "skpt_word_low_education.xml");
        checkDocxFileGeneration("profile_low_education.xml", "skpt_low_education.docx");
    }

    @Test
    public void generateXmlWithAttachments() throws IOException {
        ByteArrayOutputStream os = new ByteArrayOutputStream();

        String json = new String(Files.readAllBytes(
                Paths.get(BASE_FOLDER + "profile_with_attachment.json")));
        ParseJsonResult parseJsonResult = exportService.parseJson(json);
        Optional<Profile> profileOptional = parseJsonResult.getProfile();

        exportService.toXmlWithAttachments(os, profileOptional.get());

        String expectedXml = new String(Files.readAllBytes(
                Paths.get(BASE_FOLDER + "profile_with_attachment.xml")));
        Assert.assertEquals(
                expectedXml.replaceAll("(\n|\r|\r\n| )", ""),
                os.toString().replaceAll("(\n|\r|\r\n| )", ""));
    }

    private void checkXmlForPdfGeneration(String inputXml, String expectedXmlFile) throws Exception {
        checkXmlGeneration(inputXml, expectedXmlFile, "pdf");
    }

    private void checkXmlForDocxGeneration(String inputXml, String expectedXmlFile) throws Exception {
        checkXmlGeneration(inputXml, expectedXmlFile, "word");
    }

    private void checkPdfGeneration(String inputXml, String expectedPdfFile) throws Exception {
        Path profileFile = Paths.get(BASE_FOLDER + inputXml);
        byte[] profileContent = Files.readAllBytes(profileFile);
        Optional<Profile> profile = importService.importProfile("test_profile.xml", profileContent);

        try (InputStream expectedPdf = getClass().getResourceAsStream(expectedPdfFile);
                ByteArrayOutputStream actualPdf = new ByteArrayOutputStream()) {

            exportService.toPdf(actualPdf, profile.get(), "en", false);

            try (ByteArrayInputStream is = new ByteArrayInputStream(actualPdf.toByteArray())) {
                byte[] expected = pdfToImage(expectedPdf);
                byte[] actual = pdfToImage(is);
                assertArrayEquals(expected, actual);
            }
        }
    }

    private byte[] pdfToImage(InputStream is) throws Exception {
        try (ByteArrayOutputStream img = new ByteArrayOutputStream()) {

            PDDocument document = PDDocument.load(is);
            PDFRenderer pdfRenderer = new PDFRenderer(document);
            for (int page = 0; page < document.getNumberOfPages(); ++page) {
                BufferedImage bim = pdfRenderer.renderImageWithDPI(page, 100, ImageType.RGB);
                ImageIO.write(bim, "jpg", img);
            }

            document.close();
            return img.toByteArray();
        }
    }

    private void checkXmlGeneration(String inputXml, String expectedXmlFile, String exportFormat) throws Exception {
        Path profileFile = Paths.get(BASE_FOLDER + inputXml);
        byte[] profileContent = Files.readAllBytes(profileFile);
        Optional<Profile> profile = importService.importProfile(inputXml, profileContent);

        try (InputStream expectedIS = getClass().getResourceAsStream(expectedXmlFile);
                ByteArrayOutputStream actualIS = new ByteArrayOutputStream()) {
            if ("pdf".equals(exportFormat)) {
                exportService.toPdf(actualIS, profile.get(), "en", true);
            } else if ("word".equals(exportFormat)) {
                exportService.toWordXML(actualIS, profile.get(), "en");
            }

            StringWriter writer = new StringWriter();
            IOUtils.copy(expectedIS, writer, "UTF-8");
            String expectedTransformedXML = writer.toString();
            String actualTransformedXML = actualIS.toString("UTF-8");
            Assert.assertEquals(
                    actualTransformedXML.replaceAll("(\n|\r|\r\n| )", ""),
                    expectedTransformedXML.replaceAll("(\n|\r|\r\n| )", ""));
        }
    }

    private void checkDocxFileGeneration(String inputXml, String expectedDocxFile) throws Exception {
        byte[] profileContent = Files.readAllBytes(
                Paths.get(BASE_FOLDER + inputXml));
        Profile profile = importService.importProfile("test_profile.xml", profileContent).get();

        String actualDocxFileName = tmpPath + File.separator + "actualDocxFile.docx";
        try (ByteArrayOutputStream actualDocx = new ByteArrayOutputStream();
                OutputStream actualDocxFile = new FileOutputStream(actualDocxFileName)) {

            String filename = exportService.getFileName(profile.getPersonalInfo().getFamilyName(), profile.getPersonalInfo().getFirstName(), LocalDate.now(),  "docx");
            exportService.createDocxFile(actualDocx, filename, profile, "en", null);

            actualDocx.writeTo(actualDocxFile);

            Map<String, byte[]> actualDocxContent = getContentOfEachFile(actualDocxFileName);
            Map<String, byte[]> expectedDocxContent = getContentOfEachFile(BASE_FOLDER + expectedDocxFile);

            assertTrue(new File(actualDocxFileName).exists());
            assertEquals(expectedDocxContent.size(), actualDocxContent.size());
            expectedDocxContent.forEach((k, v) -> assertArrayEquals(k, v, actualDocxContent.get(k)));
        }
    }

    private Map<String, byte[]> getContentOfEachFile(String filename)throws IOException {
        Map<String, byte[]> files = new HashMap<>();

        ZipFile file = new ZipFile(filename);
        try {
            final Enumeration<? extends ZipEntry> entries = file.entries();
            while (entries.hasMoreElements()) {
                final ZipEntry entry = entries.nextElement();
                files.put(
                        entry.getName().replace('\\', '/'),
                        IOUtils.toString(file.getInputStream(entry), "UTF-8").replaceAll("(\\r|\\n)", "").getBytes());
            }
        } finally {
            file.close();
        }

        return files;
    }

    /**
     * !!! DO NOT REMOVE !!!
     * Method used during testing phase in order to generate a PDF file based on a specific input
     */
    private void generatePdfLocally(ByteArrayOutputStream actualPdf) throws IOException {
        FileOutputStream fos = new FileOutputStream("skpt.pdf");
        actualPdf.writeTo(fos);
        fos.flush();
        fos.close();
    }

    /**
     * !!! DO NOT REMOVE !!!
     * Method used during testing phase in order to generate a XML file based on a specific input
     */
    private void generateXmlLocally(ByteArrayOutputStream actualIS) throws IOException {
        FileOutputStream fos = new FileOutputStream("skpt.xml");
        actualIS.writeTo(fos);
        fos.flush();
        fos.close();
    }

    /**
     * !!! DO NOT REMOVE !!!
     * Method used during testing phase in order to generate a DOCX file based on a specific input
     */
    private void generateDocxLocally(ByteArrayOutputStream actualIS) throws IOException {
        FileOutputStream fos = new FileOutputStream("skpt.docx");
        actualIS.writeTo(fos);
        fos.flush();
        fos.close();
    }

}
