/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 27/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export.impl;

import eu.europa.empl.skpt.service.attachment.IAttachmentService;
import eu.europa.empl.skpt.service.export.IJsonParserService;
import eu.europa.empl.skpt.service.export.ITransformerService;
import eu.europa.empl.skpt.service.export.model.profile.Profile;
import org.apache.commons.io.IOUtils;
import org.apache.fop.apps.FOPException;
import org.apache.fop.apps.FopFactory;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import java.io.*;
import java.time.LocalDate;
import java.util.UUID;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;

/**
 * Test class for {@link ExportService}.
 *
 * @author ARHS Spikeseed
 */
@RunWith(MockitoJUnitRunner.class)
public class ExportServiceTest {

    @Mock
    private IAttachmentService attachmentService;
    @Mock
    private IJsonParserService jsonParserService;
    @Mock
    private Marshaller marshaller;
    @Mock
    private ITransformerService transformerService;
    @Mock
    private FopFactory fopFactory;

    @InjectMocks
    private ExportService exportService;

    @Test
    public void saveProfilePicture() throws Exception {
        String wordPicturePath = getClass().getResource("/word/word/media/image1.png").getPath();
        String pictureProfilePath = "./src/test/resources/images/profile_picture.jpg";
        String emptyPictureProfilePath = "./src/test/resources/word/word/media/image1.png";

        String profilePictureId = UUID.randomUUID().toString();
        String emptyProfilePictureId = UUID.randomUUID().toString();

        when(attachmentService.read(profilePictureId))
                .thenReturn(new BufferedInputStream(new FileInputStream(pictureProfilePath)));
        when(attachmentService.read(emptyProfilePictureId))
                .thenReturn(new BufferedInputStream(new FileInputStream(emptyPictureProfilePath)))
                .thenReturn(new BufferedInputStream(new FileInputStream(emptyPictureProfilePath)));

        try (InputStream emptyPicture = new BufferedInputStream(new FileInputStream(emptyPictureProfilePath));
             InputStream notEmptyPicture = new BufferedInputStream(new FileInputStream(pictureProfilePath))) {

            exportService.saveProfilePicture(wordPicturePath, emptyProfilePictureId);

            InputStream actualPicture = new BufferedInputStream(new FileInputStream(wordPicturePath));
            Assert.assertTrue(IOUtils.contentEquals(emptyPicture, actualPicture));
            Assert.assertFalse(IOUtils.contentEquals(notEmptyPicture, actualPicture));

            exportService.saveProfilePicture(wordPicturePath, profilePictureId);

            actualPicture = new BufferedInputStream(new FileInputStream(wordPicturePath));
            Assert.assertFalse(IOUtils.contentEquals(emptyPicture, actualPicture));
            Assert.assertTrue(IOUtils.contentEquals(notEmptyPicture, actualPicture));

            // replace to restore the basic image
            exportService.saveProfilePicture(wordPicturePath, emptyProfilePictureId);
        }
    }

    @Test
    public void parseJson() {
        String profileJson = "{'test'='test'}";
        jsonParserService.parseJson(profileJson);

        verify(jsonParserService).parseJson(profileJson);
    }

    @Test
    public void getFileName() {
        String lastName = "Doe";
        String firstName = "John";
        LocalDate localDate = LocalDate.now();
        String extension = "test";
        String fileName = exportService.getFileName(lastName, firstName, localDate, extension);

        String fileNameExpected =
                lastName + "_" + firstName + "_" + LocalDate.now().toString().replace("-", "") + "." + extension;
        assertEquals(fileNameExpected, fileName);
    }

    @Test
    public void getFileNameWithBlankLastName() {
        String lastName = "";
        String firstName = "John";
        LocalDate localDate = LocalDate.now();
        String extension = "test";
        String fileName = exportService.getFileName(lastName, firstName, localDate, extension);

        String fileNameExpected =
                firstName + "_" + LocalDate.now().toString().replace("-", "") + "." + extension;
        assertEquals(fileNameExpected, fileName);
    }

    @Test
    public void getFileNameWithBlankLastNameAndFirstName() {
        String lastName = "";
        String firstName = "";
        LocalDate localDate = LocalDate.now();
        String extension = "test";
        String fileName = exportService.getFileName(lastName, firstName, localDate, extension);

        String fileNameExpected =
                LocalDate.now().toString().replace("-", "") + "." + extension;
        assertEquals(fileNameExpected, fileName);
    }


    @Test(expected = RuntimeException.class)
    public void toXmlWithAttachmentsWithIOExceptionForXsltTransformProfile() throws IOException, TransformerException, JAXBException {
        doThrow().when(transformerService).transform(any(Profile.class),any(Source.class), eq(""), eq(true), any(Result.class));

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();

        exportService.toXmlWithAttachments(byteArrayOutputStream, new Profile());
    }

    @Test(expected = RuntimeException.class)
    public void toPdfWithFOPException() throws FOPException {
        when(fopFactory.newFop(anyString(), any(OutputStream.class)))
                .thenThrow(FOPException.class);

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();

        exportService.toPdf(byteArrayOutputStream, new Profile(), "en", false);
    }

}