/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool MOOC Backend
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 12/07/2018
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.service.export.impl;

import eu.europa.empl.skpt.common.Error;
import eu.europa.empl.skpt.data.domain.attachment.Attachment;
import eu.europa.empl.skpt.data.domain.country.Country;
import eu.europa.empl.skpt.data.domain.esco.EscoApiConcept;
import eu.europa.empl.skpt.data.domain.isced.IscedFoet;
import eu.europa.empl.skpt.data.domain.language.Language;
import eu.europa.empl.skpt.data.domain.nace.Nace;
import eu.europa.empl.skpt.service.attachment.AttachmentSaveResult;
import eu.europa.empl.skpt.service.attachment.IAttachmentService;
import eu.europa.empl.skpt.service.country.ICountryService;
import eu.europa.empl.skpt.service.esco.IEscoApiService;
import eu.europa.empl.skpt.service.export.XslDataService;
import eu.europa.empl.skpt.service.isced.IIscedFoetService;
import eu.europa.empl.skpt.service.language.ILanguageService;
import eu.europa.empl.skpt.service.nace.INaceService;
import org.hamcrest.Matchers;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.Instant;
import java.util.*;

import static eu.europa.empl.skpt.common.Error.FILE_EXCEEDS_MAX_SIZE;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.when;

/**
 * XslDataServiceTest
 *
 * @author ARHS Spikeseed
 */
@RunWith(MockitoJUnitRunner.class)
public class XslDataServiceTest {

    private static final String EMPTY_STRING = "";

    @Mock
    private ICountryService countryService;
    @Mock
    private IEscoApiService escoService;
    @Mock
    private IIscedFoetService iscedFoetService;
    @Mock
    private ILanguageService languageService;
    @Mock
    private INaceService naceService;
    @Mock
    private IAttachmentService attachmentService;

    @InjectMocks
    private XslDataService xslDataService;

    private String imageBase64;

    @Before
    public void init() throws IOException {
        imageBase64 = new String(Files.readAllBytes(
                Paths.get("./src/test/resources/eu/europa/empl/skpt/service/export/impl/imageBase64.txt")),
                StandardCharsets.UTF_8);
    }

    // getLabel

    @Test
    public void getLabelWithBlankKey() {
        String key = "";
        String lang = "en";
        String result = xslDataService.getLabel(key, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getLabelWithBlankLang() {
        String key = "skills.profile.tool.label";
        String lang = "";
        String result = xslDataService.getLabel(key, lang);

        assertEquals(EMPTY_STRING, result);
    }

    // getCountryLabel

    @Test
    public void getCountryLabelWithBlankCode() {
        String code = "";
        String lang = "en";
        String result = xslDataService.getCountryLabel(code, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getCountryLabelWithBlankLang() {
        String code = "EGY";
        String lang = "";
        String result = xslDataService.getCountryLabel(code, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getCountryLabelWithNullCountryValue() {
        String code = "EGY";
        String lang = "en";

        when(countryService.findByAlphaCode3(code, new String[] {lang}))
                .thenReturn(Optional.of(new Country()));

        String result = xslDataService.getCountryLabel(code, lang);

        assertEquals(EMPTY_STRING, result);
    }

    // getEscoLabel

    @Test
    public void getEscoLabelWithBlankId() {
        String id = "";
        String classification = "OCCUPATION";
        String lang = "en";
        String result = xslDataService.getEscoLabel(id, classification, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getEscoLabelWithBlankLang() {
        String id = "http://data.europa.eu/esco/occupation/52df9d56-efd4-48d0-ad93-59231943fc4c";
        String classification = "OCCUPATION";
        String lang = "";
        String result = xslDataService.getEscoLabel(id, classification, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getEscoLabelWithNullPreferredLabel() {
        String id = "http://data.europa.eu/esco/occupation/";
        String classification = "OCCUPATION";
        String lang = "en";

        EscoApiConcept escoApiConcept = new EscoApiConcept();
        escoApiConcept.setPreferredLabel(new HashMap<>());
        when(escoService.findById(id, classification))
                .thenReturn(Optional.of(escoApiConcept));

        String result = xslDataService.getEscoLabel(id, classification, lang);

        assertEquals(EMPTY_STRING, result);
    }

    // getIscedFoetLabel

    @Test
    public void getIscedFoetLabelWithBlankId() {
        String id = "";
        String lang = "en";
        String result = xslDataService.getIscedFoetLabel(id, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getIscedFoetLabelWithBlankLang() {
        String id = "1";
        String lang = "";
        String result = xslDataService.getIscedFoetLabel(id, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getIscedFoetLabelWithNullIscedFoetValue() {
        String id = "1";
        String lang = "en";

        when(iscedFoetService.findById(Long.valueOf(id), new String[] {lang}))
                .thenReturn(Optional.of(new IscedFoet()));

        String result = xslDataService.getIscedFoetLabel(id, lang);

        assertEquals(EMPTY_STRING, result);
    }

    // getLanguageLabel

    @Test
    public void getLanguageLabelWithBlankCode() {
        String code = "";
        String lang = "en";
        String result = xslDataService.getLanguageLabel(code, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getLanguageLabelWithBlankLang() {
        String code = "en";
        String lang = "";
        String result = xslDataService.getLanguageLabel(code, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getLanguageLabelWithNullLanguageValue() {
        String code = "en";
        String lang = "en";

        when(languageService.findByIsoCode1(code, new String[] {lang}))
                .thenReturn(Optional.of(new Language()));

        String result = xslDataService.getLanguageLabel(code, lang);

        assertEquals(EMPTY_STRING, result);
    }

    // getNaceLabel

    @Test
    public void getNaceLabelWithBlankId() {
        String id = "";
        String lang = "en";
        String result = xslDataService.getNaceLabel(id, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getNaceLabelWithBlankLang() {
        String id = "2";
        String lang = "";
        String result = xslDataService.getNaceLabel(id, lang);

        assertEquals(EMPTY_STRING, result);
    }

    @Test
    public void getNaceLabelWithNullNaceValue() {
        String id = "2";
        String lang = "en";

        when(naceService.findById(Long.valueOf(id), new String[] {lang}))
                .thenReturn(Optional.of(new Nace()));

        String result = xslDataService.getNaceLabel(id, lang);

        assertEquals(EMPTY_STRING, result);
    }

    // getAttachmentAsBase64

    @Test
    public void getAttachmentAsBase64() throws IOException {
        String id = "013ad43b-4772-4089-a4b7-0a1b8995900d";

        when(attachmentService.findByIdentifier(id))
                .thenReturn(Optional.of(new Attachment()));

        InputStream is = Files.newInputStream(
                Paths.get("./src/test/resources/images/profile_picture.jpg"));
        when(attachmentService.read(id))
                .thenReturn(is);

        String result = xslDataService.getAttachmentAsBase64(id);

        Assert.assertThat(result, Matchers.equalToIgnoringWhiteSpace(imageBase64));
    }

    @Test
    public void getAttachmentAsBase64NotPresent() {
        String id = "013ad43b-4772-4089-a4b7-111111111111";

        when(attachmentService.findByIdentifier(id))
                .thenReturn(Optional.empty());

        String result = xslDataService.getAttachmentAsBase64(id);

        assertEquals(null, result);
    }

    // createAttachmentFromBase64

    @Test
    public void createAttachmentFromBase64() {
        String name = "blob.jpg";
        String type = "image/png";
        long size = 14948;

        Attachment attachmentFromBase64 = new Attachment();
        attachmentFromBase64.setIdentifier(UUID.randomUUID().toString());
        attachmentFromBase64.setName(name);
        attachmentFromBase64.setSize(size);
        attachmentFromBase64.setMediaType(type);
        attachmentFromBase64.setPath(".");
        attachmentFromBase64.setCreatedDate(Instant.now());
        attachmentFromBase64.setUploadTimestamp(Instant.now());
        when(attachmentService.save(any(InputStream.class), eq(name), eq(type), eq(size)))
                .thenReturn(new AttachmentSaveResult(attachmentFromBase64, null));

        String result = xslDataService.createAttachmentFromBase64(name, type, size, imageBase64);

        assertNotEquals(null, result);
    }

    @Test(expected = RuntimeException.class)
    public void createAttachmentFromBase64WithAttachmentSaveResultHasError() {
        String name = "blobHasError.jpg";
        String type = "image/png";
        long size = 14948;

        Attachment attachmentFromBase64 = new Attachment();
        attachmentFromBase64.setIdentifier(UUID.randomUUID().toString());
        attachmentFromBase64.setName(name);
        attachmentFromBase64.setSize(size);
        attachmentFromBase64.setMediaType(type);
        attachmentFromBase64.setPath(".");
        attachmentFromBase64.setCreatedDate(Instant.now());
        attachmentFromBase64.setUploadTimestamp(Instant.now());
        List<Error> errors = new ArrayList<>();
        errors.add(FILE_EXCEEDS_MAX_SIZE);
        when(attachmentService.save(any(InputStream.class), eq(name), eq(type), eq(size)))
                .thenReturn(new AttachmentSaveResult(attachmentFromBase64, errors));

        xslDataService.createAttachmentFromBase64(name, type, size, imageBase64);
    }

    @Test(expected = RuntimeException.class)
    public void createAttachmentFromBase64WithGetAttachmentThrowError() {
        String name = "blobThrowError.jpg";
        String type = "image/png";
        long size = 14948;

        when(attachmentService.save(any(InputStream.class), eq(name), eq(type), eq(size)))
                .thenReturn(new AttachmentSaveResult(null, null));

        xslDataService.createAttachmentFromBase64(name, type, size, imageBase64);
    }

}
