/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.export.impl;

import com.fasterxml.jackson.core.TreeNode;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ValueNode;
import eu.europa.empl.skpt.service.export.XslDataService;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.when;

/**
 * Helper class for XslDataService
 *
 * @author ARHS Spikeseed
 */
public class XslDataServiceTestHelper {

    private static ObjectMapper objectMapper = new ObjectMapper();
    private static Map<String, Map<String, String>> labels = new TreeMap<>();

    public static void mockLabels(XslDataService xslDataService) throws IOException {
        // ISCED FoET label mock
        when(xslDataService.getIscedFoetLabel(anyString(), anyString())).thenReturn("Literacy and numeracy");

        initLabels(xslDataService);
        initCountryLabels(xslDataService);
        initLanguageLabels(xslDataService);
        initNaceLabels(xslDataService);
        initEscoLabels(xslDataService);
    }

    private static void initLabels(XslDataService xslDataService) throws IOException {
        JsonNode en = objectMapper.readTree(XslDataServiceTestHelper.class.getResource("en.json"));

        getLabels(en, "", "en");

        for (Map.Entry<String, Map<String, String>> label : labels.entrySet()) {
            for (Map.Entry<String, String> translation : label.getValue().entrySet()) {
                when(xslDataService.getLabel(eq(label.getKey()), eq(translation.getKey())))
                        .thenReturn(translation.getValue());
            }
        }
    }

    private static void getLabels(TreeNode node, String path, String lang) {

        if (node.size() > 0) {
            Iterator<String> it = node.fieldNames();
            while (it.hasNext()) {
                String fieldName = it.next();

                TreeNode nextNode = node.get(fieldName);

                String newPath = StringUtils.isBlank(path) ? fieldName : path.concat(".").concat(fieldName);

                if (nextNode.isValueNode()) {
                    Map<String, String> newTranslation = new HashMap<>();
                    newTranslation.put(lang, ((ValueNode) nextNode).asText());
                    Map<String, String> existingTranslation = labels.putIfAbsent(newPath, newTranslation);
                    if (existingTranslation != null) {
                        existingTranslation.put(lang, ((ValueNode) nextNode).asText());
                    }
                } else if (nextNode.isObject()) {

                    getLabels(nextNode, newPath, lang);

                }
            }
        }
    }

    private static void initNaceLabels(XslDataService xslDataService) {
        // NACE label mock
        when(xslDataService.getNaceLabel(eq("2"), anyString())).thenReturn("MINING AND QUARRYING");
        when(xslDataService.getNaceLabel(eq("4"), anyString())).thenReturn("ELECTRICITY, GAS, STEAM AND AIR CONDITIONING SUPPLY");
    }

    private static void initEscoLabels(XslDataService xslDataService) {
        // ESCO label mock
        Map<String, String> escoLabels = new HashMap<>();
        escoLabels.put("http://data.europa.eu/esco/occupation/eda0d957-3c3c-4139-b89a-a18bc9e18897", "accountant / comptable");
        escoLabels.put("http://data.europa.eu/esco/occupation/bab5fa79-7f96-4e21-87b6-1eba560b8d9a", "3D modeller");
        escoLabels.put("http://data.europa.eu/esco/occupation/8befd482-70e2-4d1e-8a09-091e15fe85c3", "Freinet school teacher");
        escoLabels.put("http://data.europa.eu/esco/occupation/4cf7be91-fed9-47a7-9ca9-e74c7eb6becb", "3D printing technician");
        escoLabels.put("http://data.europa.eu/esco/occupation/50af07f0-7a75-424e-a66d-5a9deea10f4c", "ICT accessibility tester");
        escoLabels.put("http://data.europa.eu/esco/skill/0189f448-179e-47cc-9716-c5c3ac4b1aec", "3D printing technician");
        escoLabels.put("http://data.europa.eu/esco/skill/6e53fd99-b646-4327-9580-ac062ab21188", "3D lighting");
        escoLabels.put("http://data.europa.eu/esco/skill/2afb2b59-c9a3-4cf3-b1dd-1a2fad51e583", "3D printing process");
        escoLabels.put("http://data.europa.eu/esco/skill/3e7516dc-0f7c-4686-8b2f-fade4e5d21be", "3D texturing");
        escoLabels.put("http://data.europa.eu/esco/skill/eb0e5615-1575-4a86-a1a2-7d39595033c5", "ABAP");
        escoLabels.put("1", "Horse cart / Attelage");
        escoLabels.put("8", "Train/ Train");
        escoLabels.put("13", "Aircraft / Avion");

        for (Map.Entry<String, String> entry : escoLabels.entrySet()) {
            when(xslDataService.getEscoLabel(eq(entry.getKey()), anyString(), anyString())).thenReturn(entry.getValue());
        }
    }

    private static void initLanguageLabels(XslDataService xslDataService) {
        // Language label mock
        Map<String, String> languages = new HashMap<>();
        languages.put("en", "English");
        languages.put("fr", "French");
        languages.put("ae", "Avestan");
        languages.put("af", "Afrikaans");
        languages.put("aa", "Afar");
        languages.put("ab", "Abkhazian");
        languages.put("ak", "Akan");
        languages.put("bo", "Tibetan");

        for (Map.Entry<String, String> entry : languages.entrySet()) {
            when(xslDataService.getLanguageLabel(eq(entry.getKey()), anyString())).thenReturn(entry.getValue());
        }
    }

    private static void initCountryLabels(XslDataService xslDataService) {
        // Country label mock
        Map<String, String> countries = new HashMap<>();
        countries.put("LUX", "Luxembourg");
        countries.put("GBR", "United Kingdom");
        countries.put("BEL", "Belgium");
        countries.put("DEU", "Germany");
        countries.put("FRA", "France");

        for (Map.Entry<String, String> entry : countries.entrySet()) {
            when(xslDataService.getCountryLabel(eq(entry.getKey()), anyString())).thenReturn(entry.getValue());
        }
    }

    //    public static void main(String[] args) throws Exception {
    //
    //        StringBuilder sb = new StringBuilder();
    //
    //        for (Map.Entry<String,Map<String,String>> label : labels.entrySet()) {
    //            sb.append("INSERT INTO tb_rfd_label (key_value, created_date) VALUES ('");
    //            sb.append(label.getKey());
    //            sb.append("', current_timestamp);\n");
    //
    //            sb.append("INSERT INTO tb_rfd_label_value (language, translation, label_id, created_date) VALUES\n");
    //
    //            for (Map.Entry<String,String> translation : label.getValue().entrySet()) {
    //                sb.append("  ('");
    //                sb.append(translation.getKey());
    //                sb.append("', '");
    //                sb.append(translation.getValue().replace("'", "''"));
    //                sb.append("', LAST_INSERT_ID(), current_timestamp),\n");
    //            }
    //            sb.delete(sb.length() - 2, sb.length());
    //            sb.append(";\n\n");
    //
    //
    //        }
    //
    //        System.out.println(sb.toString());
    //
    //
    //    }
}
