/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 * 
 *  CREATED BY : ARHS Spikeseed
 *          ON : 29/3/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2015 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.label.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.jayway.jsonpath.DocumentContext;
import com.jayway.jsonpath.JsonPath;
import eu.europa.empl.skpt.data.domain.label.Label;
import eu.europa.empl.skpt.data.repository.label.ILabelRepository;
import eu.europa.empl.skpt.tool.LabelGenerator;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

/**
 * Test class for {@link LabelService}
 *
 * @author ARHS Spikeseed
 */
@RunWith(MockitoJUnitRunner.class)
public class LabelServiceTest {

    @Mock
    private ILabelRepository labelRepository;

    private LabelService labelService;

    private List<Label> allLabelList;
    private List<Label> allLabelListWithSamePathElement;
    private Label labelWithNullValue;
    private ObjectMapper objectMapper;

    @Before
    public void setup() {
        objectMapper = new ObjectMapper();

        this.labelService = new LabelService(
                labelRepository, objectMapper);

        allLabelList = LabelGenerator.generateAllLabelList();
        allLabelListWithSamePathElement = LabelGenerator.generateLabelListWithPathProblem();

        labelWithNullValue = allLabelList.get(4);
    }

    @Test
    public void asJsonFr() throws Exception {
        when(labelRepository.findAllAsStream())
                .thenReturn(allLabelList.stream());

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            labelService.asJson(os, "fr");

            try (InputStream is = new ByteArrayInputStream(os.toByteArray())) {
                DocumentContext dc = JsonPath.parse(is);

                assertEquals(3, ((Map) dc.read("common")).size());
                assertEquals("Oui", dc.read("common.yes"));
                assertEquals("Non", dc.read("common.no"));
                assertEquals("Pomme", dc.read("common.nested.deep.apple"));
                assertEquals("Ananas", dc.read("common.nested.deep.pineapple"));
            }

        }

    }

    @Test
    public void asJsonEn() throws Exception {
        when(labelRepository.findAllAsStream())
                .thenReturn(allLabelList.stream());

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            labelService.asJson(os, "en");

            try (InputStream is = new ByteArrayInputStream(os.toByteArray())) {
                DocumentContext dc = JsonPath.parse(is);

                assertEquals(4, ((Map) dc.read("common")).size());
                assertEquals("Yes", dc.read("common.yes"));
                assertEquals("No", dc.read("common.no"));
                assertEquals("Apple", dc.read("common.nested.deep.apple"));
                assertEquals("Pineapple", dc.read("common.nested.deep.pineapple"));
            }

        }

    }

    @Test(expected = RuntimeException.class)
    public void asJsonWithSamePathError() throws IOException {
        when(labelRepository.findAllAsStream())
                .thenReturn(allLabelListWithSamePathElement.stream());

        try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            labelService.asJson(os, "en");
        }
    }

    @Test
    public void getLabelTranslationByKeyWithLabelNotPresent() {
        when(labelRepository.findByKey(anyString()))
                .thenReturn(Optional.ofNullable(null));

        Optional<String> stringOptional = labelService.getLabelTranslationByKey("empty", "en");

        assertEquals(Optional.empty(), stringOptional);
    }

    @Test
    public void getLabelTranslationByKeyWithNullLabelValue() {
        when(labelRepository.findByKey(anyString()))
                .thenReturn(Optional.of(labelWithNullValue));

        Optional<String> stringOptional = labelService.getLabelTranslationByKey("empty", "fr");

        assertEquals(Optional.empty(), stringOptional);
    }

}
