/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 28/01/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */

package eu.europa.empl.skpt.service.language.impl;

import eu.europa.empl.skpt.data.domain.IReferenceData;
import eu.europa.empl.skpt.data.domain.language.Language;
import eu.europa.empl.skpt.data.repository.language.ILanguageRepository;
import eu.europa.empl.skpt.service.tool.LanguageFilter;
import eu.europa.empl.skpt.tool.LanguageGenerator;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.data.domain.*;

import java.util.*;

import static org.junit.Assert.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.when;

/**
 * Test class for the {@link LanguageService} service.
 *
 * @author ARHS Spikeseed
 */
@RunWith(MockitoJUnitRunner.class)
public class LanguageServiceTest {

    @Mock
    private ILanguageRepository languageRepository;
    @Mock
    private LanguageFilter languageFilter;

    private LanguageService languageService;

    private List<Language> allLanguages;
    private List<Language> allLanguagesInEnglish;
    private List<Language> afrikaansTibetanLanguages;
    private Language afrikaansLanguage;
    private Language tibetanLanguage;
    private Pageable pageable;

    @Before
    public void setUp() {
        languageService = new LanguageService(
                languageRepository, languageFilter);

        allLanguages = LanguageGenerator.generateAllLanguagesList();
        allLanguagesInEnglish = LanguageGenerator.generateAllLanguagesInEnglishList();
        afrikaansTibetanLanguages = Arrays.asList(afrikaansLanguage, tibetanLanguage);

        afrikaansLanguage = allLanguages.get(0);
        tibetanLanguage = allLanguages.get(1);

        pageable = PageRequest.of(0, 20, Sort.Direction.ASC, "id");

        when(this.languageRepository.findAll(any(PageRequest.class)))
                .thenReturn(new PageImpl<>(allLanguages));

        when(this.languageRepository.findByIsoCode1In(anySet(), eq(pageable)))
                .thenReturn(new PageImpl<>(afrikaansTibetanLanguages));

        when(this.languageRepository.findByValueIgnoreCaseAndLanguages(anySet(), anyString(), eq(pageable)))
                .thenReturn(new PageImpl<>(Arrays.asList(afrikaansLanguage)));

        when(this.languageRepository.findByValueIgnoreCase(anyString(), eq(pageable)))
                .thenReturn(new PageImpl<>(Arrays.asList(afrikaansLanguage)));

        when(this.languageRepository.findByIsoCode1("bo"))
                .thenReturn(Optional.of(tibetanLanguage));

        when(this.languageRepository.findByIsoCode1("empty"))
                .thenReturn(Optional.ofNullable(null));

        Mockito.<Page<? extends IReferenceData>>
                when(this.languageFilter.filter(new PageImpl<>(allLanguages), new String[0]))
                .thenReturn(new PageImpl<>(allLanguages));

        Mockito.<Page<? extends IReferenceData>>
                when(this.languageFilter.filter(new PageImpl<>(afrikaansTibetanLanguages), new String[0]))
                .thenReturn(new PageImpl<>(afrikaansTibetanLanguages));

        Mockito.<Page<? extends IReferenceData>>
                when(this.languageFilter.filter(new PageImpl<>(Arrays.asList(afrikaansLanguage)), new String[0]))
                .thenReturn(new PageImpl<>(Arrays.asList(afrikaansLanguage)));

        when(this.languageFilter.filter(tibetanLanguage, null))
                .thenReturn(tibetanLanguage);
    }

    @Test
    public void findAll() {
        String searchString = null;
        Set<String> mySet = new HashSet<>();
        String[] resultLanguages = {};
        List<Language> languages = this.languageService
                .search(searchString, mySet, resultLanguages, pageable)
                .getContent();

        assertNotNull(languages);
        assertEquals(allLanguages, languages);
    }

    @Test
    public void findLanguageByIsoCode1In() {
        String searchString = null;
        Set<String> searchLanguages = new HashSet<>();
        searchLanguages.add("af");
        searchLanguages.add("bo");
        String[] resultLanguages = {};
        List<Language> languages = this.languageService
                .search(searchString, searchLanguages, resultLanguages, pageable)
                .getContent();

        assertNotNull(languages);
        assertEquals(afrikaansTibetanLanguages, languages);
    }

    @Test
    public void findSpecificLanguageWithSearchLanguage() {
        String searchString = "afrikaans";
        Set<String> mySet = new HashSet<>();
        mySet.add("af");
        String[] resultLanguages = {};
        List<Language> languages = this.languageService
                .search(searchString, mySet, resultLanguages, pageable)
                .getContent();

        assertNotNull(languages);
        assertEquals(Arrays.asList(afrikaansLanguage), languages);
    }

    @Test
    public void findSpecificLanguage() {
        String searchString = "afrikaans";
        Set<String> mySet = new HashSet<>();
        String[] resultLanguages = {};
        List<Language> languages = this.languageService
                .search(searchString, mySet, resultLanguages, pageable)
                .getContent();

        assertNotNull(languages);
        assertEquals(Arrays.asList(afrikaansLanguage), languages);
    }

    @Test
    public void findByIsoCode1() {
        Optional<Language> language = this.languageService
                .findByIsoCode1("bo", null);

        assertNotNull(language);
        assertTrue(language.isPresent());
        assertEquals("bo", language.get().getIsoCode1());
        assertEquals("bod", language.get().getIsoCode2());
        assertEquals("Tibetan", language.get().getValues().get("en").getValue());
    }

    @Test
    public void findByIsoCodelWithLanguageNotPresent() {
        Optional<Language> language = this.languageService
                .findByIsoCode1("empty", null);

        assertNotNull(language);
        assertFalse(language.isPresent());
    }

}
