package eu.europa.empl.skpt.service.statistics;

import com.fasterxml.jackson.databind.ObjectMapper;
import eu.europa.empl.skpt.data.domain.stats.ProfileStatistic;
import eu.europa.empl.skpt.service.stats.IProfileStatisticService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.security.SecureRandom;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

//To uncomment for sending sample data Elasticsearch TEST index
//@RunWith(SpringRunner.class)
//@SpringBootTest
//@ActiveProfiles("dev")
public class ProfileStatisticServiceIntegrationTest {
    private Logger LOGGER = LoggerFactory.getLogger(ProfileStatisticServiceIntegrationTest.class);

    private static final List<Integer> arrivalYears = IntStream.rangeClosed(2010, 2018).boxed().collect(Collectors.toList());
    private static final List<Integer> birthYears = IntStream.rangeClosed(1975, 2017).boxed().collect(Collectors.toList());
    private static final List<Integer> month = IntStream.rangeClosed(Calendar.JANUARY, Calendar.DECEMBER).boxed().collect(Collectors.toList());
    private static final Map<String, List<String>> escoOccupationsSkills = Collections.unmodifiableMap(Stream.of(
            new AbstractMap.SimpleEntry<>("communication manager", Arrays.asList("protect client interests", "proofread text", "forming of public opinion",
                    "market research", "identify clients' needs")),
            new AbstractMap.SimpleEntry<>("software developer", Arrays.asList("identify customer requirements", "develop software prototype",
                    "perform scientific research", "use software libraries ")),
            new AbstractMap.SimpleEntry<>("ICT application configurator", Arrays.asList("debug software", "integrated development environment software",
                    "create flowchart diagram")),
            new AbstractMap.SimpleEntry<>("battery test technician", Arrays.asList("battery fluids", "inspect quality of products",
                    "electrical equipment regulations")),
            new AbstractMap.SimpleEntry<>("truck driving instructor", Arrays.asList("assist students in their learning", "teach driving practices",
                    "park vehicles")),
            new AbstractMap.SimpleEntry<>("solar energy technician", Arrays.asList("electricity", "install circuit breakers",
                    "comply with legal regulations", "mechanics")),
            new AbstractMap.SimpleEntry<>("woodcarver", Arrays.asList("maintain edged hand tools", "tend lathe",
                    "woodworking processes", "check quality of raw materials")),
            new AbstractMap.SimpleEntry<>("parliamentary assistant", Arrays.asList("collaborate in the drafting of policies",
                    "communicate with stakeholders", "check official documents", "advise on communication strategies")),
            new AbstractMap.SimpleEntry<>("coking furnace operator", Arrays.asList("operate heat treatment furnace", "monitor gauge",
                    "use a computer", "coking process")),
            new AbstractMap.SimpleEntry<>("bed and breakfast operator", Arrays.asList("identify customer's needs", "forecast occupancy demand",
                    "manage the customer experience", "greet guests")),
            new AbstractMap.SimpleEntry<>("rolling stock electrician", Arrays.asList("read standard blueprints", "electricity",
                    "electrical wiring plans", "use testing equipment")),
            new AbstractMap.SimpleEntry<>("fat-purification worker", Arrays.asList("apply HACCP", "wash oils", "apply GMP",
                    "boil water")))
            .collect(Collectors.toMap(e -> e.getKey(), e -> e.getValue())));
    private static final List<String> educationLevel = Arrays.asList("Primary Education", "Secondary Education", "Higher Education", "PhD", "Other");
    private static final List<String> expectations = Arrays.asList("Improve language skills", "Follow integration course",
            "Undertake education and / or training", "Find work");
    private static final List<String> fieldsOfStudy = Arrays.asList("Education, Information and Communication Technologies (ICTs)",
            "Hygiene and occupational health services", "Business and administration", "Architecture and construction",
            "Security services", "Arts", "Education", "Social and behavioural sciences", "Law");
    private static final List<String> gender = Arrays.asList("Male", "Female");
    private static final List<String> hostCountries = Arrays.asList("France", "Germany", "Italy", "Spain", "Belgium", "Luxembourg",
            "Greece", "Great Britain");
    private static final List<String> languages = Arrays.asList("French", "Arabic", "English", "German", "Afrikaans",
            "Danish", "Afar", "Icelandic", "Lao", "Greek", "Croatian", "Limburgish", "Kashmiri");
    private static final List<String> nationalityCountries = Arrays.asList("Afghanistan", "Syrian Arab Republic", "Algeria", "Eritrea",
            "Albania", "South Sudan", "Burundi", "Chad", "Ethiopia", "Gambia", "Sierra Leone", "Tajikistan", "Gibraltar", "Congo");
    private static final List<String> organizationTypes = Arrays
            .asList("PES", "Reception or integration centre", "Municipality", "Education/training authority",
                    "NGO", "School", "VET institution", "Higher Education institution", "Other");

    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private IProfileStatisticService profileStatisticService;


//To uncomment for sending sample data Elasticsearch TEST index
//    @Test
//    @Repeat(70)
    public void testSendStatistics() {
        ProfileStatistic.ProfileStatisticBuilder builder = new ProfileStatistic.ProfileStatisticBuilder();
        Random random = new SecureRandom();
        Collections.shuffle(this.arrivalYears);
        int arrivalYear = this.arrivalYears.get(random.nextInt(this.arrivalYears.size()));
        Collections.shuffle(this.birthYears);
        int birthYear = this.birthYears.get(random.nextInt(this.birthYears.size()));
        int month = this.month.get(random.nextInt(this.month.size()));
        Calendar arrivalDate = Calendar.getInstance();
        arrivalDate.set(Calendar.DAY_OF_MONTH, 15);
        arrivalDate.set(Calendar.MONTH, month);
        arrivalDate.set(Calendar.YEAR, arrivalYear);
        builder.withArrivalDate(arrivalDate);

        Calendar birthDate = Calendar.getInstance();
        birthDate.set(Calendar.DAY_OF_MONTH, 15);
        birthDate.set(Calendar.MONTH, month);
        birthDate.set(Calendar.YEAR, birthYear);
        builder.withDateOfBirth(birthDate);

        int nbOccupation = random.nextInt(4);
        List<String> occupations = new ArrayList<>();
        List<String> skills = new ArrayList<>();
        for (int i = 0; i <= nbOccupation; i++) {
            List<String> keys = new ArrayList<>(this.escoOccupationsSkills.keySet());
            Collections.shuffle(keys);
            String occupation = keys.get(random.nextInt(escoOccupationsSkills.size()));
            occupations.add(occupation);
            skills.addAll(this.escoOccupationsSkills.get(occupation));
        }
        builder.withEscoOccupations(occupations);
        builder.withEscoSkills(skills);
        Collections.shuffle(this.educationLevel);
        builder.withEducationLevel(this.educationLevel.get(random.nextInt(this.educationLevel.size())));

        int nbExpectations = random.nextInt(3);
        List<String> expectations = new ArrayList<>();
        Collections.shuffle(this.expectations);
        for (int i = 0; i <= nbExpectations; i++) {
            expectations.add(this.expectations.get(random.nextInt(this.expectations.size())));
        }
        builder.withExpectations(expectations);

        int nbStudyFields = random.nextInt(3);
        List<String> studyFields = new ArrayList<>();
        Collections.shuffle(this.fieldsOfStudy);
        for (int i = 0; i <= nbStudyFields; i++) {
            studyFields.add(this.fieldsOfStudy.get(random.nextInt(this.fieldsOfStudy.size())));
        }
        builder.withFieldsOfStudy(studyFields);
        Collections.shuffle(this.gender);
        builder.withGender(this.gender.get(random.nextInt(this.gender.size())));

        builder.withHostCountry(this.hostCountries.get(random.nextInt(this.hostCountries.size())));

        int nbLanguages = random.nextInt(3);
        List<String> languages = new ArrayList<>();
        Collections.shuffle(this.languages);
        for (int i = 0; i <= nbLanguages; i++) {
            languages.add(this.languages.get(random.nextInt(this.languages.size())));
        }
        builder.withLanguages(languages);

        int nbNatCountries = random.nextInt(4);
        List<String> nationalities = new ArrayList<>();
        Collections.shuffle(this.nationalityCountries);
        for (int i = 0; i <= nbNatCountries; i++) {
            nationalities.add(this.nationalityCountries.get(random.nextInt(this.nationalityCountries.size())));
        }
        builder.withNationalitiesCountry(nationalities);
        Collections.shuffle(this.organizationTypes);
        builder.withOrganizaionTypes(Arrays.asList(this.organizationTypes.get(random.nextInt(this.organizationTypes.size()))));


        ProfileStatistic profileStatistic = builder.build();
        try {
            LOGGER.debug("Profile statistics: \n" + objectMapper.writeValueAsString(profileStatistic));
            this.profileStatisticService.sendToElasticSearch(profileStatistic);
        } catch (Exception e) {
            LOGGER.error(e.getMessage());
        }

    }

}

