/* ----------------------------------------------------------------------------
 *     PROJECT : Skills Profile Tool
 *
 *  CREATED BY : ARHS Spikeseed
 *          ON : 03/04/2017
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016 European Commission – DG Employment, Social Affairs & Inclusion
 * ----------------------------------------------------------------------------
 */
package eu.europa.empl.skpt.service.user.impl;

import eu.europa.empl.skpt.api.dto.UserDto;
import eu.europa.empl.skpt.api.dto.UserDtoToEntityConverter;
import eu.europa.empl.skpt.data.domain.user.Role;
import eu.europa.empl.skpt.data.domain.user.Roles;
import eu.europa.empl.skpt.data.domain.user.User;
import eu.europa.empl.skpt.data.repository.role.IRoleRepository;
import eu.europa.empl.skpt.data.repository.user.IUserRepository;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * Test class for the {@link UserService} service.
 *
 * @author ARHS Spikeseed
 */
@RunWith(MockitoJUnitRunner.class)
public class UserServiceTest {

    @Mock
    private IUserRepository userRepository;

    @Mock
    private IRoleRepository roleRepository;

    @InjectMocks
    private UserService userService;

    private UserDtoToEntityConverter userDtoToEntityConverter = new UserDtoToEntityConverter();

    @Before
    public void setUp() {

        // Create roles
        final Role roleAdmin = new Role();
        roleAdmin.setRole(Roles.ADMIN);
        final Role roleRoot = new Role();
        roleRoot.setRole(Roles.ROOT);
        final Role roleInterviewer = new Role();
        roleInterviewer.setRole(Roles.INTERVIEWER);

        // Create users
        final User rootUser = new User();
        rootUser.setId(1L);
        rootUser.setFirstName("Root");
        rootUser.setLastName("Admin");
        rootUser.setUsername("root@skpt.com");

        final User adminUser = new User();
        adminUser.setId(2L);
        adminUser.setFirstName("Admin");
        adminUser.setLastName("Istrator");
        adminUser.setUsername("admin@skpt.com");

        final User interviewerUser = new User();
        interviewerUser.setId(3L);
        interviewerUser.setFirstName("Inter");
        interviewerUser.setLastName("Viewer");
        interviewerUser.setUsername("interviewer@skpt.com");

        final User addedUser = new User();
        addedUser.setId(4L);
        addedUser.setFirstName("John");
        addedUser.setLastName("Doe");
        addedUser.setUsername("john.doe@skpt.com");
        HashSet<Role> roles = new HashSet<>(Arrays.asList(roleRoot, roleAdmin));
        addedUser.setRoles(roles);

        // Create collection of roles and users
        final List<User> allUsers = Arrays.asList(rootUser, adminUser, interviewerUser);
        final List<Role> allRoles = Arrays.asList(roleAdmin, roleRoot, roleInterviewer);

        // Mock
        // TODO: check tests
//        when(this.userRepository.findById(anyLong())).thenReturn(Optional.of(rootUser));
        when(this.userRepository.findAll(any(PageRequest.class))).thenReturn(new PageImpl<>(allUsers));
        when(this.userRepository.save((User) any())).thenReturn(addedUser);
        when(this.roleRepository.findAll()).thenReturn(allRoles);

    }

    @Test
    public void findAll() {
        final List<User> realList = this.userService.getUsers( PageRequest.of(0, 10)).getContent();
        Assert.assertNotNull(realList);
        Assert.assertEquals(3, realList.size());
    }

    @Test
    public void addUser() throws Exception {
        final User user = new User();
        user.setFirstName("John");
        user.setLastName("Doe");
        user.setUsername("john.doe@skpt.com");
        Role roleAdmin = new Role();
        roleAdmin.setRole(Roles.ADMIN);
        Role roleRoot = new Role();
        roleRoot.setRole(Roles.ROOT);
        Set<Role> roles = new HashSet<>();
        roles.add(roleAdmin);
        roles.add(roleRoot);
        user.setRoles(roles);

        User savedUser = this.userService.saveUser(user);

        Assert.assertNotNull(savedUser);
        Assert.assertNotNull(user.getCreatedDate());
        Assert.assertNotNull(user.getModifiedDate());
    }

    @Test(expected = Exception.class)
    public void addUserWithEmptyRoles() throws Exception {
        final User user = new User();
        user.setFirstName("John");
        user.setLastName("Doe");
        user.setUsername("John.Doe@skpt.com");

        this.userService.saveUser(user);
    }

    @Test
    public void updateUser() throws Exception {
        final User user = new User();
        user.setId(4L);
        user.setFirstName("John");
        user.setLastName("Doe");
        user.setUsername("john.doe@skpt.com");
        Role roleRoot = new Role();
        roleRoot.setRole(Roles.ROOT);
        Set<Role> roles = new HashSet<>();
        roles.add(roleRoot);
        user.setRoles(roles);

        User updatedUser = this.userService.saveUser(user);

        Assert.assertNotNull(updatedUser);
        Assert.assertNotNull(user.getModifiedDate());;
    }

    @Test
    public void deleteUser() {
        this.userService.deleteUser(1L);
        verify(this.userRepository, times(1)).deleteById(1L);
    }

    @Test
    public void dtoToUser() {
        UserDto source = new UserDto();
        source.setFirstName("Nassim");
        source.setLastName("Marzouk");
        source.setUsername("nassim.marzouk@skpt.com");
        HashSet<Roles> enumRoles = new HashSet<>();
        Roles r1 = Roles.ROOT;
        Roles r2 = Roles.ADMIN;
        enumRoles.add(r1);
        enumRoles.add(r2);
        source.setRoles(enumRoles);

        User target = this.userDtoToEntityConverter.convert(source);

        Assert.assertTrue("The first name is not equal", source.getFirstName().equals(target.getFirstName()));
        Assert.assertTrue("The last name is not equal", source.getLastName().equals(target.getLastName()));
        Assert.assertTrue("The username is not equal", source.getUsername().equals(target.getUsername()));
        Assert.assertTrue("The role list is not equal", target.getRoles().size() == 2);
    }
}