import {HomePage} from '../pages/home.po';
import {browser} from 'protractor';
import * as path from 'path';
import {FooterPage} from '../pages/footer.po';
import * as fs from 'fs';
import * as AdmZip from 'adm-zip';
import * as PDFParser from 'pdf2json';
import {ConsentPage} from '../pages/consent.po';
import {Workbook} from 'exceljs';
import {HeaderPage} from '../pages/header.po';
import {LanguageSelectorPage} from '../pages/language-selector.po';

describe('A user', function () {
  let footerPage: FooterPage;
  let homePage: HomePage;
  let consentPage: ConsentPage;
  let headerPage: HeaderPage;
  let languageSelectorPage: LanguageSelectorPage;

  beforeAll(() => {
    footerPage = new FooterPage();
    homePage = new HomePage();
    consentPage = new ConsentPage();
    headerPage = new HeaderPage();
    languageSelectorPage = new LanguageSelectorPage();
    homePage.navigateTo();
  });

  afterEach(() => {
    homePage.navigateTo();
  });

  // pdf tests
  [
    ['en'],
    ['it']
  ].forEach(([languageCode]) => {
    it('must be able to download his full profile as a pdf file in language ' + languageCode, () => {
      // path of the file
      const fileToUpload = '../resources/profile.xml';
      const absolutePath = path.resolve(__dirname, fileToUpload);

      headerPage.getChangeLanguageButton().click();
      languageSelectorPage.getLanguageButton(languageCode).click();


      // unhide input (not working in firefox otherwise)
      browser.executeScript(
        'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
      );

      homePage.getFileReaderInput().sendKeys(absolutePath);
      expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

      testDownloadAsPdf(
        './e2e/downloads/Doe_John_' + getDateAsYYYYMMDD() + '.pdf',
        './e2e/resources/profile-' + languageCode + '.pdf');
    });
  });

  function getDateAsYYYYMMDD() {
    const date = new Date();
    return [
      date.getFullYear(),
      ((date.getMonth() + 1) > 9 ? '' : '0') + (date.getMonth() + 1),
      (date.getDate() > 9 ? '' : '0') + date.getDate()
    ].join('');
  }

  function testDownloadAsPdf(downloadedFilePath: string, expectedFilePath: string) {
    if (fs.existsSync(downloadedFilePath)) {
      // Make sure the browser doesn't have to rename the download.
      fs.unlinkSync(downloadedFilePath);
    }

    footerPage.getExportButton().click();
    footerPage.getExportAsPdfButton().click();

    browser.driver.wait(function () {
      // Wait until the file has been downloaded.
      return fs.existsSync(downloadedFilePath);
    }, 40000).then(function () {
      // expected profile -> json
      let expectedJson = '';
      let expectedTransformationDone = false;
      const expectedPdfParser = new PDFParser();
      expectedPdfParser.on('pdfParser_dataError', errData => console.error('Parse error : ' + errData.parserError));
      expectedPdfParser.on('pdfParser_dataReady', pdfData => {
        expectedJson = JSON.stringify(pdfData);
        expectedTransformationDone = true;
      });
      expectedPdfParser.loadPDF(expectedFilePath);

      // downloaded profile -> json
      let downloadedJson = '';
      let downloadedTransformationDone = false;
      const downloadedPdfParser = new PDFParser();
      downloadedPdfParser.on('pdfParser_dataError', errData => console.error('Parse error : ' + errData.parserError));
      downloadedPdfParser.on('pdfParser_dataReady', pdfData => {
        downloadedJson = JSON.stringify(pdfData);
        downloadedTransformationDone = true;
      });
      downloadedPdfParser.loadPDF(downloadedFilePath);

      browser.driver.wait(function () {
        return expectedTransformationDone && downloadedTransformationDone;
      }, 60000).then(function () {
        expect(downloadedJson).toEqual(expectedJson);

        fs.unlinkSync(downloadedFilePath);
      });
    });
  }

  // word tests
  [
    ['en'],
    ['it']
  ].forEach(([languageCode]) => {
    it('must be able to download his full profile as a word file in language ' + languageCode, () => {
      // path of the file
      const fileToUpload = '../resources/profile.xml';
      const absolutePath = path.resolve(__dirname, fileToUpload);

      headerPage.getChangeLanguageButton().click();
      languageSelectorPage.getLanguageButton(languageCode).click();


      // unhide input (not working in firefox otherwise)
      browser.executeScript(
        'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
      );

      homePage.getFileReaderInput().sendKeys(absolutePath);
      expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

      testDownloadAsWord(
        './e2e/downloads/Doe_John_' + getDateAsYYYYMMDD() + '.docx',
        './e2e/resources/profile_word-' + languageCode + '.xml');
    });
  });

  function testDownloadAsWord(downloadedFilePath: string, expectedFilePath: string) {
    if (fs.existsSync(downloadedFilePath)) {
      // Make sure the browser doesn't have to rename the download.
      fs.unlinkSync(downloadedFilePath);
    }

    footerPage.getExportButton().click();
    footerPage.getExportAsWordButton().click();

    browser.driver.wait(function () {
      // Wait until the file has been downloaded.
      return fs.existsSync(downloadedFilePath);
    }, 60000).then(function () {
      let zip = new AdmZip(downloadedFilePath);

      const downloadedDocument = zip.readAsText('word' + path.sep + 'document.xml');

      const expectedDocument = fs.readFileSync(expectedFilePath, 'utf8');

      expect(downloadedDocument).toEqual(expectedDocument);

      fs.unlinkSync(downloadedFilePath);
    });
  }

  // // xml tests
    it('must be able to download his full profile as an xml file with the export button  in language ', () => {
      // path of the file
      const fileToUpload = '../resources/profile.xml';
      const absolutePath = path.resolve(__dirname, fileToUpload);

      // unhide input (not working in firefox otherwise)
      browser.executeScript(
        'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
      );

      homePage.getFileReaderInput().sendKeys(absolutePath);
      expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

      testDownloadAsXml(
        './e2e/downloads/Doe_John_' + getDateAsYYYYMMDD() + '.xml',
        './e2e/resources/profile.xml');
    });

  function testDownloadAsXml(downloadedFilePath: string, expectedFilePath: string) {
    if (fs.existsSync(downloadedFilePath)) {
      // Make sure the browser doesn't have to rename the download.
      fs.unlinkSync(downloadedFilePath);
    }

    footerPage.getExportButton().click();
    footerPage.getExportAsXmlButton().click();

    browser.driver.wait(function () {
      // Wait until the file has been downloaded.
      return fs.existsSync(downloadedFilePath);
    }, 30000).then(function () {

      const expectedXml = fs.readFileSync(expectedFilePath, 'utf8');
      const downloadedXml = fs.readFileSync(downloadedFilePath, 'utf8');

      expect(downloadedXml).toBe(expectedXml);

      fs.unlinkSync(downloadedFilePath);
    });
  }


  // excel tests
  // /!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\
  // The resources files need to be generated with a linux backend.
  // The following tests will not working on windows.
  // /!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\

  [
    ['en'],
    ['it']
  ].forEach(([languageCode]) => {
    it('must be able to download his full profile as an excel file with the export button  in language ' + languageCode, () => {
      // path of the file
      const fileToUpload = '../resources/profile.xml';
      const absolutePath = path.resolve(__dirname, fileToUpload);

      headerPage.getChangeLanguageButton().click();
      languageSelectorPage.getLanguageButton(languageCode).click();

      // unhide input (not working in firefox otherwise)
      browser.executeScript(
        'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
      );

      homePage.getFileReaderInput().sendKeys(absolutePath);
      expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

      testDownloadAsExcel(
        './e2e/downloads/Doe_John_' + getDateAsYYYYMMDD() + '.xlsx',
        './e2e/resources/profile-' + languageCode + '.xlsx');
    });
  });

  it('must be able to download his full profile 2 as an excel file with the export button', () => {
    // path of the file
    const fileToUpload = '../resources/profile_2.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

    testDownloadAsExcel(
      './e2e/downloads/Doe_John_' + getDateAsYYYYMMDD() + '.xlsx',
      './e2e/resources/profile_2.xlsx');
  });

  it('must be able to download his profile full of no as an excel file with the export button', () => {
    // path of the file
    const fileToUpload = '../resources/profile_full_of_no.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

    testDownloadAsExcel(
      './e2e/downloads/jean_didier_' + getDateAsYYYYMMDD() + '.xlsx',
      './e2e/resources/profile_full_of_no.xlsx');
  });

  it('must be able to download his profile full of no 2 as an excel file with the export button', () => {
    // path of the file
    const fileToUpload = '../resources/profile_full_of_no_2.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

    testDownloadAsExcel(
      './e2e/downloads/jean_didier_' + getDateAsYYYYMMDD() + '.xlsx',
      './e2e/resources/profile_full_of_no_2.xlsx');
  });

  it('must be able to download his empty profile as an excel file with the export button', () => {
    // path of the file
    const fileToUpload = '../resources/empty.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(homePage.getFirstSection());

    testDownloadAsExcel(
      './e2e/downloads/empty_a_' + getDateAsYYYYMMDD() + '.xlsx',
      './e2e/resources/empty.xlsx');
  });

  function testDownloadAsExcel(downloadedFilePath: string, expectedFilePath: string) {
    if (fs.existsSync(downloadedFilePath)) {
      // Make sure the browser doesn't have to rename the download.
      fs.unlinkSync(downloadedFilePath);
    }

    footerPage.getExportButton().click();
    footerPage.getExportAsExcelButton().click();

    browser.driver.wait(function () {
      // Wait until the file has been downloaded.
      return fs.existsSync(downloadedFilePath);
    }, 60000).then(function () {

      let expectedWorkbook = new Workbook();
      let downloadedWorkbook = new Workbook();
      expectedWorkbook.xlsx.readFile(expectedFilePath).then(function () {
        downloadedWorkbook.xlsx.readFile(downloadedFilePath).then(function () {

          expectedWorkbook.eachSheet(function (expectedSheet, expectedSheetId) {
            let downloadedSheet = downloadedWorkbook.getWorksheet(expectedSheetId);
            expect(JSON.stringify(downloadedSheet.model))
            .toBe(JSON.stringify(expectedSheet.model));
          });

          fs.unlinkSync(downloadedFilePath);
        });
      });
    });
  }

});
