import * as path from 'path';
import {HeaderPage} from '../../../pages/header.po';
import {HomePage} from '../../../pages/home.po';
import {SummaryPage} from '../../../pages/summary/summary.po';
import {PersonalInformationData} from '../../../data/personal-information-data';
import {GeneralInformationValidator} from './validator/summary-general-information-validator';
import {browser, protractor} from 'protractor';
import {PersonalInfoGeneralPage} from '../../../pages/personal-info.general.po';
import {SummaryGeneralInformationPage} from '../../../pages/summary/personal-info/summary-general-information.po';

describe('A user on the general information part of the summary page', function () {
  let headerPage: HeaderPage;
  let homePage: HomePage;
  let personalInfoGeneralPage: PersonalInfoGeneralPage;
  let summaryPage: SummaryPage;
  let summaryGeneralInformationPage: SummaryGeneralInformationPage;
  let personalInformationData: PersonalInformationData;

  let generalInformationValidator: GeneralInformationValidator;

  beforeAll(() => {
    headerPage = new HeaderPage();
    homePage = new HomePage();
    personalInfoGeneralPage = new PersonalInfoGeneralPage();
    summaryPage = new SummaryPage();
    summaryGeneralInformationPage = new SummaryGeneralInformationPage();
    personalInformationData = new PersonalInformationData();

    generalInformationValidator = new GeneralInformationValidator();
  });

  it('must be able to hide all empty general information', () => {
    homePage.navigateTo();
    homePage.getCreateProfileButton().click();

    headerPage.getSummaryButton().click();
    generalInformationValidator.checkGeneralInformation(personalInformationData.emptyPersonalInformation);

    // hide all elements not filled
    summaryPage.getToggleButton().click();
    expect(summaryPage.isPersonalInfoSectionExpended())
      .toBe(false, 'The section need to be collapsed');
    summaryPage.getPersonalInfoButton().click();

    generalInformationValidator.checkGeneralInformationPresent(personalInformationData.emptyPersonalInformation);
  });

  it('must be able to not hide filled general information', () => {
    homePage.navigateTo();
    // path of the file
    const fileToUpload = '../../../resources/profile.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(summaryPage.getFirstSection());

    headerPage.getSummaryButton().click();
    generalInformationValidator.checkGeneralInformation(personalInformationData.fullPersonalInformation);

    // hide all elements not filled
    summaryPage.getToggleButton().click();
    expect(summaryPage.isPersonalInfoSectionExpended())
      .toBe(true, 'The section need to be expended');

    generalInformationValidator.checkGeneralInformationPresent(personalInformationData.fullPersonalInformation);
  });

  it('must be able to hide all empty identification document information', () => {
    homePage.navigateTo();
    homePage.getCreateProfileButton().click();

    headerPage.getSummaryButton().click();
    generalInformationValidator.checkIdentificationDocument(personalInformationData.emptyPersonalInformation);

    // hide all elements not filled
    summaryPage.getToggleButton().click();
    expect(summaryPage.isPersonalInfoSectionExpended())
      .toBe(false, 'The section need to be collapsed');
    summaryPage.getPersonalInfoButton().click();

    generalInformationValidator.checkIdentificationDocumentPresent(personalInformationData.emptyPersonalInformation);
  });

  it('must be able to not hide filled identification document information', () => {
    homePage.navigateTo();
    // path of the file
    const fileToUpload = '../../../resources/profile.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(summaryPage.getFirstSection());

    headerPage.getSummaryButton().click();
    generalInformationValidator.checkIdentificationDocument(personalInformationData.fullPersonalInformation);

    // hide all elements not filled
    summaryPage.getToggleButton().click();
    expect(summaryPage.isPersonalInfoSectionExpended())
      .toBe(true, 'The section need to be expended');

    generalInformationValidator.checkIdentificationDocumentPresent(personalInformationData.fullPersonalInformation);
  });

  it('must be able to not hide information from Didier Ridoux', () => {
    homePage.navigateTo();
    // path of the file
    const fileToUpload = '../../../resources/Didier_Ridoux_20180608.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(summaryPage.getFirstSection());

    headerPage.getSummaryButton().click();
    generalInformationValidator.checkIdentificationDocument(personalInformationData.didierRidoux);

    // hide all elements not filled
    summaryPage.getToggleButton().click();
    expect(summaryPage.isPersonalInfoSectionExpended())
      .toBe(true, 'The section need to be expended');

    generalInformationValidator.checkIdentificationDocumentPresent(personalInformationData.didierRidoux);
  });

  it('must be able to see the write name of each document added', () => {
    homePage.navigateTo();
    // path of the file
    const fileToUpload = '../../../resources/profile_full_of_no.xml';
    const absolutePath = path.resolve(__dirname, fileToUpload);

    // unhide input (not working in firefox otherwise)
    browser.executeScript(
      'document.getElementById("file-reader").style.visibility = "visible"; document.getElementById("file-reader").style.display = "inline"'
    );

    homePage.getFileReaderInput().sendKeys(absolutePath);
    expect(browser.getCurrentUrl()).toEqual(summaryPage.getFirstSection());

    headerPage.getSummaryButton().click();
    generalInformationValidator.checkTitleIdentificationDocuments(personalInformationData.profileFullOfNo);
  });

  it('should not be able to see the line other first names in the summary when the list is empty', () => {
    homePage.navigateTo();
    homePage.getCreateProfileButton().click();

    personalInfoGeneralPage.getAddFirstNameButton().click();
    personalInfoGeneralPage.getOtherFirstNameInput(0).sendKeys('toto');
    headerPage.getSummaryButton().click();
    expect(summaryGeneralInformationPage.getOtherFirstNames(0).getText())
      .toBe('toto');

    headerPage.getPersonalInformationButton().click();
    personalInfoGeneralPage.getRemoveFirstNameButton(0).click();
    headerPage.getSummaryButton().click();
    expect(summaryGeneralInformationPage.isOtherFirstNamesTitlePresent())
      .toBe(false);
  });

  it('should not be able to see the line other nationalities in the summary when the list is empty', () => {
    homePage.navigateTo();
    homePage.getCreateProfileButton().click();

    personalInfoGeneralPage.getAddOtherCountryButton().click();
    personalInfoGeneralPage.getOtherCountryOfNationalityDropDown().click();
    personalInfoGeneralPage.getCountryInput().sendKeys('Japan');
    personalInfoGeneralPage.getCountryInput().sendKeys(protractor.Key.ENTER);
    headerPage.getSummaryButton().click();
    expect(summaryGeneralInformationPage.getOtherNationalities().get(0).getText())
      .toBe('Japan');

    headerPage.getPersonalInformationButton().click();
    personalInfoGeneralPage.getRemoveOtherCountryButton(0).click();
    headerPage.getSummaryButton().click();
    expect(summaryGeneralInformationPage.isOtherNationalitiesTitlePresent())
      .toBe(false);
  });

});
