import {Component, OnInit, ViewContainerRef} from '@angular/core';
import {TranslateService} from './service/translate/translate.service';
import {Title} from '@angular/platform-browser';
import {VersionService} from './service/data/version.service';
import {AuthService} from './service/security/auth.service';
import {ToasterConfig} from 'angular2-toaster';
import {RouteService} from './service/navigation/route.service';
import {LanguageService} from './service/data/language.service';
import {Modal} from 'angular2-modal/plugins/bootstrap';

declare var $: any;

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: [],
  providers: [LanguageService]
})
export class AppComponent implements OnInit {

  public toasterConfig: ToasterConfig;

  constructor(public translate: TranslateService,
              private title: Title,
              private routeService: RouteService,
              private versionService: VersionService,
              private authService: AuthService,
              vcRef: ViewContainerRef,
              public modal: Modal) {
    // this language will be used as a fallback when a translation isn't found in the current language
    translate.setDefaultLanguage('en');

    // the lang to use, if the lang isn't available, it will use the current loader to get them
    translate.initAsPrimary(translate.getPrimaryLanguage());
    translate.initAsSecondary(translate.getSecondaryLanguage());

    // notification configuration
    this.toasterConfig = new ToasterConfig({
      positionClass: 'toast-bottom-right'
    });
    modal.overlay.defaultViewContainer = vcRef;
  }

  ngOnInit(): void {
    this.setVersion();
    this.processMetadata();
  }

  isAuthenticated(): boolean {
    return this.authService.isAuthenticated();
  }

  hasRole(roles: string[]): boolean {
    return this.authService.getCurrentUser().roles.some(function (element) {
      return roles.indexOf(element) > -1;
    });
  }

  getUserFullName(): string {
    return this.authService.getCurrentUser().firstName + ' ' + this.authService.getCurrentUser().lastName;
  }

  logout(): void {
    this.authService.logout();
  }

  private processMetadata() {
    this.routeService.getCurrentRoute()
    .subscribe((route: any) => {
      if (!!route.data) {
        let metadata = route.data['metadata'];
        this.updateMetadata(metadata ? metadata : {}, route.url);
      } else {
        this.updateMetadata({}, route.url);
      }
    });
  }

  private updateMetadata(metadata: any, currentUrl: string) {
    this.setPageTitle(metadata);
    this.setMetaTags(metadata);
  }

  private setMetaTags(data) {
    if (data.hasOwnProperty('description')) {
      this.translate.translate(data['description'], this.translate.getPrimaryLanguage()).subscribe((translated) => {
        this.setMetaTag('description', translated);
      });
    } else {
      this.translate.translate('page.default_description', this.translate.getPrimaryLanguage()).subscribe((translated) => {
        this.setMetaTag('description', translated);
      });
    }
  }

  private setPageTitle(data) {
    if (data.hasOwnProperty('title')) {
      this.translate.translate(data['title'], this.translate.getPrimaryLanguage()).subscribe((title) => {
        this.translate.translate('page.title_prefix', this.translate.getPrimaryLanguage()).subscribe((prefix) => {
          this.translate.translate('page.title_suffix', this.translate.getPrimaryLanguage()).subscribe((suffix) => {
            this.title.setTitle(prefix + title + suffix);
          });
        });
      });
    } else {
      this.translate.translate('page.default_title', this.translate.getPrimaryLanguage()).subscribe((translated) => {
        this.title.setTitle(translated);
      });
    }
  }

  private setVersion() {
    this.versionService.getVersion().subscribe(version => {
      this.setMetaTag('version', version.version);
    });
  }

  private setMetaTag(name: string, content: string) {
    let metaTag = $('meta[name="' + name + '"]');
    if (metaTag.length > 0) {
      metaTag.attr('content', content);
    } else {
      $('<meta>').attr('name', name).attr('content', content).appendTo('head');
    }
  }
}
