import {BrowserModule} from '@angular/platform-browser';
import {ErrorHandler, Injectable, NgModule} from '@angular/core';
import {FormsModule} from '@angular/forms';
import {Http, HttpModule, RequestOptions, XHRBackend} from '@angular/http';
import {AppComponent} from './app.component';
import {AppRoutingModule} from './app-routing.module';
import {HashLocationStrategy, LocationStrategy} from '@angular/common';
import {ProfileModule} from './component/profile/profile.module';
import {SharedModule} from './shared.module';
import {EscoService} from './service/data/esco.service';
import {CountryService} from './service/data/country.service';
import {IscedService} from './service/data/isced.service';
import {LanguageService} from './service/data/language.service';
import {NaceService} from './service/data/nace.service';
import {VersionService} from './service/data/version.service';
import * as Sentry from '@sentry/browser';
import {UserService} from './service/data/user.service';
import {AuthService} from './service/security/auth.service';
import {EcasAuthGuard} from './service/security/ecas-auth-guard.service';
import {DefaultRequestOptions} from './config/default-request-options';
import {CustomHttp} from './network/custom-http.service';
import {AuthStateService} from './service/security/auth-state.service';
import {ValidationService} from './service/validation/validation.service';
import {UserModule} from './component/user/user.module';
import {UtilsService} from './service/utils/utils.service';
import {RouteService} from './service/navigation/route.service';
import {HttpErrorService} from './service/error/http-error.service';
import {EscoLocalService} from './service/data/esco-local.service';
import {MoocModule} from './mooc/mooc.module';
import {HomeComponent} from './component/home/home.component';
import {BreadcrumbComponent} from './component/ecl-header/breadcrumb/breadcrumb.component';
import {EclHeaderComponent} from './component/ecl-header/ecl-header.component';
import {BreadcrumbItemComponent} from './component/ecl-header/breadcrumb/breadcrumb-item/breadcrumb-item.component';
import {EclFooterComponent} from './component/ecl-footer/ecl-footer.component';
import {CustomFooterComponent} from './component/ecl-footer/custom-footer/custom-footer.component';
import {LanguageSelectorComponent} from './component/ecl-header/language-selector/language-selector.component';
import {LanguageSelectorOverlayComponent} from './component/ecl-header/language-selector/language-selector-overlay/language-selector-overlay.component';
import {LanguageSelectorOverlayItemComponent} from './component/ecl-header/language-selector/language-selector-overlay/language-selector-overlay-item/language-selector-overlay-item.component';
import {ExportService} from './service/export/export.service';
import {SurveyModalComponent} from './component/common/survey-modal/survey-modal.component';
import {ErrorComponent} from './component/error/error.component';
import {AccessDeniedComponent} from './component/access-denied/access-denied.component';
import {NotFoundComponent} from './component/not-found/not-found.component';
import {SkipSectionGuard} from './service/guard/skip-section.guard';
import {InfoModalComponent} from './component/info-modal/info-modal.component';

@Injectable()
export class SentryErrorHandler  implements ErrorHandler {
  handleError(err: any): void {
    console.log(err);
    console.log(err.stack);
    if (window['sentryDsn']) {
      Sentry.captureException(err.originalError || err);
    }
  }
}

@NgModule({
  declarations: [
    AppComponent,
    HomeComponent,
    ErrorComponent,
    NotFoundComponent,
    BreadcrumbComponent,
    BreadcrumbItemComponent,
    AccessDeniedComponent,
    EclHeaderComponent,
    EclFooterComponent,
    CustomFooterComponent,
    SurveyModalComponent,
    LanguageSelectorComponent,
    LanguageSelectorOverlayComponent,
    LanguageSelectorOverlayItemComponent,
    InfoModalComponent
  ],
  imports: [
    SharedModule,
    BrowserModule,
    FormsModule,
    HttpModule,
    ProfileModule,
    UserModule,
    AppRoutingModule,
    MoocModule
  ],
  providers: [
    EscoService,
    EscoLocalService,
    CountryService,
    IscedService,
    NaceService,
    LanguageService,
    VersionService,
    UserService,
    AuthStateService,
    AuthService,
    ValidationService,
    RouteService,
    UtilsService,
    EcasAuthGuard,
    HttpErrorService,
    ExportService,
    SkipSectionGuard,
    {provide: RequestOptions, useClass: DefaultRequestOptions},
    {
      provide: Http,
      useClass: CustomHttp,
      deps: [XHRBackend, RequestOptions, AuthStateService],
      useFactory: httpFactory
    },
    {provide: LocationStrategy, useClass: HashLocationStrategy},
    {provide: ErrorHandler, useClass: SentryErrorHandler}
  ],
  entryComponents: [
    SurveyModalComponent,
    InfoModalComponent
  ],
  bootstrap: [AppComponent]
})
export class AppModule {

  constructor() {
    if (window['sentryDsn']) {
      Sentry.init({
        dsn: window['sentryDsn']
      });
    }
  }
}

export function httpFactory(backend: XHRBackend, options: RequestOptions,
                            authStateService: AuthStateService) {
  return new CustomHttp(backend, options, authStateService);
}
