import {AfterViewInit, Component, ElementRef, Input, OnDestroy, OnInit, Renderer, ViewChild} from '@angular/core';
import {Subscription} from 'rxjs';
import {EventEmitter} from '@angular/forms/src/facade/async';
import {Output} from '@angular/core/src/metadata/directives';
import {AbstractControlValueAccessor, makeProvider} from '../../../common/abstract-control-value-accessor.component';
import {LanguageChangeEvent, TranslateService} from '../../../../service/translate/translate.service';
import {EscoService} from '../../../../service/data/esco.service';

declare var $: any;

/**
 * Component to display a dropdown field to select an ESCO value.
 */
@Component({
  selector: 'app-esco',
  templateUrl: './esco.component.html',
  styleUrls: [],
  providers: [
    makeProvider(EscoComponent)
  ]
})
export class EscoComponent extends AbstractControlValueAccessor implements OnInit, AfterViewInit, OnDestroy {

  /**
   * The HTML id attribute of the dropdown input created by this component.
   */
  @Input()
  htmlId: string;

  /**
   * The HTML disabled attribute of the dropdown input created by this component.
   */
  @Input()
  htmlDisabled: string;

  /**
   * The HTML class attribute of the input created by this component.
   */
  @Input()
  htmlClass: string;

  /**
   * The ESCO classification to filter values that can be selected.
   */
  @Input()
  classification: string;

  /**
   * The language to filter values that can be selected.
   */
  @Input()
  lang: string;
  /**
   * experience object of which the skills component belongs to.
   */
  @Input()
  experience: any;

  /**
   * The language to filter values that can be selected.
   */
  @Input()
  selectLanguage: boolean;

  /**
   * Attribute to know if the current country component is part of a collection
   */
  @Input()
  isMulti: boolean;

  secondary = {
    'selected': false
  };

  options: Object;

  initialData: Array<Object>;

  @ViewChild('selectElem')
  selectElem: ElementRef;

  /**
   * Attribute to know if the current element should get the focus
   */
  @Input()
  shouldFocus: boolean;

  @Output()
  created: EventEmitter<number> = new EventEmitter<any>();

  private select2Translations = {
    searching: '',
    loading: ''
  };

  private onPrimaryLanguageChange: Subscription;
  private onSecondaryLanguageChange: Subscription;

  constructor(private translateService: TranslateService, private renderer: Renderer, private escoService: EscoService) {
    super();

    this.onPrimaryLanguageChange = translateService.onPrimaryLanguageChange.subscribe((langChangeEvent: LanguageChangeEvent) => {
      this.changeLang();
    });

    this.onSecondaryLanguageChange = translateService.onSecondaryLanguageChange.subscribe((langChangeEvent: LanguageChangeEvent) => {
      this.changeLang();
    });
  }

  get childValue(): any {
    return this.value;
  }

  set childValue(value) {
    $(this.selectElem.nativeElement).val(value).trigger('change');
    this.value = value;
  }

  changeLang(): void {
    this.writeValue(this.childValue);
    this.setSelect2Translations();
  }

  setSelect2Translations(): void {
    this.translateService.translate('component.select2.loading', this.translateService.getPrimaryLanguage())
      .subscribe((v) => {
        this.select2Translations.loading = v;
        let elem = $(this.selectElem.nativeElement);
        elem.select2(this.options);
      });
    this.translateService.translate('component.select2.searching', this.translateService.getPrimaryLanguage())
      .subscribe((v) => {
        this.select2Translations.searching = v;
        let elem = $(this.selectElem.nativeElement);
        elem.select2(this.options);
      });

  }

  writeValue(value: any) {
    let elem = $(this.selectElem.nativeElement);

    if (value) {
      this.escoService.getEscoById(value, this.classification, this.translateService.primaryLanguage,
        this.translateService.secondaryLanguage).subscribe(
        (response) => {
          this.initialData = [{
            id: value,
            primaryLanguage: this.translateService.primaryLanguage,
            secondaryLanguage: this.translateService.secondaryLanguage,
            fallbackLanguage: this.translateService.getDefaultLanguage(),
            internal: response,
            get text(): string {
              if (this.internal && this.internal.preferredLabel[this.primaryLanguage] && this.internal.preferredLabel[this.secondaryLanguage]
                && this.primaryLanguage !== this.secondaryLanguage) {
                return this.internal.preferredLabel[this.primaryLanguage] + ' / ' + this.internal.preferredLabel[this.secondaryLanguage];
              } else if (this.internal.preferredLabel && this.internal.preferredLabel[this.primaryLanguage]) {
                return this.internal.preferredLabel[this.primaryLanguage];
              } else if (this.internal.preferredLabel && this.internal.preferredLabel[this.secondaryLanguage]) {
                return this.internal.preferredLabel[this.secondaryLanguage];
              } else if (this.internal.preferredLabel && this.internal.preferredLabel[this.fallbackLanguage]) {
                return this.internal.preferredLabel[this.fallbackLanguage];
              } else {
                return '';
              }
            }
          }];
          elem.val(value).trigger('change');
          super.writeValue(value);
        }
      );
    } else {
      elem.val(value).trigger('change');
      super.writeValue(value);
    }


  }

  ngAfterViewInit(): void {
    let elem = $(this.selectElem.nativeElement);
    elem.select2(this.options);
    elem.val(this.value).trigger('change');
    elem.on('change', (e) => {
      this.onValueChanged(e);
    });
    elem.on(
      'select2:select',
      function () {
        $(this).focus();
      }
    );
    elem.on(
      'select2:close',
      function () {
        $(this).focus();
      }
    );
    if (this.shouldFocus) {
      setTimeout(() => {
        this.renderer.invokeElementMethod(elem, 'focus');
        this.created.emit();
      });
    }
  }

  onValueChanged(event): void {
    this.value = event.target.value;
  }

  ngOnInit(): void {

    this.setSelect2Translations();
    let translateService = this.translateService;
    let classification = this.classification.toLowerCase();
    let secondary = this.secondary;
    let experience = this.experience;
    this.options = {
        width: '100%',
        // dropdownAutoWidth: true,
        allowClear: true,
        placeholder: '',
        language: {
          searching: () => {
            return this.select2Translations.searching;
          },
          loadingMore: () => {
            return this.select2Translations.loading;
          }
        },
        ajax: {
          url: function(params) {
            let baseUrl = window['escoApiEndpoint'];
            let apiMethod = '/search';
            if (classification === 'skill' && ( !params.term || params.term.length <= 0) &&
              experience.occupation != null && experience.occupation !== '') {
              apiMethod = '/resource/related/';
            }
            return baseUrl + apiMethod;
          },
          traditional: true, //  traditional style of param serialization (foo: [1,2,3] => foo=1&foo=2&foo=3)
          processResults: function (data, params) {
            let bindResult = function(value) {
              return {
                id: value.uri,
                primaryLanguage: translateService.primaryLanguage,
                secondaryLanguage: translateService.secondaryLanguage,
                fallbackLanguage: translateService.getDefaultLanguage(),
                internal: value,
                get text(): string {
                  if (this.internal && this.internal.preferredLabel[this.primaryLanguage] && this.internal.preferredLabel[this.secondaryLanguage]
                    && this.primaryLanguage !== this.secondaryLanguage) {
                    return this.internal.preferredLabel[this.primaryLanguage]
                      + ' / ' + this.internal.preferredLabel[this.secondaryLanguage];
                  } else if (this.internal.preferredLabel && this.internal.preferredLabel[this.primaryLanguage]) {
                    return this.internal.preferredLabel[this.primaryLanguage];
                  } else if (this.internal.preferredLabel && this.internal.preferredLabel[this.secondaryLanguage]) {
                    return this.internal.preferredLabel[this.secondaryLanguage];
                  } else if (this.internal.preferredLabel && this.internal.preferredLabel[this.fallbackLanguage]) {
                    return this.internal.preferredLabel[this.fallbackLanguage];
                  } else {
                    return '';
                  }
                }
              };
            };
            if (classification === 'skill' && (!params.term || params.term.length <= 0) &&
              experience.occupation != null && experience.occupation !== '') {
              return {
                results: data._embedded.hasEssentialSkill.map(function (value, index) {
                  return bindResult(value);
                }),
                pagination: {
                  more: !data.last
                }
              };
            } else {
              return {
                results: data._embedded.results.map(function (value, index) {
                  return bindResult(value);
                }),
                pagination: {
                  more: !data.last
                }
              };
            }
          },
          data: function (params) {
            let occupationId = experience != null ? experience.occupation : '';

            let filterType = classification;
            if (classification === 'occupation') {
              filterType += ',concept';
            }

            return {
              type: filterType,
              uri: occupationId,
              relation: 'hasEssentialSkill',
              language: secondary.selected ? translateService.secondaryLanguage : translateService.primaryLanguage,
              text: (params.term && params.term.indexOf(' ') >= 0) ? '"' + params.term + '"' : params.term,
              offset: params.page ? params.page - 1 : 0,
              limit: 20
            };
          }
        }
      };
  }

  ngOnDestroy(): void {
    this.onPrimaryLanguageChange.unsubscribe();
    this.onSecondaryLanguageChange.unsubscribe();
  }
}
