import {
  AfterViewInit,
  Component,
  ElementRef,
  EventEmitter,
  Input,
  OnDestroy,
  OnInit,
  Output,
  Renderer,
  ViewChild
} from '@angular/core';
import {Subscription} from 'rxjs';
import {LanguageChangeEvent, TranslateService} from '../../../../service/translate/translate.service';
import {IscedService} from '../../../../service/data/isced.service';

declare var $: any;

/**
 * Component to display a field for ISCED FoET.
 */
@Component({
  selector: 'app-isced-foet',
  templateUrl: './isced-foet.component.html',
  styleUrls: []
})
export class IscedFoetComponent implements OnInit, AfterViewInit, OnDestroy {

  private valueInternal: string;

  @Input()
  get value(): string {
    return this.valueInternal;
  }

  set value(value: string) {
    if (value !== this.valueInternal) {
      this.valueInternal = value;
      this.writeValue(value);
      this.valueChange.emit(value);
    }
  }

  get select2Value(): string {
    return this.valueInternal;
  }

  set select2Value(value: string) {
    if (value !== this.valueInternal) {
      this.valueInternal = value;
      this.valueChange.emit(value);
    }
  }

  @Output()
  valueChange = new EventEmitter<string>();

  /**
   * The HTML id attribute of the dropdown input created by this component.
   */
  @Input()
  htmlId: string;

  /**
   * The HTML disabled attribute of the dropdown input created by this component.
   */
  @Input()
  htmlDisabled: string;

  /**
   * The HTML class attribute of the input created by this component.
   */
  @Input()
  htmlClass: string;

  options: Object;

  initialData: Array<Object>;

  @ViewChild('selectElem')
  selectElem: ElementRef;

  /**
   * Attribute to know if the current element should get the focus
   */
  @Input()
  shouldFocus: boolean;

  @Output()
  created: EventEmitter<number> = new EventEmitter<any>();

  @Output()
  change = new EventEmitter();

  private select2Translations = {
    searching: '',
    loading: ''
  };

  private onPrimaryLanguageChange: Subscription;
  private onSecondaryLanguageChange: Subscription;

  constructor(private translateService: TranslateService, private renderer: Renderer, private iscedService: IscedService) {
    this.onPrimaryLanguageChange = translateService.onPrimaryLanguageChange.subscribe((langChangeEvent: LanguageChangeEvent) => {
      this.changeLang();
    });

    this.onSecondaryLanguageChange = translateService.onSecondaryLanguageChange.subscribe((langChangeEvent: LanguageChangeEvent) => {
      this.changeLang();
    });
  }

  changeLang(): void {
    this.writeValue(this.value);
    this.setSelect2Translations();
  }

  setSelect2Translations(): void {
    this.translateService.translate('component.select2.loading', this.translateService.getPrimaryLanguage())
      .subscribe((v) => {
        this.select2Translations.loading = v;
        let elem = $(this.selectElem.nativeElement);
        elem.select2(this.options);
      });
    this.translateService.translate('component.select2.searching', this.translateService.getPrimaryLanguage())
      .subscribe((v) => {
        this.select2Translations.searching = v;
        let elem = $(this.selectElem.nativeElement);
        elem.select2(this.options);
      });

  }

  writeValue(value: string) {
    let elem = $(this.selectElem.nativeElement);

    if (value) {
      this.iscedService.getIscedById(value).subscribe(
        (response) => {
          this.initialData = [{
            id: value,
            primaryLanguage: this.translateService.primaryLanguage,
            secondaryLanguage: this.translateService.secondaryLanguage,
            fallbackLanguage: this.translateService.getDefaultLanguage(),
            internal: response,
            get text(): string {
              if (this.internal.values && this.internal.values[this.primaryLanguage] && this.internal.values[this.secondaryLanguage]
                && this.primaryLanguage !== this.secondaryLanguage) {
                return this.internal.values[this.primaryLanguage].value + ' / ' + this.internal.values[this.secondaryLanguage].value;
              } else if (this.internal.values && this.internal.values[this.primaryLanguage]) {
                return this.internal.values[this.primaryLanguage].value;
              } else if (this.internal.values && this.internal.values[this.secondaryLanguage]) {
                return this.internal.values[this.secondaryLanguage].value;
              } else if (this.internal.values && this.internal.values[this.fallbackLanguage]) {
                return this.internal.values[this.fallbackLanguage].value;
              } else {
                return '';
              }
            }
          }];
          elem.val(value).trigger('change');
        }
      );
    } else {
      elem.val(value).trigger('change');
    }


  }

  ngAfterViewInit(): void {
    let elem = $(this.selectElem.nativeElement);
    elem.select2(this.options);
    this.writeValue(this.valueInternal);
    elem.on('change', (e) => {
      this.onValueChanged(e);
    });
    elem.on(
      'select2:select',
      function () {
        $(this).focus();
      }
    );
    elem.on(
      'select2:close',
      function () {
        $(this).focus();
      }
    );
    elem.on('select2:selecting', () => {
      this.hasChanged();
    });
    if (this.shouldFocus) {
      setTimeout(() => {
        this.renderer.invokeElementMethod(elem, 'focus');
        this.created.emit();
      });
    }
  }

  onValueChanged(event): void {
    this.select2Value = event.target.value;
  }

  ngOnInit(): void {

    this.setSelect2Translations();

    let translateService = this.translateService;

    let url = './api/isceds';

    this.options = {
      width: '100%',
      // dropdownAutoWidth: true,
      allowClear: true,
      placeholder: '',
      language: {
        searching: () => {
          return this.select2Translations.searching;
        },
        loadingMore: () => {
          return this.select2Translations.loading;
        }
      },
      ajax: {
        url: url,
        processResults: function (data, params) {
          return {
            results: data.content.map(function (value, index) {
              return {
                id: value.id,
                primaryLanguage: translateService.primaryLanguage,
                secondaryLanguage: translateService.secondaryLanguage,
                fallbackLanguage: translateService.getDefaultLanguage(),
                internal: value,
                get text(): string {
                  if (this.internal.values && this.internal.values[this.primaryLanguage] && this.internal.values[this.secondaryLanguage]
                    && this.primaryLanguage !== this.secondaryLanguage) {
                    return this.internal.values[this.primaryLanguage].value + ' / ' + this.internal.values[this.secondaryLanguage].value;
                  } else if (this.internal.values && this.internal.values[this.primaryLanguage]) {
                    return this.internal.values[this.primaryLanguage].value;
                  } else if (this.internal.values && this.internal.values[this.secondaryLanguage]) {
                    return this.internal.values[this.secondaryLanguage].value;
                  } else if (this.internal.values && this.internal.values[this.fallbackLanguage]) {
                    return this.internal.values[this.fallbackLanguage].value;
                  } else {
                    return '';
                  }
                }
              };
            }),
            pagination: {
              more: !data.last
            }
          };
        },
        data: function (params) {
          return {
            q: params.term,
            pageSize: 20,
            page: params.page ? params.page - 1 : 0,
            searchLanguage: translateService.secondaryLanguage
          };
        }
      }
    };
  }

  ngOnDestroy(): void {
    this.onPrimaryLanguageChange.unsubscribe();
    this.onSecondaryLanguageChange.unsubscribe();
  }

  hasChanged() {
    this.change.emit();
  }
}
