import {EventEmitter, Injectable} from '@angular/core';
import {Headers, Http, RequestOptions} from '@angular/http';
import {Observable, Observer} from 'rxjs';
import {ValidationService} from '../../service/validation/validation.service';

declare var $: any;

export let menuItemCreationEmitter: EventEmitter<any> = new EventEmitter();

/**
 * Service handling profile logic.
 */
@Injectable()
export class ProfileService {

  navigationButton = 'next';
  profile: Object;
  showEmpty = true;

  constructor(private http: Http,
              public validationService: ValidationService) {
    this.profile = this.newProfile();
  }

  private newProfile() {
    return {
      personalInfo: {
        firstNames: [],
        contactInfo: {
          phoneNumber: {},
          email: {},
          phoneNumbers: [],
          emails: [],
        },
        birthDate: {},
        dateOfArrival: {},
        nationalities: [],
        otherMaritalStatuses: [],
        identificationDocuments: [],
        otherResidencyStatuses: [],
        education: {
          primary: {},
          secondary: {},
          vocational: {},
          higher: {},
          other: {}
        }
      },
      expectations: [],
      skillsIdentification: {
        languageSkills: {
          otherMotherTongue: [],
          hostCountry: {},
          preferredGeneral: {},
          preferredProfessional: {},
          otherLanguages: [],
        },
        educationTraining: {
          studyFields: [],
          otherStudyFields: [],
          educationType: {},
          certificates: [],
        },
        literacySkills: {
          readPaperMediaType: {},
          readLinerMediaType: {},
          writeMediaType: {},
        },
        numeracySkills: {
          measuring: {
            calculator: {},
            usage: {}
          },
          calculationWholeNumbers: {
            calculator: {},
            usage: {}
          },
          calculationFractions: {
            calculator: {},
            usage: {}
          },
          calculationPercentage: {
            calculator: {},
            usage: {}
          },
          geometry: {
            calculator: {},
            usage: {}
          },
          complex: {
            calculator: {},
            usage: {}
          },
        },
        outsideSkills: {
          caringChildren: {},
          caringElderlyPeople: {},
          caringSickPeople: {},
          makingClothing: {},
          preparingMeals: {},
          cultivatingCrops: {},
          livestock: {},
          makingPottery: {},
          sellingTradingProducts: {},
          makingFurniture: {},
          houseConstruction: {},
          coachingTeaching: {},
        },
        otherBasicTransversalSkills: {},
        digitalSkills: {
          others: [],
        },
        professionalSkills: {
          experiences: [],
          otherLanguages: []
        },
        drivingSkills: {
          carriage: {},
          twoWheeled: {},
          manual: {},
          automatic: {},
          heavyTruck: {},
          bus: {},
          tram: {},
          train: {},
          machinery: {},
          vessel: {},
          forkLift: {},
          constructionMachinery: {},
          aircraft: {},
          other: {}
        },
        otherSkills: []
      },
      outcome: {
        organisations: [],
        courses: {
          selectedCourses: [],
          filters: {}
        },
        recommendations: {
          courseRecommendations: []
        }
      }
    };
  }

  get emails(): string[] {
    let emails = [];
    const contactInfo = this.profile['personalInfo']['contactInfo'];

    // primary email
    if (this.isEmailValid(contactInfo['email']['prefix'], contactInfo['email']['suffix'])) {
      const email = contactInfo['email']['prefix'] + '@' + contactInfo['email']['suffix'];
      emails.push(email);
    }

    return emails.concat(this.secondaryEmails);
  }

  get secondaryEmails(): string[] {
    let emails = [];
    const contactInfo = this.profile['personalInfo']['contactInfo'];

    // secondary emails
    contactInfo['emails'].forEach(elm => {
      if (this.isEmailValid(elm['prefix'], elm['suffix'])) {
        const email = elm['prefix'] + '@' + elm['suffix'];
        // check if the email is not already in the list
        if (!emails.some(mail => mail === email)) {
          emails.push(email);
        }
      }
    });

    return emails;
  }

  isEmailValid(prefix: string, suffix: string): boolean {
    if (prefix && suffix) {
      const mail = prefix + '@' + suffix;
      if (mail.match(this.validationService.mailRegex)) {
        return true;
      }
    }
    return false;
  }

  getOtherFirstNames(): string[] {
    let firstNames = [];

    this.profile['personalInfo']['firstNames'].forEach(elm => {
      if (this.isFirstNameValid(elm['name'])) {
        firstNames.push(elm['name']);
      }
    });

    return firstNames;
  }

  isFirstNameValid(firstName: string): boolean {
    const emptyString = new RegExp('^[ ]+$');
    return firstName && !firstName.match(emptyString);
  }

  getOtherNationalities(): string[] {
    let nationalities = [];

    this.profile['personalInfo']['nationalities'].forEach(elm => {
      if (this.isNationalityValid(elm['name'])) {
        nationalities.push(elm['name']);
      }
    });

    return nationalities;
  }

  isNationalityValid(nationality: string): boolean {
    const emptyString = new RegExp('^[ ]+$');
    return nationality && !nationality.match(emptyString);
  }

  getOtherPhoneNumbers(): string[] {
    let phoneNumbers = [];

    this.profile['personalInfo']['contactInfo']['phoneNumbers'].forEach(elm => {
      if (this.isPhoneNumberValid(elm['prefix'], elm['suffix'])) {
        phoneNumbers.push(elm['prefix'] + elm['suffix']);
      }
    });

    return phoneNumbers;
  }

  isPhoneNumberValid(prefix: string, suffix: string): boolean {
    if (!(prefix && suffix)) {
      return false;
    }
    const phoneNumber = prefix + suffix;
    return phoneNumber.match(this.validationService.phoneNumberRegex) == null ? null : true;

  }

  sendStatistics(): Observable<void> {

    return Observable.create((observer: Observer<void>) => {
      let formData = new FormData();
      let jsonProfile = JSON.stringify(this.profile, function (key, value) {
        if (typeof value === 'string') {
          value = value.replace(/'/g, '&#39;');
        }
        return value;
      });

      let headers = new Headers({'Content-Type': 'application/json'});
      let requestOptions = new RequestOptions({headers: headers});
      this.http.post('./api/statistics', jsonProfile, requestOptions)
        .map(response => response.json())
        .catch(error => {
          observer.error(error);
          return Observable.throw(error);
        }).subscribe(() => observer.complete());
    });

  }

  load(profile: File): Observable<void> {
    return Observable.create((observer: Observer<void>) => {

      this.clear();

      let formData = new FormData();
      formData.append('file', profile);

      let headers = new Headers({}); // this will default to multipart/form-data with boundary
      let requestOptions = new RequestOptions({headers: headers});

      this.http.post('./api/import', formData, requestOptions)
        .map(response => response.json())
        .catch(error => {
          observer.error(error);
          return Observable.throw(error);
        })
        .subscribe(json => {
            this.profile = $.extend(true, json, this.profile);

            this.initDateFields();

            observer.next(void (0));
            observer.complete();
          }
        );
    });
  }

  private initDateFields() {
    let emptyDate = JSON.stringify({
      'day': 0,
      'month': 0,
      'year': 0
    });

    if (JSON.stringify(this.profile['personalInfo']['birthDate']) === emptyDate) {
      this.profile['personalInfo']['birthDate'] = {};
    }

    if (JSON.stringify(this.profile['personalInfo']['dateOfArrival']) === emptyDate) {
      this.profile['personalInfo']['dateOfArrival'] = {};
    }
  }

  clear(): void {
    for (let prop in this.profile) {
      if (this.profile.hasOwnProperty(prop)) {
        delete this.profile[prop];
      }
    }
    $.extend(this.profile, this.newProfile());
  }

}
