import {Component, OnInit, ViewContainerRef, OnDestroy} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {TranslateService} from '../../service/translate/translate.service';
import {User} from '../../service/data/user';
import {UserService} from '../../service/data/user.service';
import {PageWrapper} from '../../service/common/page.wrapper';
import {Modal} from 'angular2-modal/plugins/bootstrap';
import {UserEditModal} from './modal/user-edit.modal.component';
import {BSModalContext} from 'angular2-modal/plugins/bootstrap';
import {ToasterService} from 'angular2-toaster';
import {overlayConfigFactory} from 'angular2-modal';
import {PageRequest} from '../../service/common/page.request';
import {NgProgressService} from 'ng2-progressbar';
import {UtilsService} from '../../service/utils/utils.service';
import {AuthService} from '../../service/security/auth.service';
import {Subscription} from 'rxjs';

@Component({
  selector: 'app-user-list',
  templateUrl: './user-list.component.html',
  styleUrls: []
})
export class UserListComponent implements OnInit, OnDestroy {

  pageSubscriber: Subscription;
  users: PageWrapper<User>;
  currentPage = 0;
  paginationPage = 1;
  sort: any;

  constructor(private route: ActivatedRoute,
              private router: Router,
              private userService: UserService,
              public translate: TranslateService,
              private modal: Modal,
              private toasterService: ToasterService,
              private pService: NgProgressService,
              private utilsService: UtilsService,
              private authService: AuthService) {

    this.users = new PageWrapper<User>();
    this.sort = {field: 'username', direction: 'ASC', descending: false};
  }

  ngOnInit() {
    this.pageSubscriber = this.route
      .queryParams
      .subscribe(params => {
        this.paginationPage = +params['page'] || 1;
        this.currentPage = this.paginationPage - 1;
        this.getUsers(this.currentPage, this.sort.field, this.sort.direction);
      });
  }

  ngOnDestroy() {
    this.pageSubscriber.unsubscribe();
  }

  /**
   * Get the list of users.
   *
   * @param page            the page number
   * @param sortField       the field used for sorting
   * @param sortDirection   the direction of the sort
   */
  getUsers(page: number, sortField?: string, sortDirection?: string): void {
    this.pService.start();
    this.userService.getUsers(new PageRequest(page, 10, sortField, sortDirection))
      .subscribe(response => {
        this.users = response;
        this.pService.done();
      });
  }

  /**
   * Create a user.
   */
  createUser() {
    this.modal.open(UserEditModal, overlayConfigFactory({user: new User()}, BSModalContext))
      .then((resultPromise) => {
        resultPromise.result.then((result) => {
            this.userService.saveUser(result)
              .subscribe((user: User) => {
                this.getUsers(this.currentPage);
                this.toasterService.pop('success', null, this.translate.instant('notification.user.create.success',
                  this.translate.primaryLanguage));
              });
        }, () => {});
      });
  }

  /**
   * Update an esiting user.
   *
   * @param user  the user to update.
   */
  editUser(user: User) {
    this.modal.open(UserEditModal, overlayConfigFactory({user: this.utilsService.clone(user)}, BSModalContext))
      .then((resultPromise) => {
        resultPromise.result.then((result) => {
          this.userService.saveUser(result).subscribe((res: User) => {
            this.getUsers(this.currentPage);
            this.toasterService.pop('success', null, this.translate.instant('notification.user.update.success',
              this.translate.primaryLanguage));
          });
        }, () => {});
      });
  }

  /**
   * Delete an exisitng user.
   *
   * @param user  the user to delete.
   */
  deleteUser(user: User) {
    this.modal
      .confirm()
      .title(this.translate.instant('modal.confirm.user.delete.title', this.translate.primaryLanguage))
      .body(this.translate.instant('modal.confirm.user.delete.message', this.translate.primaryLanguage))
      .okBtn(this.translate.instant('common.ok', this.translate.primaryLanguage))
      .cancelBtn(this.translate.instant('common.cancel', this.translate.primaryLanguage))
      .open()
      .then((resultPromise) => {
        resultPromise.result.then(result => {
          this.userService.deleteUser(user.id).subscribe(response => {
            this.getUsers(this.currentPage);
            this.toasterService.pop('success', null, this.translate.instant('notification.user.delete.success',
              this.translate.primaryLanguage));
          });
        }, () => {});
      });
  }

  /**
   * Check if user can be deleted.
   *
   * @param user  the current user.
   * @returns {boolean}
   */
  canDeleteUser(user: User): boolean {
    let currentUser: User = this.authService.getCurrentUser();

    // Cannot delete self
    if (currentUser.id === user.id) {
      return false;
    }

    // Root cannot be deleted
    if (currentUser.roles.indexOf('ROOT') === -1 && user.roles.indexOf('ROOT') > -1) {
      return false;
    }

    return true;
  }

  /**
   * Event triggered when table sorting changed.
   *
   * @param sortField the field used for sorting.
   */
  sortChanged(sortField: string): void {
    let sort = this.sort;
    if (sort.field === sortField) {
      sort.descending = !sort.descending;
    } else {
      sort.field = sortField;
      sort.descending = false;
    }
    sort.direction = sort.descending ? 'DESC' : 'ASC';

    this.getUsers(this.currentPage, sort.field, sort.direction);
  }

  /**
   * Event triggered when the resut page changed.
   *
   * @param page  the new page.
   */
  pageChanged(page: number) {
    this.router.navigate(['users'], { queryParams: { page: page } });
  }

}
