import {Directive, ElementRef, Input, OnDestroy, AfterViewInit} from '@angular/core';
import {Subscription} from 'rxjs';
import {TranslateService, LanguageChangeEvent} from '../service/translate/translate.service';
import {isDefined} from '@ng-bootstrap/ng-bootstrap/util/util';

@Directive({
  selector: '[appTranslate]'
})
export class TranslateDirective implements OnDestroy, AfterViewInit {
  key: string;
  lastLanguage: string;
  lastParams: any;

  onPrimaryLanguageChangeSub: Subscription;
  onSecondaryLanguageChangeSub: Subscription;

  @Input() set appTranslate(key: string) {
    if (key) {
      this.key = key;
      this.updateElement(false, this.translateTarget);
    }
  }

  @Input() translateTarget = 'primary';
  @Input() translateParams: any;

  private subscription: Subscription;

  constructor(private translateService: TranslateService, private element: ElementRef) {

    this.onPrimaryLanguageChangeSub = this.translateService.onPrimaryLanguageChange.subscribe((event: LanguageChangeEvent) => {
      this.updateElement(true, event.target, event.language);
    });

    this.onSecondaryLanguageChangeSub = this.translateService.onSecondaryLanguageChange.subscribe((event: LanguageChangeEvent) => {
      this.updateElement(true, event.target, event.language);
    });
  }

  ngAfterViewInit() {
    this.updateElement(false, this.translateTarget);
  }

  private updateElement(forceUpdate = false, target: string, language?: string) {
    if (this.translateTarget === target) {

      // Check if element has child nodes, if not create a default text child node
      if (!this.element.nativeElement.hasChildNodes()) {
        this.element.nativeElement.appendChild(document.createTextNode(''));
      }

      // Update all nodes with the new translation
      let nodes: NodeList = this.element.nativeElement.childNodes;
      for (let i = 0; i < nodes.length; ++i) {
        let node: any = nodes[i];
        if (node.nodeType === 3) { // node type 3 is a text node
          let key: string;
          if (this.key) {
            key = this.key;
          } else {
            let content = node.textContent.trim();
            if (content.length) {
              // we want to use the content as a key, not the translation value
              if (content !== node.currentValue) {
                key = content;
                // the content was changed from the user, we'll use it as a reference if needed
                node.originalContent = node.textContent;
              } else if (node.originalContent && forceUpdate) { // the content seems ok, but the lang has changed
                node.lastKey = null;
                // the current content is the translation, not the key, use the last real content as key
                key = node.originalContent.trim();
              }
            }
          }

          this.updateValue(key, language, node);
        }
      }
    }
  }

  private updateValue(key: string, language: string, node: any) {
    language = language || this.translateService.getLanguageByTarget(this.translateTarget);

    if (key) {
      let interpolateParams: Object = this.translateParams;
      if (node.lastKey === key && this.lastLanguage === language && this.lastParams === interpolateParams) {
        return;
      }

      this.lastLanguage = language;
      this.lastParams = interpolateParams;

      if (this.subscription && !this.subscription.closed) {
        this.subscription.unsubscribe();
      }

      this.subscription = this.translateService
        .translate(key, language, interpolateParams)
        .subscribe(
          data => {
            this.setNodeValue(node, data, key);
          },
          error => {
            node.lastKey = key;

            this.translateService
              .translate(key, this.translateService.getDefaultLanguage(), interpolateParams)
              .subscribe(
                data => {
                  this.setNodeValue(node, data, key);
                }
              );
          });
    }
  }

  private setNodeValue(node: any, data: string, key: string) {
    if (data !== key) {
      node.lastKey = key;
    }
    if (!node.originalContent) {
      node.originalContent = node.textContent;
    }
    node.currentValue = isDefined(data) ? data : (node.originalContent || key);
    // we replace in the original content to preserve spaces that we might have trimmed
    node.textContent = this.key ? node.currentValue : node.originalContent.replace(key, node.currentValue);
  }

  ngOnDestroy() {
    if (this.onPrimaryLanguageChangeSub) {
      this.onPrimaryLanguageChangeSub.unsubscribe();
    }

    if (this.onSecondaryLanguageChangeSub) {
      this.onSecondaryLanguageChangeSub.unsubscribe();
    }
  }
}
