import {Component, EventEmitter, OnDestroy, OnInit} from '@angular/core';
import {ElasticSearchService} from '../../service/elastic-search.service';
import {SearchFilters} from '../../service/search/searchfilter/search-filters';
import {ElasticSearchResult} from '../../service/search/elastic-search-result';
import {Topics} from '../../model/Topics';
import {IdTextElement, jsonToIdTextElement} from '../../model/id-text-element';
import {idToTopNode, IscedObj} from '../../model/IscedObj';
import {EducationLevels} from '../../model/EducationLevels';
import {MultiMatchFilter} from '../../service/search/searchfilter/filter/multi-match-filter';
import {UtilityFunctions} from '../../util/UtilityFunctions';
import {TermsFilterIsceds} from '../../service/search/searchfilter/filter/terms-filter-isceds';
import {ProfileService} from '../../../component/profile/profile.service';
import {Providers} from '../../model/Providers';
import {LanguageService} from '../../../service/data/language.service';
import {HttpErrorService} from '../../../service/error/http-error.service';
import {SectionAbstractComponent} from '../../../component/common/section-abstract.component';
import {MenuItem} from '../../../component/profile/menu/menu-item';
import {professionalSkills} from '../../../component/profile/skills-identification/professional-skills/professional-skills.component';

export let courses = new MenuItem('profile.outcome.courses.section.name',
  true, false, 'mooc-search', false, 'courses');

@Component({
  selector: 'mooc-courses-view',
  templateUrl: './courses-view.component.html',
  styleUrls: ['./courses-view.component.less']
})
export class CoursesViewComponent extends SectionAbstractComponent implements OnInit, OnDestroy {
  searchResults: ElasticSearchResult;

  searchFilters: SearchFilters;
  currentPage = 0;
  pageSize = 10;
  tempTopics = Topics;

  iscedKeys = IscedObj;
  iscedSearch = idToTopNode;

  tempEducation = EducationLevels;
  tempProviders = Providers;

  providers: any = jsonToIdTextElement(this.tempProviders);
  topics = jsonToIdTextElement(this.tempTopics);

  languages: IdTextElement[] = [];
  isceds = this.getCodeForEachIsced(jsonToIdTextElement(this.iscedSearch));
  educationEls = this.getCodeForEach(jsonToIdTextElement(this.tempEducation));

  eventLanguages: EventEmitter<any> = new EventEmitter();

  constructor(private searchService: ElasticSearchService,
              private languageService: LanguageService,
              public profileService: ProfileService,
              private httpErrorService: HttpErrorService) {
    super(profileService, professionalSkills);

    let filters = this.profileService.profile['outcome']['courses']['filters'];

    this.searchFilters = SearchFilters.builder()
      .addFilter(new MultiMatchFilter(['name^2', 'description'], filters['text'], 'text'))
      .addMultiTermQuery('topic', filters['topic'])
      .addMultiTermQuery('languages', filters['languages'])
      .addMultiTermQuery('subtitles', filters['subtitles'])
      .addFilter(new TermsFilterIsceds(null, 'isceds', filters['isceds']))
      .addMultiTermQuery('educationLevel', filters['educationLevel'])
      .addMultiTermQuery('provider', filters['provider']) // also orTermQuery
      .get();

    // for fill the drop down of available languages
    this.languages = this.languageService.supportedLanguages;
    if (this.languages === undefined) {
      this.languages = [];
      this.languageService.languageNotification.subscribe(() => this.initLanguages());
    } else {
      this.initLanguages();
    }
  }

  private initLanguages() {
    this.languageService
      .getLanguagesIn('en')
      .subscribe(response => {
        let result: IdTextElement[] = [];

        response['content'].forEach(resp => {
          let isoLang = resp['isoCode2'];

          let lang = '';
          for (let iso in resp['values']) {
            if (iso !== undefined) {
              // only one value
              lang = resp['values'][iso]['value'];
            }
          }

          result.push(
            {
              id: 'http://publications.europa.eu/resource/authority/language/' + isoLang.toUpperCase(),
              text: lang
            }
          );
        });
        this.languages = result;

        this.eventLanguages.emit(this.languages);
      });
  }

  search(page = 0) {
    this.currentPage = page;

    let filters = {
      text: this.searchFilters.getProperty('text'),
      topic: this.searchFilters.getProperty('topic'),
      languages: this.searchFilters.getProperty('languages'),
      subtitles: this.searchFilters.getProperty('subtitles'),
      isceds: this.searchFilters.getProperty('isceds'),
      educationLevel: this.searchFilters.getProperty('educationLevel'),
      provider: this.searchFilters.getProperty('provider')
    };

    this.profileService.profile['outcome']['courses']['filters'] = filters;
    this.searchService.searchWithFilters(this.searchFilters, page * this.pageSize, this.pageSize)
      .subscribe(
        n => {
          this.searchResults = n;
        },
        error => {
          this.httpErrorService.displayErrorMessage('042');
          this.searchResults = new ElasticSearchResult('{}');
        });
  }

  ngOnInit() {
    this.search();
  }

  changePage(p: number) {
    this.currentPage = p;
    this.search(p);
  }

  clearAll() {
    this.profileService.profile['outcome']['courses']['filters'] = {};
    this.searchFilters.clearAll();
    this.search();
  }

  activeFilterPresent(): boolean {
    return this.searchFilters.getFilters().some(filter => filter.isActive());
  }

  getCodeForEach(arr: IdTextElement[]) {
    return arr.map(n => {
      return {
        id: n['id'],
        text: `${UtilityFunctions.codeFromUriUpper(n.id)} - ${n.text}`
      };
    });
  }

  getCodeForEachIsced(arr: IdTextElement[]) {
    return arr.map(n => {
      return {
        id: n['id'],
        text: this.iscedKeys[n['id']]
      };
    });
  }

  ngOnDestroy(): void {
  }

  getSelectedCourses(): any[] {
    return this.profileService.profile['outcome']['courses']['selectedCourses'];
  }

}
