import {Component, OnInit} from '@angular/core';
import {ElasticSearchService} from '../../service/elastic-search.service';
import {ProfileService} from '../../../component/profile/profile.service';
import {IscedService} from '../../../service/data/isced.service';
import {Observable} from 'rxjs/Observable';
import {LanguageService} from '../../../service/data/language.service';
import {RestCallService} from '../../service/rest-call.service';
import {IdTextElement} from '../../model/id-text-element';
import {workTypes} from '../../../constants/profile-label-keys';
import {HttpErrorService} from '../../../service/error/http-error.service';
import {RecommendationCoursesService} from './service/recommendation-courses.service';
import {RecommendationLabelsService} from './service/recommendation-labels.service';
import {SectionAbstractComponent} from '../../../component/common/section-abstract.component';
import {MenuItem} from '../../../component/profile/menu/menu-item';
import {professionalSkills} from '../../../component/profile/skills-identification/professional-skills/professional-skills.component';

export let recommendation = new MenuItem('profile.outcome.recommendations.section.name',
  true, false, 'mooc-recommended', false, 'recommendation');

@Component({
  selector: 'app-recommended-courses',
  templateUrl: './recommended-courses.component.html',
  styleUrls: ['./recommended-courses.component.less']
})
export class RecommendedCoursesComponent extends SectionAbstractComponent implements OnInit {

  expectations: any[];

  languages: IdTextElement[];

  workType = workTypes;

  constructor(private searchService: ElasticSearchService,
              public profileService: ProfileService,
              private iscedService: IscedService,
              private languageService: LanguageService,
              private restCallService: RestCallService,
              private httpErrorService: HttpErrorService,
              private recommendationCoursesService: RecommendationCoursesService,
              private recommendationLabelsService: RecommendationLabelsService) {
    super(profileService, professionalSkills);

    this.languageService
      .getLanguagesIn('en')
      .subscribe(response => {
        let result: IdTextElement[] = [];

        response['content'].forEach(resp => {
          let isoLang = resp['isoCode2'];

          let lang = '';
          for (let iso in resp['values']) {
            if (iso !== undefined) {
              // only one value
              lang = resp['values'][iso]['value'];
            }
          }

          result.push(
            {
              id: 'http://publications.europa.eu/resource/authority/language/' + isoLang.toUpperCase(),
              text: lang
            }
          );
        });
        this.languages = result;
      });
  }

  ngOnInit() {
    super.ngOnInit();
    this.getLabelsAndRecommendations();
  }

  private getLabelsAndRecommendations() {
    const profileLocal = this.profileService.profile['expectations']
      .map(expectation => this.customizeExpectationsObject(expectation));

    const observables = this.observablesWithCustomizedObjects(profileLocal);

    if (observables.length === 0) {
      this.expectations = profileLocal;
      return;
    }

    Observable.zip(...observables)
      .subscribe(
        () => this.expectations = profileLocal,
        error => {
          this.expectations = [];
          this.httpErrorService.displayErrorMessage('042');
        });
  }


  private observablesWithCustomizedObjects(expectations: any[]): Observable<void>[] {
    const observables = [];

    expectations.forEach(exp => {
      this.recommendationLabelsService.getLabelsForExpectation(exp, observables);
      this.recommendationCoursesService.getCoursesForExpectation(exp, observables);
    });

    return observables;
  }

  /*adorn the expectations with custom properties*/
  customizeExpectationsObject(exp: any) {
    const mappedExp: any = {
      doNext: exp['doNext']
    };

    if (mappedExp.doNext === this.workType['undertakeEducation']) {
      mappedExp.undertakeEducationStudyField = exp['undertakeEducationStudyField'];
      mappedExp.otherUndertakeEducationStudyFields = exp['otherUndertakeEducationStudyFields'];
      mappedExp.educationType = exp['educationType'];
      mappedExp.timeCommitment = exp['undertakeEducationTimeCommitment'];
    }

    if (mappedExp.doNext === this.workType['followCourse']) {
    }

    if (mappedExp.doNext === this.workType['improveLanguage']) {
      mappedExp.language = exp['language'];
      mappedExp.otherImproveLanguage = exp['otherImproveLanguage'];
    }

    return mappedExp;
  }

  hasExpectationEnablingRecommandation(): boolean {
    const expectationWithoutRecommandation = this.expectations.filter(e => e.doNext === 'find_work' || e.doNext === undefined).length;
    return this.expectations.length - expectationWithoutRecommandation > 0;
  }

  getSelectedCourses(index: number): any[] {
    if (this.profileService.profile['outcome']['recommendations']['courseRecommendations'][index] === undefined) {
      this.profileService.profile['outcome']['recommendations']['courseRecommendations'].push({selectedCourses: []});
    }
    return this.profileService.profile['outcome']['recommendations']['courseRecommendations'][index]['selectedCourses'];
  }

  isExpectationForRecommendation(expectation): boolean {
    return expectation.doNext !== this.workType.findWork
      && expectation.doNext !== this.workType.other
      && expectation.doNext !== undefined;
  }

}
