import {isNumber} from 'util';
import {ElasticQueryJson} from './elastic-query-json';
import {DateRangeFilter} from './filter/date-range-filter';
import {SimpleTermAggregation} from './aggregation/simple-term-aggregation';
import {GlobalTermAggregation} from './aggregation/global-term-aggregation';
import {Filter} from './filter/filter';
import {TermFilter} from './filter/term-filter';
import {TermsFilter} from './filter/terms-filter';
import {MatchFilter} from './filter/match-filter';
import {RangeFilter} from './filter/range-filter';
import {MaxAggregation} from './aggregation/max-aggregation';
import {MinAggregation} from './aggregation/min-aggregation';
import {MatchFilterMust} from './filter/match-filter-must';
import {MultiTermFilter} from './filter/multi-term-filter';
import {AttributeModel} from '../../../model/attribute-model';

export class SearchFilters implements AttributeModel {

  private filters: Map<string, Filter>;
  private filterList: Filter[];

  static builder() {
    return new SearchSelectionBuilder();
  }

  constructor(filters: Map<string, Filter>) {
    this.filters = filters;
    this.filterList = [];
    filters.forEach(value => this.filterList.push(value));
  }

  getProperty(key: string) {
    return this.filters.get(key).getValue();
  }

  setProperty(key: string, value: any) {
    this.filters.get(key).setValue(value);
    this.filters.get(key).setActive(value != null && value.length > 0 || isNumber(value));
  }

  removeProperty(key: string) {
    const filter = this.filters.get(key);
    filter.setActive(false);
    filter.clearValue();
  }

  getObjectId(): string {
    return 'filter';
  }


  isActiveFilter(key: string) {
    return this.filters.get(key).isActive();
  }

  isPropertyObject(key: string) {
    return true;
  }

  clearAll() {
    if (this.getFilters().length > 0) {
      this.getFilters().forEach(n => {
        n.setActive(false);
        n.clearValue();
      });

    }

  }

  isPropertyArray(key: string) {
    return false;
  }

  getPropertyKeys(): string[] {
    return Array.from(this.filters.keys())
      .filter(key => this.filters.get(key).isActive());
  }

  getFilters(): Filter[] {
    return this.filterList;
  }

  toElasticQuery(withAggs = true): any {

    const query: ElasticQueryJson = {
      query: {
        bool: {
          filter: [],
          must: [],
          should: []
        }
      },
      aggs: {
        globalAggs: {
          global: {},
          aggs: {}
        }
      },
    };

    this.filters.forEach(filter => filter.addFilterToQuery(query));
    if (withAggs) {
      this.filters.forEach(filter => filter.addAggregationToQuery(query));

    }

    return query;
  }

  addToQueryParams(params: Object) {

    this.filters.forEach((value, key) => {
      if (value.isActive()) {
        params[key] = value.getValue();

      }
    });
  }


}

class SearchSelectionBuilder {

  private filters = new Map<string, Filter>();

  addTermQuery(key: string, value?: string): SearchSelectionBuilder {
    this.filters.set(key, new TermFilter(new SimpleTermAggregation(key), key, value));
    return this;
  }

  addTermQueryNoAgg(key: string, value?: string): SearchSelectionBuilder {
    this.filters.set(key, new TermFilter(null, key, value));
    return this;
  }

  addMultiTermQuery(key: string, value?: Array<any>) {
    let multiTermFilter = new MultiTermFilter(new SimpleTermAggregation(key), key, value ? value : []);
    multiTermFilter.setActive(value && value.length > 0);

    this.filters.set(key, multiTermFilter);
    return this;
  }

  addOrTermQuery(key: string, value?: string): SearchSelectionBuilder {
    this.filters.set(key, new TermFilter(new GlobalTermAggregation(key), key, value));
    return this;
  }

  addOrTermsQuery(key: string, value?: Array<any>) {
    this.filters.set(key, new TermsFilter(new GlobalTermAggregation(key), key, value ? value : []));
    return this;
  }

  addMatchQuery(key: string, value?: string) {
    this.filters.set(key, new MatchFilter(key, value));
    return this;
  }

  addMatchQueryMust(key: string, value?: string) {
    this.filters.set(key, new MatchFilterMust(key, value));
    return this;
  }

  addRangeQuery(filterKey: string, queryKey: string, rangeType: string, value?: string) {
    this.filters.set(filterKey, new RangeFilter(null, queryKey, filterKey, rangeType, value));
    return this;
  }

  addMaxRangeQuery(filterKey: string, queryKey: string, value?: string) {
    const agg = new MaxAggregation(queryKey, filterKey);
    this.filters.set(filterKey, new RangeFilter(agg, queryKey, filterKey, 'lte', value));
    return this;
  }

  addMinRangeQuery(filterKey: string, queryKey: string, value?: string) {
    const agg = new MinAggregation(queryKey, filterKey);
    this.filters.set(filterKey, new RangeFilter(agg, queryKey, filterKey, 'gte', value));
    return this;
  }

  addDateRangeQuery(filterKey: string, queryKey: string, rangeType: string, dateToString?: (date: Date) => string, value?: string) {
    this.filters.set(filterKey, new DateRangeFilter(null, queryKey, filterKey, rangeType, dateToString, value));
    return this;
  }

  addFilter(filter: Filter) {
    this.filters.set(filter.getFilterKey(), filter);
    return this;
  }

  get(): SearchFilters {
    return new SearchFilters(this.filters);
  }

  getFilters(): Filter[] {
    return Array.from(this.filters.values());
  }
}

