import {PipeTransform, Pipe, OnDestroy, ChangeDetectorRef} from '@angular/core';
import {TranslateService, LanguageChangeEvent} from '../service/translate/translate.service';
import {Subscription} from 'rxjs';
import {isDefined} from '@ng-bootstrap/ng-bootstrap/util/util';


@Pipe({
  name: 'translateConfiguration',
  pure: false // required to update the value when the promise is resolved
})
export class TranslateConfigurationPipe implements PipeTransform, OnDestroy {
  value = '';
  translateTarget: string;
  lastKey: string;
  lastLanguage: string;
  lastParams: any;

  onPrimaryLanguageChangeSub: Subscription;
  onSecondaryLanguageChangeSub: Subscription;

  constructor(private translateService: TranslateService) {
  }

  updateValue(key: string, language: string, interpolateParams?: Object): void {
    this.translateService
      .translate(key, language, interpolateParams)
      .subscribe(
        data => {
          if (data.indexOf('<br />') > 0) {
            let updatedData = data.split('<br />').join('');
            this.setValue(updatedData, key, language);
          } else {
            this.setValue(data, key, language);
          }
        },

        error => {
          this.translateService
            .translate(key, this.translateService.getDefaultLanguage(), interpolateParams)
            .subscribe(
              data => {
                this.setValue(data, key, language); }
            );
        }
      );
  }


  private replaceAll = function(search, replacement) {
    let target = this;
    return target.replace(new RegExp(search, 'g'), replacement);
  };

  private setValue(data: string, key: string, language: string) {
    this.value = data !== undefined ? data : key;
    this.lastKey = key;
    this.lastLanguage = language;
  }

  transform(key: string, target: string, ...args: any[]): any {
    if (!key || key.length === 0) {
      return key;
    }

    // if we ask another time for the same key, return the last value
    let language = this.translateService.getLanguageByTarget(target);
    if (this.lastKey === key && this.lastLanguage === language && this.equals(this.lastParams, args)) {
      return this.value;
    }

    let interpolateParams: Object;
    if (isDefined(args[0]) && args.length) {
      if (typeof args[0] === 'string' && args[0].length) {
        // we accept objects written in the template such as {n:1}, {'n':1}, {n:'v'}
        // which is why we might need to change it to real JSON objects such as {"n":1} or {"n":"v"}
        let validArgs: string = args[0]
          .replace(/(\')?([a-zA-Z0-9_]+)(\')?(\s)?:/g, '"$2":')
          .replace(/:(\s)?(\')(.*?)(\')/g, ':"$3"');
        try {
          interpolateParams = JSON.parse(validArgs);
        } catch (e) {
          throw new SyntaxError('Wrong parameter in TranslatePipe. Expected a valid Object, received: ${args[0]}');
        }
      } else if (typeof args[0] === 'object' && !Array.isArray(args[0])) {
        interpolateParams = args[0];
      }
    }

    // store the query, in case it changes
    this.lastKey = key;

    // store the query, in case it changes
    this.lastLanguage = language;

    // store the params, in case they change
    this.lastParams = args;

    // set the value
    this.updateValue(key, language, interpolateParams);

    // if there is a subscription to onLangChange, clean it
    this.dispose();

    // subscribe to onLangChange event, in case the language changes
    if (!this.onPrimaryLanguageChangeSub) {
      this.onPrimaryLanguageChangeSub = this.translateService.onPrimaryLanguageChange.subscribe((event: LanguageChangeEvent) => {
        if (this.lastKey && this.translateTarget === event.target) {
          this.lastKey = null; // we want to make sure it doesn't return the same value until it's been updated
          this.updateValue(key, event.language, interpolateParams);
        }
      });
    }

    if (!this.onSecondaryLanguageChangeSub) {
      this.onSecondaryLanguageChangeSub = this.translateService.onSecondaryLanguageChange.subscribe((event: LanguageChangeEvent) => {
        if (this.lastKey && this.translateTarget === event.target) {
          this.lastKey = null; // we want to make sure it doesn't return the same value until it's been updated
          this.updateValue(key, event.language, interpolateParams);
        }
      });
    }

    if (this.value.indexOf('<br />') > 0) {
      return this.value.replace('<br />', '');
    }

    return this.value;
  }

  /**
   * Clean any existing subscription to change events
   * @private
   */
  private dispose(): void {
    if (typeof this.onPrimaryLanguageChangeSub !== 'undefined') {
      this.onPrimaryLanguageChangeSub.unsubscribe();
      this.onPrimaryLanguageChangeSub = undefined;
    }

    if (typeof this.onSecondaryLanguageChangeSub !== 'undefined') {
      this.onSecondaryLanguageChangeSub.unsubscribe();
      this.onSecondaryLanguageChangeSub = undefined;
    }
  }

  private equals(o1: any, o2: any): boolean {
    if (o1 === o2) { return true; }
    if (o1 === null || o2 === null) { return false; }
    if (o1 !== o1 && o2 !== o2) { return true; } // NaN === NaN
    let t1 = typeof o1, t2 = typeof o2, length: number, key: any, keySet: any;
    if (t1 === t2 && t1 === 'object') {
      if (Array.isArray(o1)) {
        if (!Array.isArray(o2)) { return false; }
        if ((length = o1.length) === o2.length) {
          for (key = 0; key < length; key++) {
            if (!this.equals(o1[key], o2[key])) { return false; }
          }
          return true;
        }
      } else {
        if (Array.isArray(o2)) {
          return false;
        }
        keySet = Object.create(null);
        for (key in o1) {
          if (o1.hasOwnProperty(key)) {
            if (!this.equals(o1[key], o2[key])) {
              return false;
            }
            keySet[key] = true;
          }
        }
        for (key in o2) {
          if (!(key in keySet) && typeof o2[key] !== 'undefined') {
            return false;
          }
        }
        return true;
      }
    }
    return false;
  }

  ngOnDestroy(): void {
    this.dispose();
  }
}
