import {Injectable} from '@angular/core';
import {Observable} from 'rxjs/Observable';
import 'rxjs/add/observable/of';
import 'rxjs/add/operator/do';
import 'rxjs/add/operator/delay';
import {UserService} from '../data/user.service';
import {AuthStateService} from './auth-state.service';
import {Router, NavigationStart} from '@angular/router';
import {Http} from '@angular/http';
import {User} from '../data/user';

@Injectable()
export class AuthService {

  constructor(private userService: UserService,
              private authStateService: AuthStateService,
              private router: Router,
              private http: Http) {

    router.events.filter(event => event instanceof NavigationStart).subscribe(event => {
      this.userService.getCurrentUser()
        .catch(error => {
          if (error.status === 401) {
            this.authStateService.clear();
          }
          return Observable.empty();
        })
        .subscribe((user: User) => this.authStateService.setAuthenticatedUser(user));
    });

  }

  isAuthenticated(): boolean {
    return this.authStateService.authenticated;
  }

  getCurrentUser(): User {
    return this.authStateService.authenticatedUser;
  }

  logout() {

    let origin = window.location.protocol + '//' + window.location.hostname + (window.location.port ? (':' + window.location.port) : '');

    this.http.post('/logout', {redirectTo: origin}).subscribe(response => {
      this.authStateService.authenticated = false;
      window.location.href = response.json().ecasLogoutUrl;
    });
  }

  /**
   * Checks if the current user has one of the roles specified.
   * @param roles
   * @returns {Observable<R>}
   */
  canAccess(roles: Array<string>): Observable<boolean> {
    return this.userService.getCurrentUser()
      .map((user: User) => {
        this.authStateService.setAuthenticatedUser(user);

        if (user && user.roles && user.roles instanceof Array) {

          if (!roles || roles.length === 0) {
            return true; // there is no roles limiting the access, so allow access
          }

          for (let role of roles) {
            if (user.roles.indexOf(role) >= 0) {
              return true;
            }
          }
        }
        return false;
      });
  }

}
