import {Injectable}     from '@angular/core';
import {CanActivate, Router, ActivatedRoute, ActivatedRouteSnapshot, RouterStateSnapshot, CanActivateChild}    from '@angular/router';
import {AuthService} from './auth.service';
import {Observable} from 'rxjs/Observable';
import {Response} from '@angular/http';

@Injectable()
export class EcasAuthGuard implements CanActivate, CanActivateChild {

  constructor(private router: Router, private activatedRoute: ActivatedRoute, private authService: AuthService) {
  }

  /**
   * This implementation checks the roles defined on the route config against the roles of the currently logged in user.
   * At least one role should match.
   * @param route
   * @param state
   * @returns {Observable<boolean>}
   */
  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot) {

    let canAccess = this.authService.canAccess(this.getRoles(route));

    canAccess
      .filter(response => response === false)
      .catch(error => this.handleError(error, state))
      .subscribe(response => {
        this.router.navigate(['/access-denied']);
      });

    return canAccess;
  }

  canActivateChild(childRoute: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean>|Promise<boolean>|boolean {
    return this.canActivate(childRoute, state);
  }

  /**
   * Recursively checks activated routes to find the first existing roles array.
   * The recursion starts with the latest activated route and processes the routes up until the first activated route,
   * i.e. from child to parent.
   * @param route the activated route snapshot
   * @returns {any} the array of roles
   */
  private getRoles(route: ActivatedRouteSnapshot): Array<string> {
    if (route.data['roles'] && route.data['roles'] instanceof Array) {
      return route.data['roles'];
    } else if (route.parent) {
      return this.getRoles(route.parent);
    } else {
      return [];
    }
  }

  private handleError(error: Response | any, state: RouterStateSnapshot): Observable<boolean> {

    if (error instanceof Response) {
      if ((<Response> error).status === 401) {
        window.location.href = '/login?route=' + state.url;
      } else if ((<Response> error).status === 403) {
        this.router.navigate(['/access-denied']);
      }
    }

    return Observable.throw(error);
  }

}
