import {EventEmitter, Injectable, Optional} from '@angular/core';
import {Http, Response} from '@angular/http';
import {Observable} from 'rxjs/Observable';
import 'rxjs/add/observable/of';
import 'rxjs/add/operator/share';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/merge';
import 'rxjs/add/operator/toArray';
import 'rxjs/add/operator/take';
import {TranslateParser} from './translate.parser';
import {isDefined} from '@ng-bootstrap/ng-bootstrap/util/util';
import {LocalStorageService} from '../storage/local-storage.service';

export interface LanguageChangeEvent {
  language: string;
  target: string;
}

export interface MissingTranslationHandlerParams {
  /**
   * the key that's missing in translation files
   *
   * @type {string}
   */
  key: string;

  /**
   * an instance of the service that was unable to translate the key.
   *
   * @type {TranslateService}
   */
  translateService: TranslateService;

  /**
   * interpolation params that were passed along for translating the given key.
   *
   * @type {Object}
   */
  interpolateParams?: Object;
}

declare interface Window {
  navigator: any;
}
declare const window: Window;

export abstract class MissingTranslationHandler {
  /**
   * A function that handles missing translations.
   *
   * @abstract
   * @param {MissingTranslationHandlerParams} params context for resolving a missing translation
   * @returns {any} a value or an observable
   * If it returns a value, then this value is used.
   * If it return an observable, the value returned by this observable will be used (except if the method was "instant").
   * If it doesn't return then the key will be used as a value
   */
  abstract handle(params: MissingTranslationHandlerParams): any;
}

export abstract class TranslateLoader {
  abstract getTranslation(lang: string): Observable<any>;
}

export class TranslateStaticLoader implements TranslateLoader {

  constructor(private http: Http, private prefix = 'i18n', private suffix = '.json') { }

  /**
   * Gets the translations from the server
   * @param lang
   * @returns {any}
   */
  public getTranslation(lang: string): Observable<any> {
    return this.http.get(`${this.prefix}/${lang}${this.suffix}`).map((res: Response) => res.json());
  }
}

@Injectable()
export class TranslateService {

  /**
   * The default language used as a fallback when a translation isn't found in a language.
   */
  private defaultLang: string;

  /**
   * The primary language. (i.e. used for the website interface)
   */
  primaryLanguage: string;

  /**
   * The secondary language. (i.e. used for help label interface)
   */
  secondaryLanguage: string;

  /**
   * An EventEmitter to listen to primary language change events
   * onPrimaryLanguageChange.subscribe((params: LanguageChangeEvent) => {
     *     // do something
     * });
   * @type {EventEmitter<LanguageChangeEvent>}
   */
  public onPrimaryLanguageChange: EventEmitter<LanguageChangeEvent> = new EventEmitter<LanguageChangeEvent>();

  /**
   * An EventEmitter to listen to secondary language change events
   * onSecondaryLanguageChange.subscribe((params: LanguageChangeEvent) => {
     *     // do something
     * });
   * @type {EventEmitter<LanguageChangeEvent>}
   */
  public onSecondaryLanguageChange: EventEmitter<LanguageChangeEvent> = new EventEmitter<LanguageChangeEvent>();

  private pending: any = {};
  private translations: any = {};

  /**
   *
   * @param currentLoader An instance of the loader currently used
   * @param parser An instance of the parser currently used
   * @param missingTranslationHandler A handler for missing translations.
   */
  constructor(public currentLoader: TranslateLoader,
              public parser: TranslateParser,
              private localStorage: LocalStorageService,
              @Optional() private missingTranslationHandler: MissingTranslationHandler) { }

  /**
   * Sets the default language to use as a fallback
   * @param lang
   */
  public setDefaultLanguage(language: string): void {
    if (language === this.defaultLang) {
      return;
    }

    this.changeDefaultLanguage(language);

    let pending: Observable<any> = this.currentLoader.getTranslation(language);
    if (typeof pending !== 'undefined') {
      pending.take(1)
        .subscribe(
          data => {
            this.translations[language] = data;
          }
      );
    }
  }

  public getDefaultLanguage() {
    return this.defaultLang;
  }

  /**
   * Initialize  the primary language
   * @param language
   * @returns {Observable<*>}
   */
  public initAsPrimary(language: string): Observable<any> {
    if (language === this.primaryLanguage) {
      return;
    }

    let _t = this;
    this.currentLoader.getTranslation(language)
     .subscribe(function (res) {
        _t.translations[language] = res;
        _t.changePrimaryLanguage(language);
    }, function (err) {
       _t.changePrimaryLanguage(_t.getDefaultLanguage());
    });
  }

  /**
   * Changes the primary language currently used
   * @param language
   * @returns {Observable<*>}
   */
  public useAsPrimary(language: string): Observable<any> {
    if (language === this.primaryLanguage) {
      return;
    }
    this.changePrimaryLanguage(language);
  }

  public getPrimaryLanguage() {
    return this.primaryLanguage || this.localStorage.get('primaryLanguage') || this.getBrowserLang() || this.defaultLang;
  }

  /**
   * Initialize  the secondary language
   * @param language
   * @returns {Observable<*>}
   */
  public initAsSecondary(language: string): Observable<any> {
    if (language === this.secondaryLanguage) {
      return;
    }

    let _t = this;
    this.currentLoader.getTranslation(language)
      .subscribe(function (res) {
        _t.translations[language] = res;
        _t.changeSecondaryLanguage(language);
    }, function (err) {
        _t.changeSecondaryLanguage(_t.getDefaultLanguage());
    });
  }

  /**
   * Changes the secondary language currently used
   * @param language
   * @returns {Observable<*>}
   */
  public useAsSecondary(language: string): Observable<any> {
    if (language === this.secondaryLanguage) {
      return;
    }
    this.changeSecondaryLanguage(language);
  }

  public getSecondaryLanguage() {
    return this.secondaryLanguage || this.localStorage.get('secondaryLanguage') || this.getBrowserLang() || this.defaultLang;
  }

  /**
   * Get the language based on the target scope.
   *
   * @param target the targeted language scope
   * @returns {string}
   */
  public getLanguageByTarget(target: string) {
    return target === 'secondary' ? this.getSecondaryLanguage() : this.getPrimaryLanguage();
  }

  /**
   * Changes the default lang
   * @param lang
   */
  private changeDefaultLanguage(language: string): void {
    this.defaultLang = language;
  }

  /**
   * Changes the current lang
   * @param lang
   */
  private changePrimaryLanguage(language: string): void {
    this.primaryLanguage = language;
    this.onPrimaryLanguageChange.emit({language: language, target: 'primary'});
    this.localStorage.save('primaryLanguage', language);
  }

  /**
   * Changes the current lang
   * @param lang
   */
  private changeSecondaryLanguage(language: string): void {
    this.secondaryLanguage = language;
    this.onSecondaryLanguageChange.emit({language: language, target: 'secondary'});
    this.localStorage.save('secondaryLanguage', language);
  }

  /**
   * Translate the given translation i18nKey in the given language.
   *
   * @param key the translation i18nKey
   * @param language the target translation language
   * @param params the trasnlation parameters, if any
   * @returns {Observable<*>}
   */
  public translate(key: string, language: string, params?: Object): Observable<any> {

    language = language || this.defaultLang;

    if (!this.translations[language] && !this.pending[language]) {
      this.pending[language] = this.currentLoader.getTranslation(language).share();
    }

    // check if we are loading a new translation to use
    if (this.pending[language]) {
      let _t = this;

      return Observable.create(function (observer) {
        let onComplete = function (res) {
          observer.next(res);
          observer.complete();
          _t.pending[language] = null;
        };

        let onError = function (err) {
          observer.error(err);
        };

        _t.pending[language].subscribe(function (res) {
          _t.translations[language] = res;
          res = _t.getParsedResult(res, key, language, params);
          if (typeof res.subscribe === 'function') {
            res.subscribe(onComplete, onError);
          } else {
            onComplete(res);
          }
        }, onError);

      });
    } else {
      let res = this.getParsedResult(this.translations[language], key, language, params);
      if (typeof res.subscribe === 'function') {
        return res;
      } else {
        return Observable.of(res);
      }
    }
  }

  public instant(key: string, language: string, params?: Object): string|any {
    if (!isDefined(key) || !key.length) {
      throw new Error('Parameter "key" required');
    }

    language = this.translations[language] ? language : this.defaultLang;
    let data = this.getParsedResult(this.translations[language], key, language, params);

    return data ? data : key;
  }

  /**
   * Returns the parsed result of the translations
   * @param translations
   * @param key
   * @param interpolateParams
   * @returns {any}
   */
  private getParsedResult(translations: any, key: any, language: string, interpolateParams?: Object): any {
    let res: string|Observable<string>;

    if (key instanceof Array) {
      let result: any = {}, observables = false;
      for (let k of key) {
        result[k] = this.getParsedResult(translations, k, language, interpolateParams);
        if (typeof result[k].subscribe === 'function') {
          observables = true;
        }
      }
      if (observables) {
        let mergedObs: any;
        for (let k of key) {
          let obs = typeof result[k].subscribe === 'function' ? result[k] : Observable.of(result[k]);
          if (typeof mergedObs === 'undefined') {
            mergedObs = obs;
          } else {
            mergedObs = mergedObs.merge(obs);
          }
        }
        return mergedObs.toArray().map((arr: Array<string>) => {
          let obj: any = {};
          arr.forEach((value: string, index: number) => {
            obj[key[index]] = value;
          });
          return obj;
        });
      }
      return result;
    }

    if (translations) {
      res = this.parser.interpolate(this.parser.getValue(translations, key), interpolateParams);
    }

    if (typeof res === 'undefined' && this.defaultLang && this.defaultLang !== language) {
      res = this.parser.interpolate(this.parser.getValue(this.translations[this.defaultLang], key), interpolateParams);
    }

    if (!res && this.missingTranslationHandler) {
      let params: MissingTranslationHandlerParams = {key, translateService: this};
      if (typeof interpolateParams !== 'undefined') {
        params.interpolateParams = interpolateParams;
      }
      res = this.missingTranslationHandler.handle(params);
    }

    return typeof res !== 'undefined' ? res : key;
  }

  /**
   * Returns the language code name from the browser, e.g. 'de'
   *
   * @returns string
   */
  public getBrowserLang(): string {
    if (typeof window === 'undefined' || typeof window.navigator === 'undefined') {
      return undefined;
    }

    let browserLang: any = window.navigator.languages ? window.navigator.languages[0] : null;
    browserLang = (browserLang || window.navigator.language || window.navigator.browserLanguage
      || window.navigator.userLanguage);

    if (browserLang.indexOf('-') !== -1) {
      browserLang = browserLang.split('-')[0];
    }

    if (browserLang.indexOf('_') !== -1) {
      browserLang = browserLang.split('_')[0];
    }

    return browserLang;
  }

  /**
   * Returns the culture language code name from the browser, e.g. 'de-DE'
   *
   * @returns string
   */
  public getBrowserCultureLang(): string {
    if (typeof window === 'undefined' || typeof window.navigator === 'undefined') {
      return undefined;
    }

    let browserCultureLang: any = window.navigator.languages ? window.navigator.languages[0] : null;
    browserCultureLang = (browserCultureLang || window.navigator.language || window.navigator.browserLanguage
      || window.navigator.userLanguage);

    return browserCultureLang;
  }

}
