import {Injectable, OnInit, AfterViewChecked} from '@angular/core';
import {FormGroup, FormBuilder, Validators, FormControl, FormArray} from '@angular/forms';

/**
 * Allows us to control some input fields.
 */
@Injectable()
export class ValidationService {

  public formGroup: FormGroup;

  public mailRegex = /^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/;
  public phoneNumberRegex = /^[0-9+\/ ]+$/;

  public emailPrefixValidator = Validators.pattern('^[+a-zA-Z0-9._%+-]+$');
  public emailSuffixValidator = Validators.pattern('^[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,}$');

  constructor(private formBuilder: FormBuilder) {
    this.formGroup = this.formBuilder.group({
      familyName: [, [Validators.required]],
      firstName: [, [Validators.required]],
      mail: [, [Validators.pattern(this.mailRegex)]],
      phoneNumber: [, [Validators.pattern(this.phoneNumberRegex)]],
      statistics: [, [Validators.required]],
      stored: [, [Validators.required]],
      username: [, [Validators.required]],
      phonePrefix: [, [Validators.pattern('^[0-9+\/ ]+$')]],
      phoneSuffix: [, [Validators.pattern('^[0-9+\/ ]+$')]],
      emailPrefix: [, [this.emailPrefixValidator]],
      emailSuffix: [, [this.emailSuffixValidator]],
      subEmails: new FormArray([]),
      organisationPhonePrefix: [, [Validators.pattern('^[0-9+\/ ]+$')]],
      organisationPhoneSuffix: [, [Validators.pattern('^[0-9+\/ ]+$')]],
      organisationEmailPrefix: [, [Validators.pattern('^[+a-zA-Z0-9._%+-]+$')]],
      organisationEmailSuffix: [, [Validators.pattern('^[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$')]],
    });
  }

  /**
   * A control is marked `touched` once the user has triggered
   * a `blur` event on it.
   */
  public isTouched() {
    return this.formGroup.touched;
  }

  /**
   * Enable/disable the button according to the form validity
   * @returns True if the form is valid, false if the form is not valid.
   */
  public canSave(profile: Object) {
    return this.namesValid(profile);
  }

  /**
   * Allows us to display a warning if some required fields are not valid
   * @returns True if the family name field and the first name field are valid, if not false
   */
  private namesValid(profile: Object) {
    let namesInvalid = this.formGroup.controls['familyName'].invalid || this.formGroup.controls['firstName'].invalid;
    let namesEmpty = profile['personalInfo']['familyName'] === undefined || profile['personalInfo']['firstName'] === undefined
      || profile['personalInfo']['familyName'] === '' || profile['personalInfo']['firstName'] === '';
    return !(namesInvalid || namesEmpty);
  }

  public validate(profile: Object, menuName: string): boolean {
    if (menuName === 'personalInfo') {
      return this.namesValid(profile);
    }
    return true;
  }

  /**
   * Reset all the form validations
   */
  public resetFormGroup() {
    this.formGroup.reset();
  }

  /**
   * Enable/disable the button according to the form validity
   * @returns True if the form is valid, false if the form is not valid.
   */
  public canCreateUser() {
    return this.formGroup.controls['username'].valid;
  }

  /**
   * Add validators for phone number prefix.
   *
   * @param {string} controlName
   */
  public addPhoneControl(controlName: string) {
    if (!this.formGroup.contains(controlName)) {
      this.formGroup.addControl(controlName, new FormControl('', Validators.pattern('^[0-9+\/ ]+$')));
    }
  }

  /**
   * Add validators for mail prefix.
   *
   * @param {string} controlName
   */
  public addMailPrefixControl(controlName: string) {
    if (!this.formGroup.contains(controlName)) {
      this.formGroup.addControl(controlName, new FormControl('', Validators.pattern('^[+a-zA-Z0-9._%+-]+$')));
    }
  }

  /**
   * Add validators for mail suffix.
   *
   * @param {string} controlName
   */
  public addMailSuffixControl(controlName: string) {
    if (!this.formGroup.contains(controlName)) {
      this.formGroup.addControl(controlName, new FormControl('', Validators.pattern('^[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$')));
    }
  }
}
